# Script version: 4-6-0
# Script date: January 2022
#
# the following actions are performed by this script:
#
# 	- add two provider nodes and corresponding Microsoft Teams tenant nodes to anynode
# 	- add two routing domains that each contain a direct routing connection between 
#        each provider node and their Microsoft Teams tenant node
# 	- the first Microsoft Teams tenant node will create a Microsoft Teams connectivity,
#	 which will be used by both tenants
#
# to use this script, there are some variables to be defined
#
#	The path to the REST API PowerShell Module needs to be set.
#	- By default, the module is located a folder above the one in which this script is located.
#	The address of anynode has to be defined.
#
# Please make sure to review all possible parameters before
# starting the script.

# Set execution environment
Set-ExecutionPolicy -ExecutionPolicy Unrestricted -Scope Process

# The REST API PowerShell Module's path must be given here. In this case, the file is in the directory above the current one.
Import-Module "./../anynode_rest_api_powershell.psm1"

# Define anynode's Address and Port.
$AnynodeAddress='127.0.0.1'
$AnynodePort=443

# If you have not established a secure connection (HTTPS) to anynode,
# you can use HTTP instead.
# This is not recommended, since your data will NOT be encrypted!
# To activate it anyway, uncomment the following line:
# Disable-CertificateCheck

# Authentication
$Cred=Get-Credential

# net4home node properties
$ProviderNode1Name='net4home Provider'
$ProviderNode1IpAddress='172.16.2.82'
$ProviderNode1UdpTcpPort=5060
$ProviderNode1TlsPort=5061
$ProviderNode1RemoteSipDomain=(
	[pscustomobject]@{
		iri="sip:provider.anynodesbc.com"
	}
)
$ProviderNode1ClientCredentials=(
	[pscustomobject]@{
		username="user"
		password="user password"
	}
)
$ProviderNode1RegistrarUri="sip:provider.anynodesbc.com"
$ProviderNode1AddressOfRecordUser="0123456789"
$ProviderNode1ProxyUri="sip:provider.anynodesbc.com"
$ProviderNode1DefineAssertedUri=$false
# $ProviderNode1AssertedUri=""
$ProviderNode1

$MsTeamsNode1Name='net4home Microsoft Teams'
$MsTeamsNode1Fqdn='net4home'
$MsTeamsNode1

# TE-SYSTEMS node properties
$ProviderNode2Name='TE-SYSTEMS Provider'
$ProviderNode2IpAddress='172.16.2.82'
$ProviderNode2UdpTcpPort=5062
$ProviderNode2TlsPort=5063
$ProviderNode2RemoteSipDomain=(
	[pscustomobject]@{
		iri="sip:provider.anynodesbc.com"
	}
)
$ProviderNode2ClientCredentials=(
	[pscustomobject]@{
		username="user"
		password="user password"
	}
)
$ProviderNode2RegistrarUri="sip:provider.anynodesbc.com"
$ProviderNode2AddressOfRecordUser="0123456789"
$ProviderNode2ProxyUri="sip:provider.anynodesbc.com"
$ProviderNode2DefineAssertedUri=$false
# $ProviderNode2AssertedUri=""
$ProviderNode2

$MsTeamsNode2Name='TE-SYSTEMS Microsoft Teams'
$MsTeamsNode2Fqdn='te-systems'
$MsTeamsNode2
echo "`n"

# General provider properties
$ProviderIncomingDialStringRewriteRules=(
	[pscustomobject]@{
		rewriteType="prefixAndSuffix"
		prefix="601"
		deleteLeadingCharacters=3
		addPrefix="john"
		applyToSourceDialString=$true
		applyToDestinationDialString=$true
		processingDone=$true
	}, [pscustomobject]@{
		rewriteType="prefixAndSuffix"
		prefix="00"
		deleteLeadingCharacters=2
		addPrefix="+"
		applyToSourceDialString=$true
		applyToDestinationDialString=$true
		processingDone=$true
	}
)

$ProviderProfileName='*Other*'
#$ProviderProfile

# Microsoft Teams connectivity properties
$MsTeamsSbcFqdn='sbc1.customer.anynodesbc.com'
$MsTeamsIpAddress='172.18.1.55'
$MsTeamsUdpTcpPort=5060
$MsTeamsTlsPort=5061

$MsTeamsIncomingDialStringRewriteRules=(
	[pscustomobject]@{
		rewriteType="prefixAndSuffix"
		prefix="+555"
		deleteLeadingCharacters=4
		addPrefix="+49"
		applyToSourceDialString=$true
		applyToDestinationDialString=$true
		processingDone=$true
	}
)

# Properties for net4home routing domain
$RoutingDomain1Name='net4home Routing Domain'
$SourceNodes1 =@($ProviderNode1Name, $MsTeamsNode1Name) 

# Properties for TE-SYSTEMS routing domain
$RoutingDomain2Name='TE-SYSTEMS Routing Domain'
$SourceNodes2 =@($ProviderNode2Name, $MsTeamsNode2Name)

# ----------- Start of script -----------

$Output=''

#Every action is contained in a try-catch-block, so possible errors will be detectable.
try {
	# Retrieve profile IDs.
	$ProviderProfile=(Get-anynodeProviderNodeProfiles -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred | Where-Object -Property 'displayName' -Like $ProviderProfileName).id

	# Create the first provider node.
	$ProviderNode1=Add-anynodeProviderNodeV2 -Address $AnynodeAddress -Port $AnynodePort -Profile $ProviderProfile -NetworkControllerIpAddress $ProviderNode1IpAddress -UdpTcpPort $ProviderNode1UdpTcpPort -TlsPort $ProviderNode1TlsPort -ClientCredentials $ProviderNode1ClientCredentials -RemoteSipDomain $ProviderNode1RemoteSipDomain -RegistrarUri $ProviderNode1RegistrarUri -AddressOfRecord $ProviderNode1AddressOfRecordUser -ProxyUri $ProviderNode1ProxyUri -DefineAssertedUri $ProviderNode1DefineAssertedUri -IncomingDialStringRewrites $ProviderIncomingDialStringRewriteRules -DisplayName $ProviderNode1Name -Credential $Cred
	$Output+="$ProviderNode1Name :`n"
	$Output+="$ProviderNode1`n`n"

	# Create the first Microsoft Teams tenant together with its connectivity.
	$MsTeamsNode1=Add-anynodeMsTeamsTenantNodeV2 -Address $AnynodeAddress -Port $AnynodePort -CreateNewTeamsCarrierNode True -NetworkControllerIpAddress $MsTeamsIpAddress -UdpTcpPort $MsTeamsUdpTcpPort -TlsPort $MsTeamsTlsPort -SbcFqdn $MsTeamsSbcFqdn -TenantFqdn $MsTeamsNode1Fqdn -DisplayName $MsTeamsNode1Name -IncomingDialStringRewrites $MsTeamsIncomingDialStringRewriteRules -Credential $Cred
	$Output+="$MsTeamsNode1Name :`n"
	$Output+="$MsTeamsNode1`n`n"

	# Create the second provider node.
	$ProviderNode2=Add-anynodeProviderNodeV2 -Address $AnynodeAddress -Port $AnynodePort -Profile $ProviderProfile -NetworkControllerIpAddress $ProviderNode2IpAddress -UdpTcpPort $ProviderNode2UdpTcpPort -TlsPort $ProviderNode2TlsPort -ClientCredentials $ProviderNode2ClientCredentials -RemoteSipDomain $ProviderNode2RemoteSipDomain -RegistrarUri $ProviderNode2RegistrarUri -AddressOfRecord $ProviderNode2AddressOfRecordUser -ProxyUri $ProviderNode2ProxyUri -DefineAssertedUri $ProviderNode2DefineAssertedUri -IncomingDialStringRewrites $ProviderIncomingDialStringRewriteRules -DisplayName $ProviderNode2Name -Credential $Cred
	$Output+="$ProviderNode2Name :`n"
	$Output+="$ProviderNode2`n`n"

	# Create the second Microsoft Teams tenant and use the same connectivity as the first tenant.
	$MsTeamsNode2=Add-anynodeMsTeamsTenantNodeV2 -Address $AnynodeAddress -Port $AnynodePort -CreateNewTeamsCarrierNode False -TeamsCarrierNodeId $MsTeamsNode1.id -TenantFqdn $MsTeamsNode2Fqdn -DisplayName $MsTeamsNode2Name -Credential $Cred
	$Output+="$MsTeamsNode2Name :`n"
	$Output+="$MsTeamsNode2`n`n`n"

    # Create a routing domain for the created TE-SYSTEMS nodes.
    $RoutingDomain1=(Add-anynodeRoutingDomain -Address $AnynodeAddress -Port $AnynodePort -DisplayName $RoutingDomain1Name -SourceNodes $SourceNodes1 -Credential $Cred).id
    $Output+="$RoutingDomain1Name for net4home nodes:"
    $Output+="$RoutingDomain`n`n"

    # Create a routing domain for the created net4home nodes.
    $RoutingDomain2=(Add-anynodeRoutingDomain -Address $AnynodeAddress -Port $AnynodePort -DisplayName $RoutingDomain2Name -SourceNodes $SourceNodes2 -Credential $Cred).id
    $Output+="$RoutingDomain2Name for TE-SYSTEMS nodes:`n"
    $Output+="$RoutingDomain`n`n"

    # Create route from TE-SYSTEMS provider node to TE-SYSTEMS MS Teams tenant node.
    $Route1Filter=[pscustomobject]@{sourceNodeIds=@($MsTeamsNode1.id)}
    $Route1FilterList=@($Route1Filter)    

    $Route1 =Add-anynodeEstablishmentRoute -Address $AnynodeAddress -Port $AnynodePort -RoutingDomain $RoutingDomain1 -UseUnconditionalRouting False -Filters $Route1FilterList  -DestinationNodeId $ProviderNode1.id -RoutingForwardProfileSelection NONE -DisplayName "To net4home Provider node"
    $Output+="Route from $ProviderNode1Name to $MsTeamsNode1Name :"
    $Output+="$Route1`n`n"

    # Create route from TE-SYSTEMS MS Teams tenant node to TE-SYSTEMS provider node.
    $Route2Filter=[pscustomobject]@{sourceNodeIds=@($ProviderNode1.id)}
    $Route2FilterList=@($Route2Filter)
   
    $Route2 =Add-anynodeEstablishmentRoute -Address $AnynodeAddress -Port $AnynodePort -RoutingDomain $RoutingDomain1 -UseUnconditionalRouting False -Filters $Route2FilterList -DestinationNodeId $MsTeamsNode1.id -RoutingForwardProfileSelection NONE -DisplayName "To net4home MS Teams Tenant node"
    $Output+="Route from $MsTeamsNode1Name to $ProviderNode1Name :"
    $Output+="$Route2`n`n"

    # Create Route from net4home provider node to net4home MS Teams tenant node.
    $Route3Filter=[pscustomobject]@{sourceNodeIds=@($MsTeamsNode2.id)}
    $Route3FilterList=@($Route3Filter)
    
    $Route3 =Add-anynodeEstablishmentRoute -Address $AnynodeAddress -Port $AnynodePort -RoutingDomain $RoutingDomain2 -UseUnconditionalRouting False -Filters $Route3FilterList -DestinationNodeId $ProviderNode2.id -RoutingForwardProfileSelection NONE -DisplayName "To TE-SYSTEMs Provider node"
    $Output+="Route from $ProviderNode2Name to $MsTeamsNode2Name :"
    $Output+="$Route3`n`n"

    # Create routes from net4home MS Teams tenant node to net4home provider node
    $Route4Filter=[pscustomobject]@{sourceNodeIds=@($ProviderNode2.id)}
    $Route4FilterList=@($Route4Filter)
    
    $Route4 =Add-anynodeEstablishmentRoute -Address $AnynodeAddress -Port $AnynodePort -RoutingDomain $RoutingDomain2 -UseUnconditionalRouting False -Filters $Route4FilterList -DestinationNodeId $MsTeamsNode2.id -RoutingForwardProfileSelection NONE -DisplayName "To TE-SYSTEMS MS Teams Tenant node"
    $Output+="Route from $MsTeamsNode2Name to $ProviderNode2Name :"
    $Output+="$Route4`n`n"

	# Commit configuration changes.
	$Output+=Push-anynodeConfig -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred
} catch {
	$_.Exception
} finally {
	echo $Output
}

pause
