# Script version: 4-6-0
# Script date: January 2022
#
# the following actions are performed by this script:
#
#	- get all provider profiles from anynode and look for a match of the chosen display name
#	- filter the requested provider nodes for the requested profile
#	- add the provider node to anynode
#
# to use this script, there are some variables to be defined
#
#	The path to the REST API PowerShell Module needs to be set.
#		- By default, the module is located a folder above the one in which this script is located.
#	The address of anynode has to be defined.
#
# Please make sure to review all possible parameters before
# starting the script.

# Set execution environment
Set-ExecutionPolicy -ExecutionPolicy Unrestricted -Scope Process

# The REST API PowerShell Module's path must be given here. In this case, the file is in the directory above the current one.
Import-Module "./../anynode_rest_api_powershell.psm1"

# Define anynode's Address and Port.
$AnynodeAddress='127.0.0.1'
$AnynodePort=443

# If you have not established a secure connection (HTTPS) to anynode,
# you can use HTTP instead.
# This is not recommended, since your data will NOT be encrypted!
# To activate it anyway, uncomment the following line:
# Disable-CertificateCheck

# Authentication
$Cred=Get-Credential

$ProviderProfile

# The display name of the provider template to be used.
$ProviderProfileDisplayName="babyTel"

# Necessary variable for "babyTel"-Template.
$ClientCredentials=[PSCustomObject]@{username="user";password="password"} 

# ----------- Start of script -----------

try {
	# Get all provider profiles from anynode and look for a match of the comment.
	echo "Requesting profiles that match `"$ProviderProfileDisplayName`"..."
	$ProviderProfileRequest=Get-anynodeProviderNodeProfiles -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred

	if ($ProviderProfileRequest -EQ $null) {
		echo "Could not connect to server at $AnynodeAddress`:$AnynodePort or entered invalid credentials."

		return
	}

	if (!($ProviderProfileRequest.GetType().Name -EQ "Object[]")) {
		# Something must have gone wrong in this case. Display error message.
		echo $ProviderProfileRequest

		return
	}

	# Filter for the requested profile
	$ProviderProfile=$ProviderProfileRequest | Where-Object -Property "displayName" -Match $ProviderProfileDisplayName

	if (($ProviderProfile -EQ $null) -OR ($ProviderProfile -EQ '')) {
		echo "No profiles found with requested display names."

		return
	}

	if ($ProviderProfile.GetType().Name -EQ "Object[]") {
		echo "`nProfile display name `"$ProviderProfileDisplayName`" is ambiguous. Please define it more clearly. Found:"

		# Show all profiles that have been found.
		foreach ($Profile in $ProviderProfile) {
			echo $Profile.displayName
		}

		return
	}

	echo "Match found.`n"

	# Finally, add the provider node with ONLY necessary variables.
	echo "`nRequesting to add node..."
	$Result=Add-anynodeProviderNodeV2 -Address $AnynodeAddress -Port $AnynodePort -Profile $ProviderProfile.id -ActivateClientAuthentication True -ClientCredentials $ClientCredentials -Credential $Cred

	# Check whether the node has been added or not.
	if (!($Result.id -EQ $null)) {
		# Display the added node's information.
		echo "Successfully added node:"
		echo $Result
	} else {
		# This will display the error received by anynode.
		echo $Result
	}

	# Commit configuration changes.
	Push-anynodeConfig -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred

} catch {
	$_.Exception

} finally {
	# This Job prevents the script from returning before everything has been displayed properly.
	Start-Job -Name ' ' -ScriptBlock { Start-Sleep -s 2 } > $null
	(Wait-Job ' ').Name
	pause
}
