# Script version: 4-6-0
# Script date: January 2022
#
# the following actions are performed by this script:
#
#	- download all recorded sessions from a chosen call direction and time range
#	- create a folder structure for each session
#		The structure will hold a text file that contains various information about the session.
#
# to use this script, there are some variables to be defined
#
#	The address of anynode have to be defined.
#	Define which call direction recordings should be used.
#	Define which type of recordings should be downloaded.
#	Define the Connection Protocol to be used.
#
# Please make sure to review all possible parameters before
# starting the script.

# Set execution environment
Set-ExecutionPolicy -ExecutionPolicy Unrestricted -Scope Process

# Define Address to reach anynode.
$AnynodeAddress="127.0.0.1:443"

# Request user credentials from a user with permission to download media recordings.
$Cred=Get-Credential

# Encode user credentials to log in to anynode.
$Username=$Cred.UserName
$Password=$Cred.GetNetworkCredential().Password

$Credentials=$Username+":"+$Password
$Base64=[Convert]::ToBase64String([Text.Encoding]::ASCII.GetBytes($Credentials))

# Set the authorization header to gain access to anynode.
$Headers=@{Authorization="Basic "+$Base64}

# Set the content-type header to use the URL as a POST body.
$Headers.Add("Content-Type","application/x-www-form-urlencoded")

# Which call direction recordings should be used? "incoming" or "outgoing"
$CallDirection="outgoing"

# Which type of recordings should be downloaded? "wav" or "g711"
$RecordingType="wav"

# Connection Protocol (https or http)
$Protocol="https"

# Insert a template to retrieve data from today (filterDateTimePeriod=TODAY).
$FilterDateTimePeriod="TODAY"
$SessionsUrl=$Protocol+"://"+$AnynodeAddress+"/app/monitor/callhistory/export/v0?filterCount=2147483647&filterDateTimePeriod="+$FilterDateTimePeriod+"&filename=export.txt&template=%3C%23macro+escape+strValue%3E%24%7BstrValue%3Freplace('%22'%2C+'%22%22')%7D%3C%2F%23macro%3E%0A%5B%3C%23list+sessions+as+session%3E%0A%7Bid%3A+%22%3C%40escape+'%24%7Bsession.sessionIdentifier!%22%22%7D'%2F%3E%22%2C%0Arecordings%3A%5B%3C%23list+(session."+$CallDirection+"Call."+$RecordingType+"Recordings)!+as+recordingId%2C+recordingUrl%3E%0A%7Bfilename%3A%22%3C%40escape+'%24%7B(recordingId)!%22%22%7D'%2F%3E%22%7D%2C%0A%3C%2F%23list%3E%7B%7D%5D%2C%0AstartDate%3A+%22%3C%40escape+'%24%7B(session."+$CallDirection+"Call.startDateTime%3Fstring%5B%22yyyy.MM.dd%22%5D)!%22%22%7D'%2F%3E%22%2C%0AstartTime%3A+%22%3C%40escape+'%24%7B(session."+$CallDirection+"Call.startDateTime%3Fstring%5B%22HH%3Amm%3Ass%22%5D)!%22%22%7D'%2F%3E%22%2C%0Aduration%3A+%22%3C%40escape+'%24%7B(session."+$CallDirection+"Call.duration%3Fc)!%22%22%7D'%2F%3E%22%2C%0AsourceDialString%3A+%22%3C%40escape+'%24%7B(session."+$CallDirection+"Call.sourceDialString)!%22%22%7D'%2F%3E%22%2C%0AdestinationDialString%3A+%22%3C%40escape+'%24%7B(session."+$CallDirection+"Call.destinationDialString)!%22%22%7D'%2F%3E%22%7D%2C%0A%3C%2F%23list%3E%7B%7D%5D"

# Preset a URL to download files.
$FilesUrl=$Protocol+"://"+$AnynodeAddress+"/app/monitor/callhistory/recording/v0?file="

# ----------- Start of script -----------

# Invoke an HTTP request and store the result in $Sessions.
$Sessions=Invoke-RestMethod -Method POST -URI $SessionsUrl -Headers $Headers -SessionVariable webSession

$Found=0

# Iterate through each session to store recordings.
foreach ($Session in $Sessions) {
	$SessionId=$Session.id

	# In the unexpected case of a missing session ID, use a default.
	# A directory will be created with $SessionId as its path.
	if ($SessionId -eq $null -Or ($SessionId -eq '')) {
		$SessionId="Unkown"
	}
	$SessionId="Recordings/"+$SessionId

	# Iterate through the list of recordings of a session.
	$Recordings=$Session.recordings
	foreach ($Recording in $Recordings) {

		# Get the filename of a recording.
		# If it is not given or empty, there is no file.
		$Filename=$Recording.filename
		if ($Filename -eq $null -Or $Filename -eq '') {
			continue
		}

		# Create a URL to download a file, invoke it and store the content.
		$URL=$FilesUrl+$Filename
		$FileContent=Invoke-RestMethod -Method GET -URI $URL -WebSession $webSession

		# Create a directory with $SessionId as its path.
		md $SessionId -Force
		cd $SessionId

		# Create a file inside the new directory with $Filename as its path and $FileContent as its content.
		New-Item $Filename -Type file -Force
		Set-Content $Filename $FileContent
        $Found+=1

        # Gather session information
        $StartDate=$Session.startDate
        $StartTime=$Session.startTime
        $Duration=$Session.duration
        $SourceDialString=$Session.sourceDialString
        $DestinationDialString=$Session.destinationDialString

        $SessionInfo="Session ID: "+$Session.id+"`n`nStart Date: "+$StartDate+"`nStart Time: "+$StartTime+"`nCall Duration: "+$Duration+" seconds`nSource Dial String: "+$SourceDialString+"`nDestination Dial String: "+$DestinationDialString

        # Create a file inside the session's directory containing the session's information.
        New-Item "Info.txt" -Type file -Force
        Set-Content "Info.txt" $SessionInfo

		cd ../..
	}
}

if ($Found -eq 0) {
    echo "No recordings found for given filters."
}
pause
