# Script version: 4-6-0
# Script date: January 2022
#
# the following actions are performed by this script:
#
#	- import a Microsoft Teams node and a provider node into anynode
#	- set the previously entered TLS port
#	- set the previously entered FQDN
#	- add a certificate and a private key for the imported Teams node
#	- add a list of trusted certificates to the Teams node
#	- determine the name of the provider's transport connection
#	- check if the imported provider node contains a transport connection
#	- selection of the transport connection to be configured
#	- set the transport connection's credentials
#	- set the transport connection's asserted-URI
#	- set the transport connection's address-of-record
#	- create a routing domain which contains a direct routing
#  	  connection between the two imported nodes
#
# to use this script there are some variables to be defined
#
#	The path to the REST API PowerShell Module needs to be set.
#		- By default, the module is located a folder above the one in which this script is located.
#	The address of anynode have to be defined.
#	The nodes to import have to be exported previously.
#		- this can be done by using the "Export-anynodeObject" PowerShell-Module command or the anynode GUI.
#	The paths to the nodes to be imported must be defined.
#	The certificate and private key to be imported have to be created beforehand.
#	The paths to the created certificate and private key must be defined.
#
# Please make sure to review all possible parameters before
# starting the script.

# Set execution environment
Set-ExecutionPolicy -ExecutionPolicy Unrestricted -Scope Process

# The REST API PowerShell Module's path must be given here. In this case, the file is in the directory above the current one.
Import-Module "./../anynode_rest_api_powershell.psm1"

# Define anynode's Address and Port.
$AnynodeAddress='127.0.0.1'
$AnynodePort=443

# If you have not established a secure connection (HTTPS) to anynode,
# you can use HTTP instead.
# This is not recommended, since your data will NOT be encrypted!
# To activate it anyway, uncomment the following line:
# Disable-CertificateCheck

# Authentication
$Cred=Get-Credential

# Define the path to a previously exported Teams node.
$TeamsNodeFilepath='./Teams_Node.xzconfig'

# Define the path to a previously exported provider node.
$ProviderNodeFilepath='./Provider_Node.xzconfig'

# Define a new TLS port for the Teams node.
$NodePort='9999'

# Define a new fully qualified domain name for the Teams node.
$NodeFQDN='fully.qualified.domain.id'

# Define a username for the authentication of the provider's transport connection.
$Username='user'

# Define a password for the authentication of the provider's transport connection.
$Password='password'

# Define your own Asserted-URI. This URI can serve as the basis for special SIP headers (e.g. PAI header) in the case of outgoing calls.
$URI='sip:123456789'
$DisplayName='Display Name'

# Address-Of-Record (to be registered URI).
$AnynodeAddressOfRecordURI='sip:987654321'

# Define prefix filters for the route to be created.
$TeamsNodePrefix='111'
$ProviderNodePrefix='222'

# Define a certificate for the Teams node.
$Certificate='./cert.pem'

# Define a matching key to the certificate for the Teams node.
$Key='./key.pem'

# Create a list of trusted certificate file paths to be added to the Teams node.
$TrustedCertificates=@(
    './cert.pem' #,
	# 'Other Path',
	# 'Other Path',
	# ...
)

# ----------- Start of script -----------

# Import two nodes.
try {
    $NodesBeforeImportTeams=Get-anynodeNodes -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred

	Import-anynodeObject -Address $AnynodeAddress -Port $AnynodePort -Filepath $TeamsNodeFilepath -DuplicateOption CLONE -Credential $Cred

    $NodesAfterImportTeams=Get-anynodeNodes -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred

    if($NodesBeforeImportTeams -and $NodesAfterImportTeams) {
        $TeamsNode=compare -ReferenceObject $NodesBeforeImportTeams.id -DifferenceObject $NodesAfterImportTeams.id | ?{$_.SideIndicator -eq "=>"} | select -Expand InputObject
        $TeamsNode
    } else {
        $TeamsNode = $NodesAfterImportTeams
    }

    $NodesBeforeImportProvider=Get-anynodeNodes -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred

	Import-anynodeObject -Address $AnynodeAddress -Port $AnynodePort -Filepath $ProviderNodeFilepath -DuplicateOption CLONE -Credential $Cred

    $NodesAfterImportProvider=Get-anynodeNodes -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred

    if($NodesBeforeImportProvider -and $NodesAfterImportProvider) {
        $ProviderNode=compare -ReferenceObject $NodesBeforeImportProvider.id -DifferenceObject $NodesAfterImportProvider.id | ?{$_.SideIndicator -eq "=>"} | select -Expand InputObject
        $ProviderNode
    } else {
        $ProviderNode = $NodesAfterImportProvider
    }

} catch {
	$_.Exception.Message
	pause
	return
}

# Set the previously entered TLS port.
try {
	Set-anynodeNodeTlsPort -Address $AnynodeAddress -Port $AnynodePort -Node $TeamsNode -NewPort $NodePort -Credential $Cred
} catch {
	$_.Exception.Message
	pause
	return
}

# Set the previously entered FQDN.
try {
	Set-anynodeNodeFQDN -Address $AnynodeAddress -Port $AnynodePort -Node $TeamsNode -NewFQDN $NodeFQDN -Credential $Cred
} catch {
	$_.Exception.Message
	pause
	return
}

# Add a certificate and a private key to the Teams node.
try {
	Set-anynodeNodeCertificate -Address $AnynodeAddress -Port $AnynodePort -Node $TeamsNode -CertificateFilePath $Certificate -KeyFilePath $Key -Credential $Cred
} catch {
	$_.Exception.Message
	pause
	return
}

# Add the previously defined list of trusted certificates to the cloned node.
foreach ($TrustedCertificate in $TrustedCertificates) {
	try {
		Add-anynodeNodeTrustedCertificate -Address $AnynodeAddress -Port $AnynodePort -Node $TeamsNode -CertificateFilePath $TrustedCertificate -Credential $Cred
	} catch {
		$_.Exception.Message
		continue
	}
}

# Determine the name of the provider's transport connection.
try {
	$TransportConnection=Get-anynodeNodeTransportConnections -Address $AnynodeAddress -Port $AnynodePort -Node $ProviderNode -Credential $Cred
} catch {
	$_.Exception.Message
	pause
	return
}

# Set the transport connection's credentials.
try {
	Set-anynodeTransportConnectionAuthentication -Address $AnynodeAddress -Port $AnynodePort -TransportConnection $TransportConnection.transportConnectionId -TransportConnectionUsername $Username -TransportConnectionPassword $Password -Credential $Cred
} catch {
	$_.Exception.Message
	pause
	return
}

# Set the transport connection's Asserted-URI.
try {
	Set-anynodeTransportConnectionAssertedURI -Address $AnynodeAddress -Port $AnynodePort -TransportConnection $TransportConnection.transportConnectionId -URI $URI -DisplayName $DisplayName -Credential $Cred
} catch {
	$_.Exception.Message
	pause
	return
}

# Set the transport connection's Address-Of-Record.
try {
	Set-anynodeTransportConnectionAddressOfRecord -Address $AnynodeAddress -Port $AnynodePort -TransportConnection $TransportConnection.transportConnectionId -URI $AnynodeAddressOfRecordURI -Credential $Cred
} catch {
	$_.Exception.Message
	pause
	return
}

# Add a simple routing domain.
try {
	$RoutingDomain = (Add-anynodeRoutingDomain -Address $AnynodeAddress -Port $AnynodePort -DisplayName 'import Template Routing Domain' -SourceNodes @($TeamsNode, $ProviderNode) -Credential $Cred).id
} catch {
	$_.Exception.Message
	pause
	return
}


# Add routes from provider node to Teams node.
if($Route1FilterList -ne $null){
    $Route1FilterList.Clear()    
}

$Route1SourceFilter=[pscustomobject]@{sourceNodeIds=@($ProviderNode)}
$Route1PrefixFilter=[pscustomobject]@{sourceDialString=[pscustomobject]@{type="PLAIN";plain=[pscustomobject]@{prefix=$ProviderNodePrefix}}}
$Route1FilterList=@($Route1SourceFilter, $Route1PrefixFilter)

try {
	Add-anynodeEstablishmentRoute -Address $AnynodeAddress -Port $AnynodePort -RoutingDomain $RoutingDomain -UseUnconditionalRouting False -Filters $Route1FilterList -DestinationNodeId $TeamsNode -RoutingForwardProfileSelection NONE -DisplayName "To Teams node" -Credential $Cred
} catch {
	$_.Exception.Message
	pause
	return
}

# Add routes from Teams node to provider node.
if($Route2FilterList -ne $null){
    $Route2FilterList.Clear()    
}

$Route2SourceFilter=[pscustomobject]@{sourceNodeIds=@($TeamsNode)}
$Route2PrefixFilter=[pscustomobject]@{sourceDialString=[pscustomobject]@{type="PLAIN";plain=[pscustomobject]@{prefix=$TeamsNodePrefix}}}
$Route2FilterList=@($Route2SourceFilter, $Route2PrefixFilter)

try {
	Add-anynodeEstablishmentRoute -Address $AnynodeAddress -Port $AnynodePort -RoutingDomain $RoutingDomain -UseUnconditionalRouting False -Filters $Route2FilterList -DestinationNodeId $ProviderNode -RoutingForwardProfileSelection NONE -DisplayName "To Provider node" -Credential $Cred
} catch {

	$_.Exception.Message
	pause
	return
}

# Commit configuration changes.
Push-anynodeConfig -Address $AnynodeAddress -Port $AnynodePort -Credential $Cred
echo "push done"
pause
