#!/bin/bash

# Version anynode 4.6.0
# January 2022

# This script uses curl, cat and jq.
#  In case curl does not trust your web certificate, this script will not work. Please
#  edit all curl commands after the ---- Start of script ---- part to make it work.

# The following actions are performed by this script:
#
# - add two provider nodes and corresponding Microsoft Teams tenant nodes to anynode
# - add two routing domains that each contain a direct routing connection between
#   each provider node and their Microsoft Teams tenant node
# - the first Microsoft Teams tenant node will create a Microsoft Teams connectivity,
#   which will be used by both tenants

# to use this script, there are some variables to be defined
#
# Please make sure to review all possible parameters before starting the script.

#---- Variables ----

# Define anynode's address and port.
anynode='127.0.0.1'

# Put in credentials of a user with configuration write access and REST access.
cred='anadmin:password'

# Properties of first provider node (i.e. net4home)
net4homeProviderName='net4home Provider'
net4homeIpAddress='172.16.2.82'
net4homeUdpTcpPort=5060
net4homeTlsPort=5061
net4homeRemoteSipDomainIri='sip:net4home.anynodesbc.com'
net4homeClientUsername='user'
net4homeClientPassword='user password'
net4homeAddressOfRecordUser='0123456789'
net4homeRegistrarUri='sip:net4home.anynodesbc.com'
net4homeDefineAssertedUri=false
net4homeProxyUri='sip:net4home.anynodesbc.com'

# Properties of the first provider node's corresponding connectivity
net4homeMsTeamsName='net4home Microsoft Teams'
net4homeMsTeamsTenantFqdn='net4home'

# Properties of second provider node (i.e. TE-SYSTEMS)
tesystemsProviderName='TE-SYSTEMS Provider'
tesystemsIpAddress='172.16.2.82'
tesystemsUdpTcpPort=5062
tesystemsTlsPort=5063
tesystemsRemoteSipDomainIri='sip:tesystems.anynodesbc.com'
tesystemsClientUsername='user'
tesystemsClientPassword='user password'
tesystemsAddressOfRecordUser='9876543210'
tesystemsRegistrarUri='sip:tesystems.anynodesbc.com'
tesystemsDefineAssertedUri=false
tesystemsProxyUri='sip:tesystems.anynodesbc.com'

# Properties of the second provider node's corresponding connectivity
tesystemsMsTeamsName='TE-SYSTEMS Microsoft Teams'
tesystemsMsTeamsTenantFqdn='te-systems'

# General provider properties
providerIncomingDialStringRewriteRules='[
  {
    "rewriteType":"prefixAndSuffix",
    "prefix":"601",
    "deleteLeadingCharacters":3,
    "addPrefix":"john",
    "applyToSourceDialString":true,
    "applyToDestinationDialString":true,
    "processingDone":true
  },
  {
    "rewriteType":"prefixAndSuffix",
    "prefix":"00",
    "deleteLeadingCharacters":2,
    "addPrefix":"+",
    "applyToSourceDialString":true,
    "applyToDestinationDialString":true,
    "processingDone":true
  }
]
'
providerProfileName='Other'

# Microsoft Teams connectivity general properties
msTeamsSbcFqdn='sbc1.customer.anynodesbc.com'
msTeamsIpAddress='172.18.1.55'
msTeamsUdpTcpPort=5054
msTeamsTlsPort=5055
msTeamsIncomingDialStringRewriteRules='[
  {
    "rewriteType":"prefixAndSuffix",
    "prefix":"+555",
    "deleteLeadingCharacters":4,
    "addPrefix":"+49",
    "applyToSourceDialString":true,
    "applyToDestinationDialString":true,
    "processingDone":true
  }
]'

# Properties for first routing domain (i.e. net4home)
net4homeRdName='net4home Routing Domain'
net4homeRdSourceNodes=$(cat << END
["${net4homeProviderName}","${net4homeMsTeamsName}"]
END
)

# Properties for second routing domain (i.e. TE-SYSTEMS)
tesystemsRdName='TE-SYSTEMS Routing Domain'
tesystemsRdSourceNodes=$(cat << END
["${tesystemsProviderName}","${tesystemsMsTeamsName}"]
END
)

# ---- Start of script ----
# The next 4 lines check whether a secure connection can be established
# to the given address.
# Delete them and change all following curl requests if you want to use
# an insecure connection.
ssl=$(curl -s https://$anynode/)
if [[ $ssl == "" ]]; then
  echo 'No secure connection could be established to anynode.'
fi

net4homeNode=$(cat << END
{
  "displayName":"${net4homeProviderName}",
  "networkControllerIpAddress":"${net4homeIpAddress}",
  "udpTcpPort":${net4homeUdpTcpPort},
  "tlsPort":${net4homeTlsPort},
  "remoteSipDomain":{
    "iri":"${net4homeRemoteSipDomainIri}"
  },
  "credentials":{
    "username":"${net4homeClientUsername}",
    "password":"${net4homeClientPassword}"
  },
  "registrarUri":"${net4homeRegistrarUri}",
  "addressOfRecord":"${net4homeAddressOfRecordUser}",
  "proxyUri":"${net4homeProxyUri}",
  "defineAssertedUri":${net4homeDefineAssertedUri},
  "incomingDialStringRewriteRules":${providerIncomingDialStringRewriteRules}
}
END
)

tesystemsNode=$(cat << END
{
  "displayName":"${tesystemsProviderName}",
  "networkControllerIpAddress":"${tesystemsIpAddress}",
  "udpTcpPort":${tesystemsUdpTcpPort},
  "tlsPort":${tesystemsTlsPort},
  "remoteSipDomain":{
    "iri":"${tesystemsRemoteSipDomainIri}"
  },
  "credentials":{
    "username":"${tesystemsClientUsername}",
    "password":"${tesystemsClientPassword}"
  },
  "registrarUri":"${tesystemsRegistrarUri}",
  "addressOfRecord":"${tesystemsAddressOfRecordUser}",
  "proxyUri":"${tesystemsProxyUri}",
  "defineAssertedUri":${tesystemsDefineAssertedUri},
  "incomingDialStringRewriteRules":${providerIncomingDialStringRewriteRules}
}
END
)

net4homeMsTeamsNode=$(cat << END
{
  "displayName":"${net4homeMsTeamsName}",
  "tenantFqdn":"${net4homeMsTeamsTenantFqdn}",
  "sbcFqdn":"${msTeamsSbcFqdn}",
  "networkControllerIpAddress":"${msTeamsIpAddress}",
  "udpTcpPort":${msTeamsUdpTcpPort},
  "tlsPort":${msTeamsTlsPort},
  "incomingDialStringRewriteRules":${msTeamsIncomingDialStringRewriteRules}
}
END
)

net4homeRd=$(cat << END
{
  "displayName":"${net4homeRdName}",
  "sourceNodes":${net4homeRdSourceNodes}
}
END
)

tesystemsRd=$(cat << END
{
  "displayName":"${tesystemsRdName}",
  "sourceNodes":${tesystemsRdSourceNodes}
}
END
)

# Retrieve the provider profile ID mandatory to add providers
# If you want to use an insecure connection, add the -k argument for curl
providerProfileId=$(curl -s -u ${cred} https://${anynode}/api/nodes/providerProfiles/get?version=0 | jq -c ".[] | select(.displayName | contains(\"${providerProfileName}\"))" | jq -r .id)
echo 'Provider Profile ID: '${providerProfileId}

# Retrieve the MS Teams profile ID mandatory to add connectivities
# If you want to use an insecure connection, add the -k argument for curl
msTeamsProfileId=$(curl -s -u ${cred} https://${anynode}/api/nodes/systemProfiles/get?version=0 | jq -c ".[] | select(.displayName | contains(\"Microsoft Teams Direct Routing Carrier\"))" | jq -r .id)
echo 'Microsoft Teams Profile ID: '${msTeamsProfileId}

# Add the first provider node
# If you want to use an insecure connection, add the -k argument for curl
node1=$(curl -s -u ${cred} -d "${net4homeNode}" https://${anynode}/api/nodes/addProvider?version=1\&profile=${providerProfileId})
echo 'Node1: '${node1}

# Save the node's ID for later use
node1Id=$(jq -r '.[] | .id' <<< "${node1}")

# Add the first tenant
# If you want to use an insecure connection, add the -k argument for curl
tenant1=$(curl -s -u ${cred} -d "${net4homeMsTeamsNode}" https://${anynode}/api/nodes/addSystem?version=1\&profile=${msTeamsProfileId})
echo 'Tenant1: '${tenant1}

# Save the tenant's ID for later use
tenant1Id=$(jq -r '.[] | .id' <<< "${tenant1}")

# Add the second provider node
# If you want to use an insecure connection, add the -k argument for curl
node2=$(curl -s -u ${cred} -d "${tesystemsNode}" https://${anynode}/api/nodes/addProvider?version=1\&profile=${providerProfileId})
echo 'Node2: '${node2}

# Save the node's ID for later use
node2Id=$(jq -r '.[] | .id' <<< "${node2}")

# Add the second tenant
tesystemsMsTeamsNode=$(cat << END
{
  "displayName":"${tesystemsMsTeamsName}",
  "tenantFqdn":"${tesystemsMsTeamsTenantFqdn}",
  "createNewTeamsCarrierNode":false,
  "teamsCarrierNodeId":"${tenant1Id}"
}
END
)
# If you want to use an insecure connection, add the -k argument for curl
tenant2=$(curl -s -u ${cred} -d "${tesystemsMsTeamsNode}" https://${anynode}/api/nodes/addSystem?version=1\&profile=${msTeamsProfileId})
echo 'Tenant2: '${tenant2}

# Save the tenant's ID for later use
tenant2Id=$(jq -r '.[] | .id' <<< "${tenant2}")

# Create the first routing domain
# If you want to use an insecure connection, add the -k argument for curl
rd1=$(curl -s -u ${cred} -d "${net4homeRd}" https://${anynode}/api/routingDomains/add?version=1)
echo 'Routing Domain1: '${rd1}

# Save the routing domain's ID for later use
rd1Id=$(jq -r '.id' <<< "${rd1}")

# Create a route from the first provider node to its tenant node
rd1Route1=$(cat << END
{
  "displayName":"To ${net4homeProviderName}",
  "filters":[
    {
      "sourceNodeIds":[
        "${tenant1Id}",
      ]
    }
  ],
  "establishment":{
    "destinationNodeId":"${node1Id}",
    "routingForwardProfileSelection":"NONE"
  }
}
END
)
# If you want to use an insecure connection, add the -k argument for curl
curl -s -u ${cred} -d "${rd1Route1}" https://${anynode}/api/routingDomains/routes/add?version=0\&routingDomain=${rd1Id}
echo

# Create a route from the first tenant node to its provider node
rd1Route2=$(cat << END
{
  "displayName":"To ${net4homeMsTeamsName}",
  "filters":[
    {
      "sourceNodeIds":[
        "${node1Id}",
      ]
    }
  ],
  "establishment":{
    "destinationNodeId":"${tenant1Id}",
    "routingForwardProfileSelection":"NONE"
  }
}
END
)
# If you want to use an insecure connection, add the -k argument for curl
curl -s -u ${cred} -d "${rd1Route2}" https://${anynode}/api/routingDomains/routes/add?version=0\&routingDomain=${rd1Id}
echo


# Create the second routing domain
# If you want to use an insecure connection, add the -k argument for curl
rd2=$(curl -s -u ${cred} -d "${tesystemsRd}" https://${anynode}/api/routingDomains/add?version=1)
echo 'Routing Domain2: '${rd2}

# Save the routing domain's ID for later use
rd2Id=$(jq -r '.id' <<< "${rd2}")

# Create a route from the second provider node to its tenant node
rd2Route1=$(cat << END
{
  "displayName":"To ${tesystemsProviderName}",
  "filters":[
    {
      "sourceNodeIds":[
        "${tenant2Id}",
      ]
    }
  ],
  "establishment":{
    "destinationNodeId":"${node2Id}",
    "routingForwardProfileSelection":"NONE"
  }
}
END
)
# If you want to use an insecure connection, add the -k argument for curl
curl -s -u ${cred} -d "${rd2Route1}" https://${anynode}/api/routingDomains/routes/add?version=0\&routingDomain=${rd2Id}
echo

# Create a route from the second tenant node to its provider node
rd2Route2=$(cat << END
{
  "displayName":"To ${tesystemsMsTeamsName}",
  "filters":[
    {
      "sourceNodeIds":[
        "${node2Id}",
      ]
    }
  ],
  "establishment":{
    "destinationNodeId":"${tenant2Id}",
    "routingForwardProfileSelection":"NONE"
  }
}
END
)
# If you want to use an insecure connection, add the -k argument for curl
curl -s -u ${cred} -d "${rd2Route2}" https://${anynode}/api/routingDomains/routes/add?version=0\&routingDomain=${rd2Id}
echo

# Commit changes
# If you want to use an insecure connection, add the -k argument for curl
curl -s -u ${cred} -X POST https://${anynode}/api/anynodeConfig/commit?version=0
echo
