#!/bin/bash

# Version anynode 4.6.0
# January 2022

# This script uses curl, cat and jq.
#  In case curl does not trust your web certificate, this script will not work. Please
#  edit all curl commands after the ---- Start of script ---- part to make it work.

# the following actions are performed by this script:
# - download all recorded sessions from a chosen call direction and time range
# - create a folder structure for each session
#
# The structure will hold a text file that contains various information about the session.

# to use this script, there are some variables to be defined
# - The address of anynode have to be defined.
# - Define wich call direction recordings should be used.
# - Define wich type of recordings should be downloaded.
# - Define the Connection Protocol to be used.

# Please make sure to review all possible parameters before
# starting the script.

# ---- Variables ----

# Define anynode's address and port.
anynode='127.0.0.1'

# Put in credentials of a user with configuration write access and REST access.
cred='anadmin:password'

# Which call direction recordings should be used? "incoming" or "outgoing"
callDirection='outgoing'

# Which type of recordings should be downloaded? "wav" or "g711"
type='wav'

# -- Request URL
# Insert a template to rietrieve data from today (filterDateTimePeriod=TODAY)
filterDateTimePeriod='TODAY'

filterCount=2147483647
filename='export.txt'

# Split the template into lines to make it easier to read
t1="%3C%23macro%20escape%20strValue%3E%24%7BstrValue%3Freplace(%27%22%27%2C%20%27%22%22%27)%7D%3C%2F%23macro%3E"
t2="%0A%5B%3C%23list%20sessions%20as%20session%3E%0A%7B%22id%22%3A%22%3C%40escape%20%27%24%7Bsession.sessionIdentifier!%22%22%7D%27%2F%3E%22%2C%0A"
t3="%22recordings%22%3A%5B%3C%23list%20(session.${callDirection}Call.${type}Recordings)!%20as%20recordingId%2C%20recordingUrl%3E"
t4="%0A%7B%22filename%22%3A%22%3C%40escape%20%27%24%7B(recordingId)!%22%22%7D%27%2F%3E%22%7D%2C%0A"
t5="%3C%2F%23list%3E%7B%7D%5D%2C%0A"
t6="%22startDate%22%3A%22%3C%40escape%20%27%24%7B(session.${callDirection}Call.startDateTime%3F"
t7="string%5B%22yyyy.MM.dd%22%5D)!%22%22%7D%27%2F%3E%22%2C%0A"
t8="%22startTime%22%3A%22%3C%40escape%20%27%24%7B(session.${callDirection}Call.startDateTime%3F"
t9="string%5B%22HH%3Amm%3Ass%22%5D)!%22%22%7D%27%2F%3E%22%2C%0A"
t10="%22duration%22%3A%22%3C%40escape%20%27%24%7B(session.${callDirection}Call.duration%3Fc)!%22%22%7D%27%2F%3E%22%2C%0A"
t11="%22sourceDialString%22%3A%22%3C%40escape%20%27%24%7B(session.${callDirection}Call.sourceDialString)!%22%22%7D%27%2F%3E%22%2C%0A"
t12="%22destinationDialString%22%3A%22%3C%40escape%20%27%24%7B(session.${callDirection}Call.destinationDialString)!%22%22%7D%27%2F%3E%22%7D%2C%0A"
t13="%3C%2F%23list%3E%7B%7D%5D"

template=$t1$t2$t3$t4$t5$t6$t7$t8$t9$t10$t11$t12$t13

# Preset a URL to download files
filesUrl="https://${anynode}/app/monitor/callhistory/recording/v0?file="

# --- Start of script ----
# The next 4 lines check whether a secure connection can be established
# to the given address.
# Delete them and change all following curl requests if you want to use
# an insecure connection.
ssl=$(curl -s https://$anynode/)
if [[ $ssl == "" ]]; then
  echo 'No secure connection could be established to anynode.'
fi

# Create a directory for results
if [ ! -d Recordings ]
then
  mkdir ./Recordings
fi

# Request the filtered sessions from the dashboard
# If you want to use an insecure connection, add the -k argument for curl
sessions=$(curl -s -u $cred -X POST -H "Content-Type=application/x-www-form-urlencoded" https://$anynode/app/monitor/callhistory/\export/v0?filterCount=$filterCount\&filterDateTimePeriod=$filterDateTimePeriod\&filename=$filename\&template=$template)

# Iterate over every session
for key in $(jq -r 'keys | .[]' <<< $sessions); do
  session=$(jq -r ".[$key]" <<< $sessions)

  # Retrieve the session ID to create a directory with its name
  id=$(jq -r .id <<< $session)
  # Retrieve a list of recordings saved to this session
  recordings=$(jq -r .recordings <<< $session)

  # Iterate over all recordings
  for innerKey in $(jq -r 'keys | .[]' <<< $recordings); do
    recording=$(jq -r ".[$innerKey]" <<< $recordings)

    # Retrieve the recording's file name
    filename=$(jq -r .filename <<< $recording)

    # If the recording does not have a file name, skip this recording
    if [[ $filename == "null" ]]
    then
      continue
    fi

    # In case this session does not have its own directory yet, create one
    if [ ! -d Recordings/$id ]
    then
      mkdir Recordings/$id
    else
      rm Recordings/$id/*
    fi

    # Create a file inside the session's directory containing a recording
    # If you want to use an insecure connection, add the -k argument for curl
    curl -s -u $cred $filesUrl$filename >> ./Recordings/$id/$filename

    # Gather session information
    startDate=$(jq -r .startDate <<< $session)
    startTime=$(jq -r .startTime <<< $session)
    duration=$(jq -r .duration <<< $session)
    sourceDialString=$(jq -r .sourceDialString <<< $session)
    destinationDialString=$(jq -r .destinationDialString <<< $session)

    echo 'Session '$key
    echo 'Recording '$innerKey
    echo Recordings/$id/$filename

    # Create a file inside the session's directory containing the session's information
    printf "Session ID: $id\n\nStart Date: $startDate\nStart Time: $startTime\nCall Duration: $duration\nSource Dial String: $sourceDialString\nDestination Dial String: $destinationDialString" >> ./Recordings/$id/Info.txt
  done
done
