#!/bin/bash

# Version anynode 4.6.0
# January 2022

# This script uses curl, cat and jq.
#  In case curl does not trust your web certificate, this script will not work. Please
#  edit all curl commands after the ---- Start of script ---- part to make it work.

# The following actions are perfomed by this script:
# - import a Microsoft Teams node and a provider node into anynode
# - set the previously entered TLS port
# - set the previously entered FQDN
# - add a certificate and private key for the imported Teams node
# - add a list of trusted certificates to the Teams node
# - check if the imported provider node contains a transport connection
#   - determine the name of the provider's transport connection
#   - selection of the transport connection to be configured
#   - set the transport connection's credentials
#   - set the transport connection's asserted URI
#   - set the transport connection's address-of-record
# - create a routing domain which contains a direct routing connection
#   between the two imported nodes

# To use this script, there are some variables to be defined.
# Please make sure to review all possible parameters before
# starting the script.

# The nodes have to be exported previously.

# The certificate and private key need to be created beforehand.

# --- Variables ----

# Define anynode's address
anynode='127.0.0.1'

# Authentication
cred='anadmin:password'

# Define the path to a previously exported Teams node
teamsFile='./msteams.xzconfig'

# Define the path to a previously exported provider node
providerFile='./provider.xzconfig'

# Define a new TLS port for the Teams node
teamsPort=5067

# Define a new fully qualified domain name for the Teams node
teamsFqdn='fully.qualified.domain.name'

# Define a username for the authentication of the provider's transport connection
providerAuthUsername='user'

# Define a password for the authentication of the provider's transport connection
providerAuthPassword='user password'

# Define your own asserted URI. This URI can serve as the basis for special SIP
# headers (e.g. PAI header) in the case of outgoing calls.
assertedUri='sip:123456789'
assertedDisplayName='Display Name'

# Define an address-of-record (to be registered URI)
providerAor='sip:987654321'

# Define prefix filters for the route to be created
teamsNodePrefix='111'
providerNodePrefix='222'

# Define a certificate path for the Teams node
certFile='./cert.cer'

# Define a matching key path to the certificate for the Teams node
keyFile='./key.pem'

# Optional: The password for the certificate or private key
# certPass=''

# Create a list of trusted certificate file paths to be added to the Teams node
trustedCertFiles=$(cat << END
[
  "./cert.cer",
  "./other_cert.cer"
]
END
)

echo $trustedCertFiles

# ---- Start of script ----
# The next 4 lines check whether a secure connection can be established
# to the given address.
# Delete them and change all following curl requests if you want to use
# an insecure connection.
ssl=$(curl -s https://$anynode/)
if [[ $ssl == "" ]]; then
  echo 'No secure connection could be established to anynode.'
fi

# Check whether the given paths contain files
if [ ! -f $teamsNode ]
then
  echo "Teams node file not found at $teamsFile"
  exit
elif [ ! -f $providerFile ]
then
  echo "Provider node file not found at $providerFile"
  exit
elif [ ! -f $certFile ]
then
  echo "Certificate file not found at $certFile"
  exit
elif [ ! -f $keyFile ]
then
  echo "Key file not found at $keyFile"
  exit
fi

# Import the Teams node
teamsNode=$(curl -s -u $cred -X PUT -F "node=@$teamsFile" https://$anynode/api/objects/import?version=0\&action=OVERRIDE)
echo "Teams Node: $teamsNode"

# Retrieve the node's ID for later use
teamsId=$(jq -r '.id' <<< "$teamsNode")

# Import the provider node
providerNode=$(curl -s -u $cred -X PUT -F "node=@$providerFile" https://$anynode/api/objects/import?version=0\&action=OVERRIDE)
echo "provider Node: $providerNode"

# Retrieve the node's ID for later use
providerId=$(jq -r '.id' <<< "$providerNode")

# Set the previously entered TLS port
curl -s -u $cred -X PUT -d "$teamsPort" https://$anynode/api/nodes/tlsPort/set?version=0\&node=$teamsId
echo

# Set the previously entered FQDN
curl -s -u $cred -X PUT -d "$teamsFqdn" https://$anynode/api/nodes/tenants/fqdn/set?version=0\&node=$teamsId
echo

# Add a certificate to the Teams node
curl -s -u $cred -X PUT -F "cert=@$certFile" https://$anynode/api/nodes/certificates/set?version=0\&node=$teamsId\&password=$certPass
echo

# Add a private key to the Teams node
curl -s -u $cred -X PUT -F "key=@$keyFile" https://$anynode/api/nodes/certificates/set?version=0\&node=$teamsId\&password=$certPass
echo

# Add the previously defined list of trusted certificates to the cloned node
for key in $(jq -r 'keys | .[]' <<< "$trustedCertFiles"); do
  file=$(jq -r ".[$key]" <<< "$trustedCertFiles")
  if [ ! -f $file ]
  then
    echo "Certificate file not found at $file"
    continue
  fi

  curl -su $cred -X PUT -F "cert=@$file" https://$anynode/api/nodes/trustedCertificates/add?version=0\&node=$teamsId
  echo
done

# Determine the name of the provider's transport connection
tcs=$(curl -su $cred https://$anynode/api/nodes/transportConnections/get?version=0\&node=$providerId)
# Use the first one in the list
tc=$(jq -r '.[0].transportConnection' <<< $tcs)
echo "Transport Connection: $tc"

tcId=$(jq -r .id <<< $tc)

# Set the transport connection's credentials
echo 'Transport Connection Authentication: '$(curl -su $cred -X PUT -d "{\"username\":$providerAuthUsername,\"password\":$providerAuthPassword}" https://$anynode/api/transportConnections/authentication/set?version=0\&transportConnection=$tcId)

# Set the transport connection's asserted URI
echo 'Transport Connection asserted URI: '$(curl -su $cred -X PUT -d "{\"iri\":$assertedUri,\"displayName\":$assertedDisplayName}" https://$anynode/api/transportConnections/assertedUri/set?version=0\&transportConnection=$tcId)

# Add a simple routing domain
rd=$(curl -su $cred -X POST -d "{\"displayName\":\"Import Template Routing Domain\",\"sourceNodes\":[$teamsId,$providerId]}" https://$anynode/api/routingDomains/add?version=1)
echo 'Routing Domain: '$rd

rdId=$(jq -r .id <<< $rd)

# Add a route from provider node to Teams node
route1=$(cat << END
{
  "displayName":"To Teams node",
  "useUnconditionalRouting":"false",
  "establishment":{
    "destinationNodeId":$teamsId,
    "routingForwardProfileSelection":"NONE"
  },
  "filters":[
    {
      "sourceNodeIds":[
        $providerId
      ],
      "sourceDialString":{
        "type":"PLAIN",
        "plain":{
          "prefix":$providerNodePrefix
        }
      }
    }
  ]
}
END
)

echo 'Route 1: '$(curl -su $cred -X POST -d "$route1" https://$anynode/api/routingDomains/routes/add?version=0\&routingDomain=$rdId)

# Add a route from Teams node to provider node
route2=$(cat << END
{
  "displayName":"To Provider node",
  "useUnconditionalRouting":"false",
  "establishment":{
    "destinationNodeId":$providerId,
    "routingForwardProfileSelection":"NONE"
  },
  "filters":[
    {
      "sourceNodeIds":[
        $teamsId
      ],
      "sourceDialString":{
        "type":"PLAIN",
        "plain":{
          "prefix":$teamsNodePrefix
        }
      }
    }
  ]
}
END
)

echo 'Route 2: '$(curl -su $cred -X POST -d "$route2" https://$anynode/api/routingDomains/routes/add?version=0\&routingDomain=$rdId)

# Commit configuration changes
curl -su $cred -X POST https://$anynode/api/anynodeConfig/commit?version=0
echo
