if ($host.Version.Major -LT 5) {
	throw "PowerShell version outdated. Please update. Required version: 5.1.x.x."
}

if ($host.Version.Minor -LT 1) {
	if ($host.Version.Major -LE 5) {
		throw "PowerShell version outdated. Please update. Required version: 5.1.x.x."
	}
}

Add-Type @"
using System;
using System.Net;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
public class ServerCertificateValidationCallback {
    public static void DoNotIgnore(Object[] certificates) {
		ServicePointManager.ServerCertificateValidationCallback += 
			delegate(Object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors errors) {
                foreach(Object trustedCertificate in certificates) {
                    if (trustedCertificate.Equals(certificate)) {
                        return true;
                    }
                }
				return false;
			};
	}

	public static void Ignore() {
		ServicePointManager.ServerCertificateValidationCallback += 
			delegate(Object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors errors) {
				return true;
			};
	}

	public static void AcceptOnly(X509Certificate cert, Object[] certificates) {
		ServicePointManager.ServerCertificateValidationCallback += 
			delegate(Object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors errors) {
				if (cert.Equals(certificate)) {
                    return true;
                }

                foreach(Object trustedCertificate in certificates) {
                    if (trustedCertificate.Equals(certificate)) {
                        return true;
                    }
                }

                return false;
			};
	}
}
"@

$script:Version='4.12.5'
$script:APIVersion=0
$script:Address=''
$script:Port=443
$script:Credential=$null
$script:CertificateCheckDisabled=$false

# Disable Certificate check (at your own risk!)
# This will not only disable the certificate check for anynode requests, but for
# all requests.
function Disable-CertificateCheck {
    $script:CertificateCheckDisabled=$true
	[ServerCertificateValidationCallback]::Ignore()
}

function Enable-CertificateCheck {
    $script:CertificateCheckDisabled=$false
	[ServerCertificateValidationCallback]::DoNotIgnore((Get-ChildItem 'Cert:/CurrentUser/Root/'))
}

function Initialize-anynodeAddress {
    [CmdletBinding()]
    Param(
        [Parameter(mandatory=$true)]
        [string] $Address
    )

    if ($Address -eq $null -OR $Address -eq '') {
        echo 'Please enter an address.'
        return
    }

    $script:Address=$Address
    echo "Address to communicate with anynode is set to: $Address"
}

function Reset-anynodeAddress {
    $script:Address=''
    echo 'Reset address used to communicate with anynode.'
}

function Initialize-anynodePort {
    [CmdletBinding()]
    Param(
        [Parameter(mandatory=$true)]
        [string] $Port
    )

    if ($Port -eq $null -OR $Port -le 0) {
        echo 'Please enter a valid port.'
        return
    }

    $script:Port=$Port
    echo "Port to communicate with anynode is set to: $Port"
}

function Reset-anynodePort {
    $script:Port=443
    echo 'Reset port used to communicate with anynode to value: 443'
}

function Initialize-anynodeCredential {
    [CmdletBinding(DefaultParameterSetName='cred')]
    Param(
        [Parameter(mandatory=$true,ParameterSetName='cred')]
        [PSCredential] $Credential,
        [Parameter(mandatory=$true,ParameterSetName='userpass')]
        [string] $Username,
        [Parameter(mandatory=$true,ParameterSetName='userpass')]
        [string] $Password
    )

    if ($Credential -ne $null) {
        $script:Credential=$Credential
    } elseif ($Username -ne $null) {
        try {
            $SecretPassword=ConvertTo-SecureString $Password -AsPlainText -Force
	        $script:Credential=New-Object System.Management.Automation.PSCredential($Username,$SecretPassword)
        } catch {
            echo 'Could not set default credential.'
            return
        }
    }

    echo 'Set default credential.'
}

function Reset-anynodeCredential {
    $script:Credential=$null
    echo 'Reset default credential.'
}

# During this session, you can only communicate with the given interface.
function Get-anynodeWebserverCertificate {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $CertificateFilePath
	)

	if ($Address -eq '' -OR $Address -eq $null) {
        if ($script:Address -eq '') {
            $Address=Read-Host 'Address'
        } else {
            $Address=$script:Address
        }

        if ($Address -eq '') {
            echo 'Address required.'
            return
        }
	}

	if ($Port -eq 0) {
		if ($script:Port -eq 0) {
            $Port=Read-Host 'Port'
        } else {
            $Port=$script:Port
        }

        if ($Port -eq 0) {
            echo 'Port required.'
            return
        }
	}

    Disable-CertificateCheck

	try {
		$Request=[System.Net.WebRequest]::Create('https://'+$Address+':'+$Port+'/')
		$Request.GetResponse().Dispose()
		$Certificate=$Request.ServicePoint.Certificate

		if (!($CertificateFilePath -eq $null) -AND !($CertificateFilePath -eq '')) {
			$CertBytes=$Certificate.Export([System.Security.Cryptography.X509Certificates.X509ContentType]::Cert)
			Set-Content -Value $CertBytes -Encoding Byte -Path $CertificateFilePath
		}

		[ServerCertificateValidationCallback]::AcceptOnly($Certificate, (Get-ChildItem 'Cert:/CurrentUser/Root/'))
	} catch {
        Enable-CertificateCheck
		$_.Exception
		return
	}
}

function Import-anynodeWebserverCertificate {
	[CmdletBinding()]
	Param(
		[Parameter(mandatory=$true)]
		[string] $CertificatePath
	)

	try {
		$Certificate=[X509Certificate]::CreateFromCertFile($CertificatePath)
	} catch {
		$_.Exception
		return
	}

	[ServerCertificateValidationCallback]::AcceptOnly($Certificate, (Get-ChildItem 'Cert:/CurrentUser/Root/'))
    echo "Set certificate to $CertificatePath"
}

# Invokes
#	Download
function Invoke-anynodeDownload {
	[CmdletBinding()]
	Param(		
		[Parameter()]
		[ValidateSet('Get','Put','Post','Delete')]
		[string] $Method="GET",
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $URITail,
		[Parameter()]
		[string] $Filename,
		[Parameter()]
		[Hashtable] $Parameters,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	if ($APIVersion -eq 0) {
		$APIVersion=$script:APIVersion
	}

	if ($Address -eq '' -OR $Address -eq $null) {
        if ($script:Address -eq '') {
            $Address=Read-Host 'Address'
        } else {
            $Address=$script:Address
        }

        if ($Address -eq '') {
            echo 'Address required.'
            return
        }
	}
	
	if ($Port -eq 0) {
		if ($script:Port -eq 0) {
            $Port=Read-Host 'Port'
        } else {        
            $Port=$script:Port
        }

        if ($Port -eq 0) {
            echo 'Port required.'
            return
        }
	}

	if (!($Credential -eq $null)) {
		if (!($Username -eq '') -Or !($Password -eq '')) {
			echo 'Use either Credentials OR Username and Password. Not both.'
			return
		}
	}

	if (!($Username -eq '')) {
		if ($Password -eq '') {
			echo 'To use Username and Password, both must be given.'
			return
		}

		$SecretPassword=ConvertTo-SecureString $Password -AsPlainText -Force
		$Credential=New-Object System.Management.Automation.PSCredential($Username,$SecretPassword)
	}

    if ($Credential -eq $null -AND $script:Credential -ne $null) {
        $Credential=$script:Credential
    }

	$URI='https://'+$Address+':'+$Port

	$ParameterString='?version='+$APIVersion+'&filename='+$Filename
	foreach ($Parameter in $Parameters.Keys) {
		$ParameterString+='&'+$Parameter+'='+$Parameters.$Parameter
	}

	if (!($BackendName -eq '')) {
		$ParameterString+='&backendName='+$BackendName
	}

	if (!($Credential -eq $null)) {
		$Username=$Credential.Username
		$Password=$Credential.GetNetworkCredential().Password

		$Base64String=[System.Convert]::ToBase64String([Text.Encoding]::ASCII.GetBytes($Username+':'+$Password))
	}

	if (!($Base64String -eq $null)) {
		$Headers=@{Authorization="Basic "+$Base64String}
	}

    $Base64String=$null

	$URI='https://'+$Address+':'+$Port+$URITail+$ParameterString

	try {
		$Request=Invoke-RestMethod -Method $Method -URI $URI -Headers $Headers -OutFile $Filename -ContentType 'application/json'

		$Request
	} catch {
        try {
            if ($script:CertificateCheckDisabled) {
                $URI='http://'+$Address+':'+$Port+$URITail+$ParameterString
                $Request=Invoke-RestMethod -Method $Method -URI $URI -Headers $Headers -OutFile $Filename -ContentType 'application/json'

		        $Request
            } else {
                throw $_
            }
        } catch {
            $e=$_.Exception
		    $output=$e.InnerException

		    if ($output.Length -eq 0) {
			    $output=$e.Message+' '+$_.ToString()
		    }

		    $output
			return
        }
	}

	if ($Filename -ne $null -AND $Filename -ne '') {
		echo "Saved file at `"$Filename`"."
		return
	}
}

#	Request
function Invoke-anynodeRequest {
	[CmdletBinding()]
	Param(
		[Parameter(mandatory=$true)]
		[ValidateSet('Get','Put','Post','Delete')]
		[string] $Method,
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $URITail,
		[Parameter()]
		[string] $Body,
		[Parameter()]
		[Hashtable] $Parameters,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	if ($APIVersion -eq 0) {
		$APIVersion=$script:APIVersion
	}

	if ($Address -eq '' -OR $Address -eq $null) {
        if ($script:Address -eq '') {
            $Address=Read-Host 'Address'
        } else {
            $Address=$script:Address
        }

        if ($Address -eq '') {
            echo 'Address required.'
            return
        }
	}

	if ($Port -eq 0) {
		if ($script:Port -eq 0) {
            $Port=Read-Host 'Port'
        } else {
            $Port=$script:Port
        }

        if ($Port -eq 0) {
            echo 'Port required.'
            return
        }
	}

	if (!($Credential -eq $null)) {
		if (!($Username -eq '') -Or !($Password -eq '')) {
			echo 'Use either Credentials OR Username and Password. Not both.'
			return
		}
	}

	if (!($Username -eq '')) {
		if ($Password -eq '') {
			echo 'To use Username and Password, both must be given.'
			return
		}

		$SecretPassword=ConvertTo-SecureString $Password -AsPlainText -Force
		$Credential=New-Object System.Management.Automation.PSCredential($Username,$SecretPassword)
	}

    if ($Credential -eq $null -AND $script:Credential -ne $null) {
        $Credential=$script:Credential
    }

	$URI='https://'+$Address+':'+$Port

	$ParameterString='?version='+$APIVersion
	foreach ($Parameter in $Parameters.Keys) {
		$ParameterString+='&'+$Parameter+'='+$Parameters.$Parameter
	}

	if (!($BackendName -eq '')) {
		$ParameterString+='&backendName='+$BackendName
	}

	if (!($Credential -eq $null)) {
		$Username=$Credential.Username
		$Password=$Credential.GetNetworkCredential().Password

		$Base64String=[System.Convert]::ToBase64String([Text.Encoding]::ASCII.GetBytes($Username+':'+$Password))
	}

	if (!($Base64String -eq $null)) {
		$Headers=@{Authorization="Basic "+$Base64String}
	}

    $Base64String=$null

	$URI='https://'+$Address+':'+$Port+$URITail+$ParameterString

	try {
		if (($Method -eq 'PUT') -Or ($Method -eq 'Put') -Or ($Method -eq 'put') -Or ($Method -eq 'POST') -Or ($Method -eq 'Post') -Or ($Method -eq 'post')) {
			$Request=Invoke-RestMethod -Method $Method -ContentType 'application/json' -URI $URI -Headers $Headers -Body $Body

		} else {
			$Request=Invoke-RestMethod -Method $Method -URI $URI -Headers $Headers
		}

		$Request
	} catch {
        try {
            if ($script:CertificateCheckDisabled) {
                $URI='http://'+$Address+':'+$Port+$URITail+$ParameterString
                if (($Method -eq 'PUT') -Or ($Method -eq 'Put') -Or ($Method -eq 'put') -Or ($Method -eq 'POST') -Or ($Method -eq 'Post') -Or ($Method -eq 'post')) {
			        $Request=Invoke-RestMethod -Method $Method -ContentType 'application/json' -URI $URI -Headers $Headers -Body $Body

		        } else {
			        $Request=Invoke-RestMethod -Method $Method -URI $URI -Headers $Headers
		        }

		        $Request
				return
            } else {
                throw $_
            }
        } catch {
			$e=$_.Exception
			$output=$e.InnerException

			if ($output.Length -eq 0) {
				$output=$e.Message+' '+$_.ToString()
			}

			$output
        }
	}
}

#	Upload
function Invoke-anynodeUpload {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $URITail,
		[Parameter(mandatory=$true)]
		[Object[]] $Filenames,
		[Parameter()]
		[Hashtable] $Parameters,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	if ($APIVersion -eq 0) {
		$APIVersion=$script:APIVersion
	}

	if ($Address -eq '' -OR $Address -eq $null) {
        if ($script:Address -eq '') {
            $Address=Read-Host 'Address'
        } else {
            $Address=$script:Address
        }

        if ($Address -eq '') {
            echo 'Address required.'
            return
        }
	}

	if ($Port -eq 0) {
		if ($script:Port -eq 0) {
            $Port=Read-Host 'Port'
        } else {
            $Port=$script:Port
        }

        if ($Port -eq 0) {
            echo 'Port required.'
            return
        }
	}

	if (!($Credential -eq $null)) {
		if (!($Username -eq '') -Or !($Password -eq '')) {
			echo 'Use either Credentials OR Username and Password. Not both.'
			return
		}
	}

	if (!($Username -eq '')) {
		if ($Password -eq '') {
			echo 'To use Username and Password, both must be given.'
			return
		}

		$SecretPassword=ConvertTo-SecureString $Password -AsPlainText -Force
		$Credential=New-Object System.Management.Automation.PSCredential($Username,$SecretPassword)
	}

    if ($Credential -eq $null -AND $script:Credential -ne $null) {
        $Credential=$script:Credential
    }

	$URI='https://'+$Address+':'+$Port

	$ParameterString='?version='+$APIVersion
	foreach ($Parameter in $Parameters.Keys) {
		$ParameterString+='&'+$Parameter+'='+$Parameters.$Parameter
	}

	if (!($BackendName -eq '')) {
		$ParameterString+='&backendName='+$BackendName
	}

	if (!($Credential -eq $null)) {
		$Username=$Credential.Username
		$Password=$Credential.GetNetworkCredential().Password

		$Base64String=[System.Convert]::ToBase64String([Text.Encoding]::ASCII.GetBytes($Username+':'+$Password))
	}

	$URI='https://'+$Address+':'+$Port+$URITail+$ParameterString

	if (!($Base64String -eq $null)) {
		$Headers=@{Authorization="Basic "+$Base64String}
	}

    $Base64String=$null

	$Boundary=[System.Guid]::NewGuid().ToString()
	$ContentType='multipart/form-data; boundary="'+$Boundary+'"'

	$NL="`r`n";

	$Body=[byte[]]::new(0)

	foreach ($Filename in $Filenames) {
		$Bytes=[System.IO.File]::ReadAllBytes($Filename)

		$Head=(
			"--$Boundary",
		    "Content-Disposition: form-data; name=`"file`"; filename=`"$Filename`"",
		    "Content-Type: application/octet-stream$NL$NL"
		) -join $NL

		$Body+=[Text.Encoding]::UTF8.GetBytes($Head)
		$Body+=$Bytes

		$Foot="$NL--$Boundary--$NL"
		$Body+=[Text.Encoding]::UTF8.GetBytes($Foot)
	}

	$tmp=New-TemporaryFile
	$tmpName=$tmp.FullName
	[IO.File]::WriteAllBytes($tmpName,$Body)

	try {
		Invoke-RestMethod -Method PUT -Uri $URI -Headers $Headers -ContentType $ContentType -InFile $tmpName
	} catch {
        try {
            if ($script:CertificateCheckDisabled) {
                $URI='http://'+$Address+':'+$Port+$URITail+$ParameterString
                Invoke-RestMethod -Method PUT -Uri $URI -Headers $Headers -ContentType $ContentType -InFile $tmpName

		        $Request
				return
            } else {
                throw $_
            }
        } catch {
        }

		$e=$_.Exception
		$output=$e.InnerException

		if ($output.Length -eq 0) {
			$output=$e.Message+' '+$_.ToString()
		}

		$output
	} finally {
		Remove-Item $tmpName -Force	
	}
}

# Module Version
#	Get
function Get-anynodeModuleVersion {
	$script:Version
}

#	DEPRECATED
function Add-anynodeProviderNodeV2 {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Profile,
		[Parameter()]
		[psobject] $DeutscheTelekomNumberExtensionRange,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $CreateNewDeutscheTelekomSipTrunk=$true,
		[Parameter()]
		[string] $DeutscheTelekomSipTrunkId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $CreateNewNetworkController=$true,
		[Parameter()]
		[string] $NetworkControllerDisplayName,
		[Parameter()]
		[string] $NetworkControllerInterface='[Any Interface]',
		[Parameter()]
		[ValidateSet('[Any Address Version]','IN_ADDRESS_VERSION_4','IN_ADDRESS_VERSION_6')]
		[string] $NetworkControllerIpVersion='[Any Address Version]',
		[Parameter()]
		[string] $NetworkControllerIpAddress='[Any IP Address]',
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $EnableReverseDnsLookup,
		[Parameter()]
		[string] $NetworkControllerId,
		[Parameter()]
		[int] $UdpTcpPort,
		[Parameter()]
		[int] $TlsPort,
		[Parameter()]
		[psobject] $UdpPortRange,
		[Parameter()]
		[psobject] $TcpPortRange,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $ActivateNatTraversal,
		[Parameter()]
		[string] $ExternalHost,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $NatDeviceHasFixedPortMapping,
		[Parameter()]
		[int] $FixedUdpTcpPort,
		[Parameter()]
		[int] $FixedTlsPort,
		[Parameter()]
		[int] $FixedUdpPortRangeStart,
		[Parameter()]
		[int] $FixedTcpPortRangeStart,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $NodeViaSipTrunking,
		[Parameter()]
		[psobject] $RemoteSipDomain,
		[Parameter()]
		[string] $LocalSipDomain,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $ActivateClientAuthentication,
		[Parameter()]
		[psobject] $ClientCredentials,
		[Parameter()]
		[string] $RegistrarUri,
		[Parameter()]
		[string] $AddressOfRecord,
		[Parameter()]
		[string] $ProxyUri,
		[Parameter()]
		[string] $PrimaryProxyUri,
		[Parameter()]
		[string] $SecondaryProxyUri,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $DefineAssertedUri,
		[Parameter()]
		[string] $AssertedUri,
		[Parameter()]
		[string] $AssertedUriHost,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $UseNetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $IncludeRemoteSipDomainInWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $IncludeSipRegistrarInWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $IncludeProxyInWhitelist,
		[Parameter()]
		[psobject] $NetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $AllowOnlyNegotiatedPeersForRtpRtcp,
        [Parameter()]
		[psobject] $IncomingDialStringRewriteRules,
		[Parameter()]
		[psobject] $OutgoingDialStringRewriteRules,
		[Parameter()]
		[psobject] $RoutingDomains,
        [Parameter()]
		[ValidateSet("SCHWEIZ","LIECHTENSTEIN")]
		[psobject] $Region,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion=2,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{profile=$Profile}

	$Body=@{}

    if(!($Region -eq $null)) {
        $Body.Add("region", $Region)
    }

	if (!($DeutscheTelekomNumberExtensionRange -eq $null)) {
		$Body.Add("deutscheTelekomNumberExtensionRange",$DeutscheTelekomNumberExtensionRange)
	}

	$Body.Add("createNewDeutscheTelekomSipTrunk",$CreateNewDeutscheTelekomSipTrunk)

	if (!($DeutscheTelekomSipTrunkId -eq $null) -AND !($DeutscheTelekomSipTrunkId -eq '')) {
		$Body.Add("deutscheTelekomSipTrunkId",$DeutscheTelekomSipTrunkId)
	}

	$Body.Add("createNewNetworkController",$CreateNewNetworkController)
	if ($CreateNewNetworkController) {
		if (!($NetworkControllerDisplayName -eq $null) -AND !($NetworkControllerDisplayName -eq '')) {
			$Body.Add("networkControllerDisplayName",$NetworkControllerDisplayName)
		}
		$Body.Add("networkControllerInterface",$NetworkControllerInterface)
		$Body.Add("networkControllerIpVersion",$NetworkControllerIpVersion)
		$Body.Add("networkControllerIpAddress",$NetworkControllerIpAddress)
		if (!($EnableReverseDnsLookup -eq $null)) {
			$Body.Add("enableReverseDnsLookup",$EnableReverseDnsLookup)
		}
	}

	if (!($NetworkControllerId -eq $null) -AND !($NetworkControllerId -eq '')) {
		$Body.Add("networkControllerId",$NetworkControllerId)
	}

	if (!($UdpTcpPort -eq $null) -AND $UdpTcpPort -gt 0) {
		$Body.Add("udpTcpPort",$UdpTcpPort)
	}

	if (!($TlsPort -eq $null) -AND $TlsPort -gt 0) {
		$Body.Add("tlsPort",$TlsPort)
	}

	if (!($UdpPortRange -eq $null)) {
		$Body.Add("udpPortRange",$UdpPortRange)
	}

	if (!($TcpPortRange -eq $null)) {
		$Body.Add("tcpPortRange",$TcpPortRange)
	}

	if (!($ActivateNatTraversal -eq $null)) {
		$Body.Add("activateNatTraversal",$ActivateNatTraversal)
	}
	if ($ActivateNatTraversal) {
		$Body.Add("externalHost",$ExternalHost)

		if (!($NatDeviceHasFixedPortMapping -eq $null)) {
			$Body.Add("natDeviceHasFixedPortMapping",$NatDeviceHasFixedPortMapping)
		}
		if ($NatDeviceHasFixedPortMapping) {
			$Body.Add("fixedUdpTcpPort",$FixedUdpTcpPort)
			$Body.Add("fixedTlsPort",$FixedTlsPort)
			$Body.Add("fixedUdpPortRangeStart",$FixedUdpPortRangeStart)
			$Body.Add("fixedTcpPortRangeStart",$FixedTcpPortRangeStart)
		}
	}

	if (!($NodeViaSipTrunking -eq $null)) {
		$Body.Add("nodeViaSipTrunking",$NodeViaSipTrunking)
	}

	if (!($RemoteSipDomain -eq $null)) {
		$Body.Add("remoteSipDomain",$RemoteSipDomain)
	}

	if (!($LocalSipDomain -eq $null) -AND !($LocalSipDomain -eq '')) {
		$Body.Add("localSipDomain",$LocalSipDomain)
	}

	if (!($ActivateClientAuthentication -eq $null)) {
		$Body.Add("activateClientAuthentication",$ActivateClientAuthentication)
	}

	if (!($ClientCredentials -eq $null)) {
		$Body.Add("credentials",$ClientCredentials)
	}

	if (!($RegistrarUri -eq $null) -AND !($RegistrarUri -eq '')) {
		$Body.Add("registrarUri",$RegistrarUri)
	}

	if (!($AddressOfRecord -eq $null)) {
		$Body.Add("addressOfRecord",$AddressOfRecord)
	}

	if (!($ProxyUri -eq $null) -AND !($ProxyUri -eq '')) {
		$Body.Add("proxyUri",$ProxyUri)
	}

	if (!($PrimaryProxyUri -eq $null) -AND !($PrimaryProxyUri -eq '')) {
		$Body.Add("primaryProxyUri",$PrimaryProxyUri)
	}

	if (!($SecondaryProxyUri -eq $null) -AND !($SecondaryProxyUri -eq '')) {
		$Body.Add("secondaryProxyUri",$SecondaryProxyUri)
	}

	if (!($DefineAssertedUri -eq $null)) {
		$Body.Add("defineAssertedUri",$DefineAssertedUri)
	}
	if ($DefineAssertedUri) {
		$Body.Add("assertedUri",$AssertedUri)
	}

	if ($AssertedUriHost) {
		$Body.Add("assertedUriHost",$AssertedUriHost)
	}

	if (!($UseNetworkPeerWhitelist -eq $null)) {
		$Body.Add("useNetworkPeerWhitelist",$UseNetworkPeerWhitelist)
	}

	$Body.Add("includeRemoteSipDomainInWhitelist",$IncludeRemoteSipDomainInWhitelist)
	$Body.Add("includeSipRegistrarInWhitelist",$IncludeSipRegistrarInWhitelist)
	$Body.Add("includeProxyInWhitelist",$IncludeProxyInWhitelist)
	$Body.Add("networkPeerWhitelist",$NetworkPeerWhitelist)
	$Body.Add("allowOnlyNegotiatedPeersForRtpRtcp",$AllowOnlyNegotiatedPeersForRtpRtcp)

    if (!($IncomingDialStringRewriteRules -eq $null)) {
		if ($IncomingDialStringRewriteRules.GetType().Name -eq 'Object[]') {
			$Body.Add("incomingDialStringRewriteRules",$IncomingDialStringRewriteRules)
		} else {
			$Body.Add("incomingDialStringRewriteRules",@{0=$IncomingDialStringRewriteRules}.Values)
		}
	}

	if (!($OutgoingDialStringRewriteRules -eq $null)) {
		if ($OutgoingDialStringRewriteRules.GetType().Name -eq 'Object[]') {
			$Body.Add("outgoingDialStringRewriteRules",$OutgoingDialStringRewriteRules)
		} else {
			$Body.Add("outgoingDialStringRewriteRules",@{0=$OutgoingDialStringRewriteRules}.Values)
		}
	}

	if (!($RoutingDomains -eq $null)) {
		$Body.Add("routingDomains",$RoutingDomains)
	}

	if (!($DisplayName -eq $null) -AND !($DisplayName -eq '')) {
		$Body.Add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/addProvider -Body ($Body | ConvertTo-Json -Depth 99) -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	DEPRECATED
function Add-anynodeSystemNodeV2 {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Profile,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $CreateNewNetworkController=$true,
		[Parameter()]
		[string] $NetworkControllerDisplayName,
		[Parameter()]
		[string] $NetworkControllerInterface='[Any Interface]',
		[Parameter()]
		[ValidateSet('[Any Address Version]','IN_ADDRESS_VERSION_4','IN_ADDRESS_VERSION_6')]
		[string] $NetworkControllerIpVersion='[Any Address Version]',
		[Parameter()]
		[string] $NetworkControllerIpAddress='[Any IP Address]',
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $EnableReverseDnsLookup,
		[Parameter()]
		[string] $NetworkControllerId,
		[Parameter()]
		[int] $UdpTcpPort,
		[Parameter()]
		[int] $TlsPort,
		[Parameter()]
		[psobject] $UdpPortRange,
		[Parameter()]
		[psobject] $TcpPortRange,
		[Parameter()]
		[ValidateSet("REGISTRAR","REGISTRATION","TRUNK")]
		[psobject] $SipInterconnection,
		[Parameter()]
		[psobject] $RemoteSipDomain,
		[Parameter()]
		[string] $LocalSipDomain,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $ActivateClientAuthentication,
		[Parameter()]
		[psobject] $ClientCredentials,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $ActivateServerAuthentication,
		[Parameter()]
		[psobject] $ServerCredentials,
		[Parameter()]
		[string] $RegistrarUri,
		[Parameter()]
		[string] $AddressOfRecord,
		[Parameter()]
		[psobject] $SipRegistrar,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $UseNetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $IncludeRemoteSipDomainInWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $IncludeSipRegistrarInWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $IncludeOwnSubnetInWhitelist,
		[Parameter()]
		[psobject] $NetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $AllowOnlyNegotiatedPeersForRtpRtcp,
        [Parameter()]
		[psobject] $IncomingDialStringRewriteRules,
		[Parameter()]
		[psobject] $OutgoingDialStringRewriteRules,
		[Parameter()]
		[psobject] $RoutingDomains,
		[Parameter()]
		[string] $DisplayName,
        [Parameter()]
        [string] $GooglePbxTrunkSecretKey,
        [Parameter()]
		[ValidateSet("NA","EMEA","AUS","APAC","LATAM","CHINA","JAPAN")]
        [psobject] $Region,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion=3,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{profile=$Profile}

	$Body=@{}

	$Body.Add("createNewNetworkController",$CreateNewNetworkController)
	if ($CreateNewNetworkController) {
		if (!($NetworkControllerDisplayName -eq $null) -AND !($NetworkControllerDisplayName -eq '')) {
			$Body.Add("networkControllerDisplayName",$NetworkControllerDisplayName)
		}
		$Body.Add("networkControllerInterface",$NetworkControllerInterface)
		$Body.Add("networkControllerIpVersion",$NetworkControllerIpVersion)
		$Body.Add("networkControllerIpAddress",$NetworkControllerIpAddress)
		if (!($EnableReverseDnsLookup -eq $null)) {
			$Body.Add("enableReverseDnsLookup",$EnableReverseDnsLookup)
		}
	}

	if (!($NetworkControllerId -eq $null) -AND !($NetworkControllerId -eq '')) {
		$Body.Add("networkControllerId",$NetworkControllerId)
	}

	if (!($UdpTcpPort -eq $null) -AND $UdpTcpPort -gt 0) {
		$Body.Add("udpTcpPort",$UdpTcpPort)
	}

	if (!($TlsPort -eq $null) -AND $TlsPort -gt 0) {
		$Body.Add("tlsPort",$TlsPort)
	}

	if (!($UdpPortRange -eq $null)) {
		$Body.Add("udpPortRange",$UdpPortRange)
	}

	if (!($TcpPortRange -eq $null)) {
		$Body.Add("tcpPortRange",$TcpPortRange)
	}

	if (!($SipInterconnection -eq $null)) {
		$Body.Add("sipInterconnection",$SipInterconnection)
	}

	if (!($RemoteSipDomain -eq $null)) {
		$Body.Add("remoteSipDomain",$RemoteSipDomain)
	}

	if (!($LocalSipDomain -eq $null) -AND !($LocalSipDomain -eq '')) {
		$Body.Add("localSipDomain",$LocalSipDomain)
	}

	if (!($ActivateClientAuthentication -eq $null)) {
		$Body.Add("activateClientAuthentication",$ActivateClientAuthentication)
	}
	if (!($ClientCredentials -eq $null)) {
		$Body.Add("clientCredentials",$ClientCredentials)
	}

	if (!($ActivateServerAuthentication -eq $null)) {
		$Body.Add("activateServerAuthentication",$ActivateServerAuthentication)
	}
	if (!($ServerCredentials -eq $null)) {
		$Body.Add("serverCredentials",$ServerCredentials)
	}

	if (!($RegistrarUri -eq $null) -AND !($RegistrarUri -eq '')) {
		$Body.Add("registrarUri",$RegistrarUri)
	}

	if (!($AddressOfRecord -eq $null)) {
		$Body.Add("addressOfRecord",$AddressOfRecord)
	}

	if (!($SipRegistrar -eq $null)) {
		$Body.Add("sipRegistrar",$SipRegistrar)
	}

	if (!($UseNetworkPeerWhitelist -eq $null)) {
		$Body.Add("useNetworkPeerWhitelist",$UseNetworkPeerWhitelist)
	}

	$Body.Add("includeRemoteSipDomainInWhitelist",$IncludeRemoteSipDomainInWhitelist)
	$Body.Add("includeSipRegistrarInWhitelist",$IncludeSipRegistrarInWhitelist)
	$Body.Add("includeProxyInWhitelist",$IncludeProxyInWhitelist)
	$Body.Add("includeOwnSubnetInWhitelist",$IncludeOwnSubnetInWhitelist)
	$Body.Add("networkPeerWhitelist",$NetworkPeerWhitelist)
	$Body.Add("allowOnlyNegotiatedPeersForRtpRtcp",$AllowOnlyNegotiatedPeersForRtpRtcp)

    if (!($IncomingDialStringRewriteRules -eq $null)) {
		if ($IncomingDialStringRewriteRules.GetType().Name -eq 'Object[]') {
			$Body.Add("incomingDialStringRewriteRules",$IncomingDialStringRewriteRules)
		} else {
			$Body.Add("incomingDialStringRewriteRules",@{0=$IncomingDialStringRewriteRules}.Values)
		}
	}

	if (!($OutgoingDialStringRewriteRules -eq $null)) {
		if ($OutgoingDialStringRewriteRules.GetType().Name -eq 'Object[]') {
			$Body.Add("outgoingDialStringRewriteRules",$OutgoingDialStringRewriteRules)
		} else {
			$Body.Add("outgoingDialStringRewriteRules",@{0=$OutgoingDialStringRewriteRules}.Values)
		}
	}

    if(!($GooglePbxTrunkSecretKey -eq $null)) {
        $Body.Add("googlePbcTrunkSecretKey", $GooglePbxTrunkSecretKey)
    }

    if(!($Region -eq $null)) {
        $Body.Add("region", $Region)
    }

	if (!($RoutingDomains -eq $null)) {
		$Body.Add("routingDomains",$RoutingDomains)
	}

	if (!($DisplayName -eq $null) -AND !($DisplayName -eq '')) {
		$Body.Add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/addSystem -Body ($Body | ConvertTo-Json -Depth 99) -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Add MS Teams Tenant - deprecated parameters incomingManipulations and outgoingManipulations
function Add-anynodeMsTeamsTenantNode {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $CreateNewTeamsCarrierNode=$false,
		[Parameter()]
		[string] $TeamsCarrierNodeId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $CreateNewNetworkController=$true,
		[Parameter()]
		[string] $NetworkControllerDisplayName,
		[Parameter()]
		[string] $NetworkControllerInterface='[Any Interface]',
		[Parameter()]
		[ValidateSet('[Any Address Version]','IN_ADDRESS_VERSION_4','IN_ADDRESS_VERSION_6')]
		[string] $NetworkControllerIpVersion='[Any Address Version]',
		[Parameter()]
		[string] $NetworkControllerIpAddress='[Any IP Address]',
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $EnableReverseDnsLookup,
		[Parameter()]
		[string] $NetworkControllerId,
		[Parameter()]
		[int] $UdpTcpPort,
		[Parameter()]
		[int] $TlsPort,
		[Parameter()]
		[psobject] $UdpPortRange,
		[Parameter()]
		[psobject] $TcpPortRange,
		[Parameter()]
		[string] $SbcFqdn,
		[Parameter()]
		[string] $TenantFqdn,
        [Parameter()]
		[psobject] $IncomingDialStringRewriteRules,
		[Parameter()]
		[psobject] $OutgoingDialStringRewriteRules,
		[Parameter()]
		[psobject] $RoutingDomains,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion=3,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password,
		[Parameter()]
		[psobject] $IncomingManipulations,
		[Parameter()]
		[psobject] $OutgoingManipulations
	)

    $Profile=Get-anynodeSystemNodeProfiles -Address $Address -Port $Port -Credential $Credential -Username $Username -Password $Password | Where-Object -Property "displayName" -EQ "Microsoft Teams Direct Routing Carrier"

	$Parameters=@{profile=$Profile.id}

	$Body=@{}

	$Body.Add("createNewTeamsCarrierNode",$CreateNewTeamsCarrierNode)
	if ($CreateNewTeamsCarrierNode) {

		$Body.Add("createNewNetworkController",$CreateNewNetworkController)
		if ($CreateNewNetworkController) {
			if (!($NetworkControllerDisplayName -eq $null) -AND !($NetworkControllerDisplayName -eq '')) {
				$Body.Add("networkControllerDisplayName",$NetworkControllerDisplayName)
			}
			$Body.Add("networkControllerInterface",$NetworkControllerInterface)
			$Body.Add("networkControllerIpVersion",$NetworkControllerIpVersion)
			$Body.Add("networkControllerIpAddress",$NetworkControllerIpAddress)
			if (!($EnableReverseDnsLookup -eq $null)) {
				$Body.Add("enableReverseDnsLookup",$EnableReverseDnsLookup)
			}
		}

		if (!($NetworkControllerId -eq $null) -AND !($NetworkControllerId -eq '')) {
			$Body.Add("networkControllerId",$NetworkControllerId)
		}
	
		if (!($UdpTcpPort -eq $null) -AND $UdpTcpPort -gt 0) {
			$Body.Add("udpTcpPort",$UdpTcpPort)
		}
	
		if (!($TlsPort -eq $null) -AND $TlsPort -gt 0) {
			$Body.Add("tlsPort",$TlsPort)
		}
	
		if (!($UdpPortRange -eq $null)) {
			$Body.Add("udpPortRange",$UdpPortRange)
		}
	
		if (!($TcpPortRange -eq $null)) {
			$Body.Add("tcpPortRange",$TcpPortRange)
		}

		if (!($SbcFqdn -eq $null) -AND !($SbcFqdn -eq '')) {
			$Body.Add("sbcFqdn",$SbcFqdn)
		}
	}

	if (!($TeamsCarrierNodeId -eq $null) -AND !($TeamsCarrierNodeId -eq '')) {
		$Body.Add("teamsCarrierNodeId",$TeamsCarrierNodeId)
	}

	if (!($TenantFqdn -eq $null) -AND !($TenantFqdn -eq '')) {
		$Body.Add("tenantFqdn",$TenantFqdn)
	}

	if (!($IncomingManipulations -eq $null)) {
		if ($IncomingManipulations.GetType().Name -eq 'Object[]') {
			$Body.Add("incomingManipulations",$IncomingManipulations)
		} else {
			$Body.Add("incomingManipulations",@{0=$IncomingManipulations}.Values)
		}
	}

	if (!($OutgoingManipulations -eq $null)) {
		if ($OutgoingManipulations.GetType().Name -eq 'Object[]') {
			$Body.Add("outgoingManipulations",$OutgoingManipulations)
		} else {
			$Body.Add("outgoingManipulations",@{0=$OutgoingManipulations}.Values)
		}
	}

	if (!($IncomingDialStringRewriteRules -eq $null)) {
		if ($IncomingDialStringRewriteRules.GetType().Name -eq 'Object[]') {
			$Body.Add("incomingDialStringRewriteRules",$IncomingDialStringRewriteRules)
		} else {
			$Body.Add("incomingDialStringRewriteRules",@{0=$IncomingDialStringRewriteRules}.Values)
		}
	}

	if (!($OutgoingDialStringRewriteRules -eq $null)) {
		if ($OutgoingDialStringRewriteRules.GetType().Name -eq 'Object[]') {
			$Body.Add("outgoingDialStringRewriteRules",$OutgoingDialStringRewriteRules)
		} else {
			$Body.Add("outgoingDialStringRewriteRules",@{0=$OutgoingDialStringRewriteRules}.Values)
		}
	}

	if (!($RoutingDomains -eq $null)) {
		$Body.Add("routingDomains",$RoutingDomains)
	}

	if (!($DisplayName -eq $null) -AND !($DisplayName -eq '')) {
		$Body.Add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/addSystem -Body ($Body | ConvertTo-Json -Depth 99) -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	DEPRECATED!
function Add-anynodeMsTeamsTenantNodeV2 {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $CreateNewTeamsCarrierNode=$false,
		[Parameter()]
		[string] $TeamsCarrierNodeId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $CreateNewNetworkController=$true,
		[Parameter()]
		[string] $NetworkControllerDisplayName,
		[Parameter()]
		[string] $NetworkControllerInterface='[Any Interface]',
		[Parameter()]
		[ValidateSet('[Any Address Version]','IN_ADDRESS_VERSION_4','IN_ADDRESS_VERSION_6')]
		[string] $NetworkControllerIpVersion='[Any Address Version]',
		[Parameter()]
		[string] $NetworkControllerIpAddress='[Any IP Address]',
		[Parameter()]
		[ValidateSet($true,$false)]
		[psobject] $EnableReverseDnsLookup,
		[Parameter()]
		[string] $NetworkControllerId,
		[Parameter()]
		[int] $UdpTcpPort,
		[Parameter()]
		[int] $TlsPort,
		[Parameter()]
		[psobject] $UdpPortRange,
		[Parameter()]
		[psobject] $TcpPortRange,
		[Parameter()]
		[string] $SbcFqdn,
		[Parameter()]
		[string] $TenantFqdn,
        [Parameter()]
		[psobject] $IncomingDialStringRewriteRules,
		[Parameter()]
		[psobject] $OutgoingDialStringRewriteRules,
		[Parameter()]
		[psobject] $RoutingDomains,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion=3,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

    $Profile=Get-anynodeSystemNodeProfiles -Address $Address -Port $Port -Credential $Credential -Username $Username -Password $Password | Where-Object -Property "displayName" -EQ "Microsoft Teams Direct Routing Carrier"

	$Parameters=@{profile=$Profile.id}

	$Body=@{}

	$Body.Add("createNewTeamsCarrierNode",$CreateNewTeamsCarrierNode)
	if ($CreateNewTeamsCarrierNode) {

		$Body.Add("createNewNetworkController",$CreateNewNetworkController)
		if ($CreateNewNetworkController) {
			if (!($NetworkControllerDisplayName -eq $null) -AND !($NetworkControllerDisplayName -eq '')) {
				$Body.Add("networkControllerDisplayName",$NetworkControllerDisplayName)
			}
			$Body.Add("networkControllerInterface",$NetworkControllerInterface)
			$Body.Add("networkControllerIpVersion",$NetworkControllerIpVersion)
			$Body.Add("networkControllerIpAddress",$NetworkControllerIpAddress)
			if (!($EnableReverseDnsLookup -eq $null)) {
				$Body.Add("enableReverseDnsLookup",$EnableReverseDnsLookup)
			}
		}

		if (!($NetworkControllerId -eq $null) -AND !($NetworkControllerId -eq '')) {
			$Body.Add("networkControllerId",$NetworkControllerId)
		}
	
		if (!($UdpTcpPort -eq $null) -AND $UdpTcpPort -gt 0) {
			$Body.Add("udpTcpPort",$UdpTcpPort)
		}
	
		if (!($TlsPort -eq $null) -AND $TlsPort -gt 0) {
			$Body.Add("tlsPort",$TlsPort)
		}
	
		if (!($UdpPortRange -eq $null)) {
			$Body.Add("udpPortRange",$UdpPortRange)
		}
	
		if (!($TcpPortRange -eq $null)) {
			$Body.Add("tcpPortRange",$TcpPortRange)
		}

		if (!($SbcFqdn -eq $null) -AND !($SbcFqdn -eq '')) {
			$Body.Add("sbcFqdn",$SbcFqdn)
		}
	}

	if (!($TeamsCarrierNodeId -eq $null) -AND !($TeamsCarrierNodeId -eq '')) {
		$Body.Add("teamsCarrierNodeId",$TeamsCarrierNodeId)
	}

	if (!($TenantFqdn -eq $null) -AND !($TenantFqdn -eq '')) {
		$Body.Add("tenantFqdn",$TenantFqdn)
	}

    if (!($IncomingDialStringRewriteRules -eq $null)) {
		if ($IncomingDialStringRewriteRules.GetType().Name -eq 'Object[]') {
			$Body.Add("incomingDialStringRewriteRules",$IncomingDialStringRewriteRules)
		} else {
			$Body.Add("incomingDialStringRewriteRules",@{0=$IncomingDialStringRewriteRules}.Values)
		}
	}

	if (!($OutgoingDialStringRewriteRules -eq $null)) {
		if ($OutgoingDialStringRewriteRules.GetType().Name -eq 'Object[]') {
			$Body.Add("outgoingDialStringRewriteRules",$OutgoingDialStringRewriteRules)
		} else {
			$Body.Add("outgoingDialStringRewriteRules",@{0=$OutgoingDialStringRewriteRules}.Values)
		}
	}

	if (!($RoutingDomains -eq $null)) {
		$Body.Add("routingDomains",$RoutingDomains)
	}

	if (!($DisplayName -eq $null) -AND !($DisplayName -eq '')) {
		$Body.Add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/addSystem -Body ($Body | ConvertTo-Json -Depth 99) -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}


#	Get Incoming Manipulations
function Get-anynodeNodeIncomingManipulations {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Node,
		[switch] $IncludeDefaults,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{node=$Node}
	if ($IncludeDefaults) {
		$Parameters.Add("includeDefaults",$true)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/incomingManipulations/get -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Set Incoming Manipulations
function Set-anynodeNodeIncomingManipulations {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Node,
		[Parameter()]
		[psobject] $Manipulations,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{node=$Node}

	$Body=($Manipulations | ConvertTo-Json)

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/incomingManipulations/set -Parameters $Parameters -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Add Incoming Manipulations
function Add-anynodeNodeIncomingManipulations {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Node,
		[Parameter()]
		[psobject] $Manipulations,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{node=$Node}

	$Body=($Manipulations | ConvertTo-Json)

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/incomingManipulations/add -Parameters $Parameters -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Remove Incoming Manipulation
function Remove-anynodeNodeIncomingManipulation {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Node,
		[Parameter()]
		[int] $Index=-1,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{node=$Node}

	if ($Index -gt -1) {
		$Parameters.Add("index",$Index)
	} elseif ($DisplayName -ne $null -AND $DisplayName -ne '') {
		$Parameters.Add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/incomingManipulations/remove -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Get Outgoing Manipulations
function Get-anynodeNodeOutgoingManipulations {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Node,
		[switch] $IncludeDefaults,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{node=$Node}
	if ($IncludeDefaults) {
		$Parameters.Add("includeDefaults",$true)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/outgoingManipulations/get -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Set Outgoing Manipulations
function Set-anynodeNodeOutgoingManipulations {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Node,
		[Parameter()]
		[psobject] $Manipulations,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{node=$Node}

	$Body=($Manipulations | ConvertTo-Json)

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/outgoingManipulations/set -Parameters $Parameters -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Add Outgoing Manipulations
function Add-anynodeNodeOutgoingManipulations {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Node,
		[Parameter()]
		[psobject] $Manipulations,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{node=$Node}

	$Body=($Manipulations | ConvertTo-Json)

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/outgoingManipulations/add -Parameters $Parameters -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Remove Outgoing Manipulation
function Remove-anynodeNodeOutgoingManipulation {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $Node,
		[Parameter()]
		[int] $Index=-1,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{node=$Node}

	if ($Index -gt -1) {
		$Parameters.Add("index",$Index)
	} elseif ($DisplayName -ne $null -AND $DisplayName -ne '') {
		$Parameters.Add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/outgoingManipulations/remove -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Add
function Add-anynodeSimpleRoutingDomain {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter(mandatory=$true)]
		[string] $Node1,
		[Parameter(mandatory=$true)]
		[string] $Node2,
		[Parameter()]
		[string] $PrefixTo1,
		[Parameter()]
		[string] $PrefixTo2,
		[Parameter()]
		[string] $RoutingForwardProfile,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{"node1"=$Node1}
	$Body.Add("node2",$Node2)

	if (!($PrefixTo1 -eq '')) {
		$Body.Add("prefixTo1",$PrefixTo1)
	}

	if (!($PrefixTo2 -eq '')) {
		$Body.Add("prefixTo2",$PrefixTo2)
	}

	if (!($RoutingForwardProfile -eq '')) {
		$Body.Add("routingForwardProfile",$RoutingForwardProfile)
	}

	if (!($DisplayName -eq '')) {
		$Body.Add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/add -Body ($Body | ConvertTo-Json) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}


#	Add
function Add-anynodeEstablishmentRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $RoutingDomain,
		[Parameter()]
		[ValidateSet($true,$false)]
		[pscustomobject] $UseUnconditionalRouting,
		[Parameter()]
		[ValidateSet($true,$false)]
		[pscustomobject] $UseRouteSupervision,
		[Parameter()]
		[string] $SupervisionId,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[Object[]] $Filters,
		[Parameter()]
		[ValidateSet('ROUTE','IGNORE','REJECT','REDIRECT','ESTABLISH_PARALLEL')]
		[string] $Type,
		[Parameter()]
		[string] $DestinationNodeId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[pscustomobject] $SkipIfNodeNotOperational,
		[Parameter()]
		[ValidateSet('CREATE_NEW','USE_REFERENCE','NONE')]
		[string] $RoutingForwardProfileSelection,
		[Parameter()]
		[string] $RoutingForwardProfileDisplayName,
		[Parameter()]
		[ValidateSet($true,$false)]
		[pscustomobject] $AdjustRoutingForwardProfileSettings,
		[Parameter()]
		[ValidateSet('PASSTHROUGH','TRANSCODING','BYPASS','PASSTHROUGH_WITH_FALLBACK_TO_BYPASS')]
		[string] $RoutingForwardProfileMode,
		[Parameter()]
        [string] $RoutingForwardProfileId,
        [Parameter()]
        [ValidateSet('SUCCESS','DIAL_STRING','NETWORK_PERMISSION','NETWORK_CONGESTION','NETWORK_EQUIPMENT','USER_BUSY','USER_REDIRECTED','USER_NOT_RESPONDING','USER_NOT_SELECTED','USER_REJECTED','USER_TERMINATED','MEDIA_NEGOTIATION','ERROR','DOMAIN_SPECIFIC_0','DOMAIN_SPECIFIC_1','DOMAIN_SPECIFIC_2','DOMAIN_SPECIFIC_3','DOMAIN_SPECIFIC_4','DOMAIN_SPECIFIC_5','DOMAIN_SPECIFIC_6','DOMAIN_SPECIFIC_7','DOMAIN_SPECIFIC_8','DOMAIN_SPECIFIC_9','DOMAIN_SPECIFIC_10','DOMAIN_SPECIFIC_11','DOMAIN_SPECIFIC_12','DOMAIN_SPECIFIC_13','DOMAIN_SPECIFIC_14','DOMAIN_SPECIFIC_15','DOMAIN_SPECIFIC_16','DOMAIN_SPECIFIC_17','DOMAIN_SPECIFIC_18','DOMAIN_SPECIFIC_19')]
        [string] $RejectStatus,
        [Parameter()]
        [string] $ParallelCallDialStringDirectoryId,
        [Parameter()]
        [PSCustomObject] $SourceDialStringRewriting,
        [Parameter()]
        [PSCustomObject] $DestinationDialStringRewriting,
        [Parameter()]
        [Object[]] $ParallelCalls,
        [Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

    $Parameters=@{}
    $Parameters.Add("routingDomain",$RoutingDomain)
    $Parameters.Add("mode","establishment")

	$Body=@{}

    if (!($UseUnconditionalRouting -eq $null)) {
        $Body.Add("useUnconditionalRouting",$UseUnconditionalRouting)
    }

    if (!($UseRouteSupervision -eq $null)) {
        $Body.Add("useRouteSupervision",$UseRouteSupervision)
    }

    if (!($SupervisionId -eq $null)) {
        $Body.Add("supervisionId",$SupervisionId)
    }
    
    if (!($DisplayName -eq $null)) {
        $Body.Add("displayName",$DisplayName)
    }

    if (!($Filters -eq $null)) {
        $Body.Add("filters",$Filters)
    }
    
    $Establishment=@{}

    if (!($Type -eq $null)) {
        $Establishment.Add("type",$Type)
    }

    if (!($DestinationNodeId -eq $null)) {
        $Establishment.Add("destinationNodeId",$DestinationNodeId)
    }

    if (!($SkipIfNodeNotOperational -eq $null)) {
        $Establishment.Add("skipIfNodeNotOperational",$SkipIfNodeNotOperational)
    }

    if (!($RoutingForwardProfileSelection -eq $null)) {
        $Establishment.Add("routingForwardProfileSelection",$RoutingForwardProfileSelection)
    }

    if (!($RoutingForwardProfileDisplayName -eq $null)) {
        $Establishment.Add("routingForwardProfileDisplayName",$RoutingForwardProfileDisplayName)
    }

    if (!($AdjustRoutingForwardProfileSettings -eq $null)) {
        $Establishment.Add("adjustRoutingForwardProfileSettings",$AdjustRoutingForwardProfileSettings)
    }

    if (!($RoutingForwardProfileMode -eq $null)) {
        $Establishment.Add("routingForwardProfileMode",$RoutingForwardProfileMode)
    }

    if (!($RoutingForwardProfileId -eq $null)) {
        $Establishment.Add("routingForwardProfileId",$RoutingForwardProfileId)
    }

    if (!($RejectStatus -eq $null)) {
        $Establishment.Add("rejectStatus",$RejectStatus)
    }

    if (!($ParallelCallDialStringDirectoryId -eq $null)) {
        $Establishment.Add("parallelCallDialStringDirectoryId",$ParallelCallDialStringDirectoryId)
    }

    if (!($SourceDialStringRewriting -eq $null)) {
        $Establishment.Add("sourceDialStringRewriting",$SourceDialStringRewriting)
    }

    if (!($DestinationDialStringRewriting -eq $null)) {
        $Establishment.Add("destinationDialStringRewriting",$DestinationDialStringRewriting)
    }

    if (!($ParallelCalls -eq $null)) {
        $Establishment.Add("parallelCalls",$ParallelCalls)
    }

    $Body.Add("establishment",$Establishment)

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routes/add -Parameters $Parameters -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

function Add-anynodePathReplacementRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $RoutingDomain,
		[Parameter()]
		[ValidateSet($true,$false)]
        [pscustomobject] $UseUnconditionalRouting,
		[Parameter()]
		[ValidateSet($true,$false)]
        [pscustomobject] $UseRouteSupervision,
        [Parameter()]
        [string] $SupervisionId,
        [Parameter()]
        [string] $DisplayName,
        [Parameter()]
        [Object[]] $Filters,
        [Parameter()]
        [ValidateSet('REPLACE','REJECT')]
        [string] $Type,
        [Parameter()]
        [string] $DestinationNodeId,
        [Parameter()]
        [ValidateSet($true,$false)]
        [pscustomobject] $SkipIfNodeNotOperational,
        [Parameter()]
        [ValidateSet('CREATE_NEW','USE_REFERENCE','NONE')]
        [string] $RoutingForwardProfileSelection,
        [Parameter()]
        [string] $RoutingForwardProfileDisplayName,
        [Parameter()]
        [ValidateSet($true,$false)]
        [pscustomobject] $AdjustRoutingForwardProfileSettings,
        [Parameter()]
        [ValidateSet('PASSTHROUGH','TRANSCODING','BYPASS','PASSTHROUGH_WITH_FALLBACK_TO_BYPASS')]
        [string] $RoutingForwardProfileMode,
        [Parameter()]
        [string] $RoutingForwardProfileId,
        [Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

    $Parameters=@{}
    $Parameters.Add("routingDomain",$RoutingDomain)
    $Parameters.Add("mode","pathReplacement")

	$Body=@{}

    if (!($UseUnconditionalRouting -eq $null)) {
        $Body.Add("useUnconditionalRouting",$UseUnconditionalRouting)
    }

    if (!($UseRouteSupervision -eq $null)) {
        $Body.Add("useRouteSupervision",$UseRouteSupervision)
    }

    if (!($SupervisionId -eq $null)) {
        $Body.Add("supervisionId",$SupervisionId)
    }
    
    if (!($DisplayName -eq $null)) {
        $Body.Add("displayName",$DisplayName)
    }

    if (!($Filters -eq $null)) {
        $Body.Add("filters",$Filters)
    }

    $PathReplacement=@{}

    if (!($Type -eq $null)) {
        $PathReplacement.Add("type",$Type)
    }

    if (!($RoutingForwardProfileSelection -eq $null)) {
        $PathReplacement.Add("routingForwardProfileSelection",$RoutingForwardProfileSelection)
    }

    if (!($RoutingForwardProfileDisplayName -eq $null)) {
        $PathReplacement.Add("routingForwardProfileDisplayName",$RoutingForwardProfileDisplayName)
    }

    if (!($AdjustRoutingForwardProfileSettings -eq $null)) {
        $PathReplacement.Add("adjustRoutingForwardProfileSettings",$AdjustRoutingForwardProfileSettings)
    }

    if (!($RoutingForwardProfileMode -eq $null)) {
        $PathReplacement.Add("routingForwardProfileMode",$RoutingForwardProfileMode)
    }

    if (!($RoutingForwardProfileId -eq $null)) {
        $PathReplacement.Add("routingForwardProfileId",$RoutingForwardProfileId)
    }

    $Body.Add("pathReplacement",$PathReplacement)

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routes/add -Parameters $Parameters -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Set
function Set-anynodeEstablishmentRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $RoutingDomain,
        [Parameter()]
        [string] $Index,
        [Parameter()]
        [string] $RouteName,
		[Parameter()]
		[ValidateSet($true,$false)]
		[pscustomobject] $UseUnconditionalRouting,
		[Parameter()]
		[ValidateSet($true,$false)]
		[pscustomobject] $UseRouteSupervision,
		[Parameter()]
		[string] $SupervisionId,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[Object[]] $Filters,
		[Parameter()]
		[ValidateSet('ROUTE','IGNORE','REJECT','REDIRECT','ESTABLISH_PARALLEL')]
		[string] $Type,
		[Parameter()]
		[string] $DestinationNodeId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[pscustomobject] $SkipIfNodeNotOperational,
		[Parameter()]
		[ValidateSet('CREATE_NEW','USE_REFERENCE','NONE')]
		[string] $RoutingForwardProfileSelection,
		[Parameter()]
		[string] $RoutingForwardProfileDisplayName,
		[Parameter()]
		[ValidateSet($true,$false)]
		[pscustomobject] $AdjustRoutingForwardProfileSettings,
		[Parameter()]
		[ValidateSet('PASSTHROUGH','TRANSCODING','BYPASS','PASSTHROUGH_WITH_FALLBACK_TO_BYPASS')]
		[string] $RoutingForwardProfileMode,
		[Parameter()]
        [string] $RoutingForwardProfileId,
        [Parameter()]
        [ValidateSet('SUCCESS','DIAL_STRING','NETWORK_PERMISSION','NETWORK_CONGESTION','NETWORK_EQUIPMENT','USER_BUSY','USER_REDIRECTED','USER_NOT_RESPONDING','USER_NOT_SELECTED','USER_REJECTED','USER_TERMINATED','MEDIA_NEGOTIATION','ERROR','DOMAIN_SPECIFIC_0','DOMAIN_SPECIFIC_1','DOMAIN_SPECIFIC_2','DOMAIN_SPECIFIC_3','DOMAIN_SPECIFIC_4','DOMAIN_SPECIFIC_5','DOMAIN_SPECIFIC_6','DOMAIN_SPECIFIC_7','DOMAIN_SPECIFIC_8','DOMAIN_SPECIFIC_9','DOMAIN_SPECIFIC_10','DOMAIN_SPECIFIC_11','DOMAIN_SPECIFIC_12','DOMAIN_SPECIFIC_13','DOMAIN_SPECIFIC_14','DOMAIN_SPECIFIC_15','DOMAIN_SPECIFIC_16','DOMAIN_SPECIFIC_17','DOMAIN_SPECIFIC_18','DOMAIN_SPECIFIC_19')]
        [string] $RejectStatus,
        [Parameter()]
        [string] $ParallelCallDialStringDirectoryId,
        [Parameter()]
        [PSCustomObject] $SourceDialStringRewriting,
        [Parameter()]
        [PSCustomObject] $DestinationDialStringRewriting,
        [Parameter()]
        [Object[]] $ParallelCalls,
        [Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

    $Parameters=@{}
    $Parameters.Add("routingDomain",$RoutingDomain)
    $Parameters.Add("mode","establishment")

       if (!($Index -eq $null)) {
        $Parameters.Add("index",$Index)
    }

    if (!($RouteName -eq $null)) {
        $Parameters.Add("routeName",$RouteName)
    }

	$Body=@{}

    if (!($UseUnconditionalRouting -eq $null)) {
        $Body.Add("useUnconditionalRouting",$UseUnconditionalRouting)
    }

    if (!($UseRouteSupervision -eq $null)) {
        $Body.Add("useRouteSupervision",$UseRouteSupervision)
    }

    if (!($SupervisionId -eq $null)) {
        $Body.Add("supervisionId",$SupervisionId)
    }
    
    if (!($DisplayName -eq $null)) {
        $Body.Add("displayName",$DisplayName)
    }

    if (!($Filters -eq $null)) {
        $Body.Add("filters",$Filters)
    }
    
    $Establishment=@{}

    if (!($Type -eq $null)) {
        $Establishment.Add("type",$Type)
    }

    if (!($DestinationNodeId -eq $null)) {
        $Establishment.Add("destinationNodeId",$DestinationNodeId)
    }

    if (!($SkipIfNodeNotOperational -eq $null)) {
        $Establishment.Add("skipIfNodeNotOperational",$SkipIfNodeNotOperational)
    }

    if (!($RoutingForwardProfileSelection -eq $null)) {
        $Establishment.Add("routingForwardProfileSelection",$RoutingForwardProfileSelection)
    }

    if (!($RoutingForwardProfileDisplayName -eq $null)) {
        $Establishment.Add("routingForwardProfileDisplayName",$RoutingForwardProfileDisplayName)
    }

    if (!($AdjustRoutingForwardProfileSettings -eq $null)) {
        $Establishment.Add("adjustRoutingForwardProfileSettings",$AdjustRoutingForwardProfileSettings)
    }

    if (!($RoutingForwardProfileMode -eq $null)) {
        $Establishment.Add("routingForwardProfileMode",$RoutingForwardProfileMode)
    }

    if (!($RoutingForwardProfileId -eq $null)) {
        $Establishment.Add("routingForwardProfileId",$RoutingForwardProfileId)
    }

    if (!($RejectStatus -eq $null)) {
        $Establishment.Add("rejectStatus",$RejectStatus)
    }

    if (!($ParallelCallDialStringDirectoryId -eq $null)) {
        $Establishment.Add("parallelCallDialStringDirectoryId",$ParallelCallDialStringDirectoryId)
    }

    if (!($SourceDialStringRewriting -eq $null)) {
        $Establishment.Add("sourceDialStringRewriting",$SourceDialStringRewriting)
    }

    if (!($DestinationDialStringRewriting -eq $null)) {
        $Establishment.Add("destinationDialStringRewriting",$DestinationDialStringRewriting)
    }

    if (!($ParallelCalls -eq $null)) {
        $Establishment.Add("parallelCalls",$ParallelCalls)
    }

    $Body.Add("establishment",$Establishment)

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routes/set -Parameters $Parameters -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

function Set-anynodePathReplacementRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $RoutingDomain,
        [Parameter()]
        [string] $Index,
        [Parameter()]
        [string] $RouteName,
		[Parameter()]
		[ValidateSet($true,$false)]
        [pscustomobject] $UseUnconditionalRouting,
		[Parameter()]
		[ValidateSet($true,$false)]
        [pscustomobject] $UseRouteSupervision,
        [Parameter()]
        [string] $SupervisionId,
        [Parameter()]
        [string] $DisplayName,
        [Parameter()]
        [Object[]] $Filters,
        [Parameter()]
        [ValidateSet('REPLACE','REJECT')]
        [string] $Type,
        [Parameter()]
        [string] $DestinationNodeId,
        [Parameter()]
        [ValidateSet($true,$false)]
        [pscustomobject] $SkipIfNodeNotOperational,
        [Parameter()]
        [ValidateSet('CREATE_NEW','USE_REFERENCE','NONE')]
        [string] $RoutingForwardProfileSelection,
        [Parameter()]
        [string] $RoutingForwardProfileDisplayName,
        [Parameter()]
        [ValidateSet($true,$false)]
        [pscustomobject] $AdjustRoutingForwardProfileSettings,
        [Parameter()]
        [ValidateSet('PASSTHROUGH','TRANSCODING','BYPASS','PASSTHROUGH_WITH_FALLBACK_TO_BYPASS')]
        [string] $RoutingForwardProfileMode,
        [Parameter()]
        [string] $RoutingForwardProfileId,
        [Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

    $Parameters=@{}
    $Parameters.Add("routingDomain",$RoutingDomain)
    $Parameters.Add("mode","pathReplacement")

      if (!($Index -eq $null)) {
        $Parameters.Add("index",$Index)
    }

    if (!($RouteName -eq $null)) {
        $Parameters.Add("routeName",$RouteName)
    }


	$Body=@{}

    if (!($UseUnconditionalRouting -eq $null)) {
        $Body.Add("useUnconditionalRouting",$UseUnconditionalRouting)
    }

    if (!($UseRouteSupervision -eq $null)) {
        $Body.Add("useRouteSupervision",$UseRouteSupervision)
    }

    if (!($SupervisionId -eq $null)) {
        $Body.Add("supervisionId",$SupervisionId)
    }
    
    if (!($DisplayName -eq $null)) {
        $Body.Add("displayName",$DisplayName)
    }

    if (!($Filters -eq $null)) {
        $Body.Add("filters",$Filters)
    }

    $PathReplacement=@{}

    if (!($Type -eq $null)) {
        $PathReplacement.Add("type",$Type)
    }

    if (!($RoutingForwardProfileSelection -eq $null)) {
        $PathReplacement.Add("routingForwardProfileSelection",$RoutingForwardProfileSelection)
    }

    if (!($RoutingForwardProfileDisplayName -eq $null)) {
        $PathReplacement.Add("routingForwardProfileDisplayName",$RoutingForwardProfileDisplayName)
    }

    if (!($AdjustRoutingForwardProfileSettings -eq $null)) {
        $PathReplacement.Add("adjustRoutingForwardProfileSettings",$AdjustRoutingForwardProfileSettings)
    }

    if (!($RoutingForwardProfileMode -eq $null)) {
        $PathReplacement.Add("routingForwardProfileMode",$RoutingForwardProfileMode)
    }

    if (!($RoutingForwardProfileId -eq $null)) {
        $PathReplacement.Add("routingForwardProfileId",$RoutingForwardProfileId)
    }

    $Body.Add("pathReplacement",$PathReplacement)

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routes/set -Parameters $Parameters -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

#	Remove
function Remove-anynodeEstablishmentRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $RoutingDomain,
		[Parameter()]
		[int] $Index=-1,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{routingDomain=$RoutingDomain}
	$Parameters.Add("mode","establishment")

	if ($Index -gt -1) {
		$Parameters.Add("index",$Index)
	} elseif ($DisplayName -ne $null -AND $DisplayName -ne '') {
		$Parameters.Add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/routingDomains/routes/remove -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}

function Remove-anynodePathReplacementRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter(mandatory=$true)]
		[string] $RoutingDomain,
		[Parameter()]
		[int] $Index=-1,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[string] $BackendName,
		[Parameter()]
		[int] $APIVersion,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{routingDomain=$RoutingDomain}
	$Parameters.Add("mode","pathReplacement")

	if ($Index -gt -1) {
		$Parameters.Add("index",$Index)
	} elseif ($DisplayName -ne $null -AND $DisplayName -ne '') {
		$Parameters.Add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/routingDomains/routes/remove -Parameters $Parameters -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}




function Create-anynodeConfigBackup {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[psobject] $IncludedData,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $EncryptData,
		[Parameter()]
		[string] $ZipPassword,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filename,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($IncludedData -ne $null) {
		$Body.add("includedData",$IncludedData)
	}

	if ($EncryptData -ne $null) {
		$Body.add("encryptData",$EncryptData)
	}

	if ($ZipPassword -ne $null) {
		$Body.add("password",$ZipPassword)
	}

	Invoke-anynodeDownload -Address $Address -Port $Port -URITail /api/backup -Filename $Filename -Method Post -Credential $Credential -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeActiveSessions {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('ISO-8601','XZWO')]
		[string] $DateFormat,
		[Parameter()]
		[int] $Count,
		[Parameter()]
		[string] $NodeFilter,
		[Parameter()]
		[string] $RouteFilter,
		[Parameter()]
		[string] $NumberFilter,
		[Parameter()]
		[ValidateSet('equal','startswith','endswith','contains')]
		[string] $NumberFilterMode,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DateFormat -ne $null -and $DateFormat -ne '') {
		$Parameters.add("dateFormat",$DateFormat)
	}

	$Body=@{}

	if ($Count -ne $null) {
		$Body.add("count",$Count)
	}

	if ($NodeFilter -ne $null) {
		$Body.add("nodeFilter",$NodeFilter)
	}

	if ($RouteFilter -ne $null) {
		$Body.add("routeFilter",$RouteFilter)
	}

	if ($NumberFilter -ne $null) {
		$Body.add("numberFilter",$NumberFilter)
	}

	if ($NumberFilterMode -ne $null) {
		$Body.add("numberFilterMode",$NumberFilterMode)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/activeSessions/get -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeBackends {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/backends/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeConfig {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filename,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeDownload -Address $Address -Port $Port -URITail /api/anynodeConfig/get -Filename $Filename -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Import-anynodeConfig {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[switch] $CommitProtection,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filepath,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($CommitProtection -ne $null -and $CommitProtection -ne '') {
		$Parameters.add("commitProtection",$CommitProtection)
	}

	$Body=''
	try {
		$Body=[IO.File]::ReadAllText($Filepath)
	} catch {
		echo 'Could not find file.'
		return
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/anynodeConfig/set -Filenames @($Filepath) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Push-anynodeConfig {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/anynodeConfig/commit -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Remove-anynodeConfig {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	if (($Force -eq $null) -Or ($Force -eq $false)) {
		$Answer=Read-Host 'Are you sure you want to clean the whole anynode configuration? (y/n)'
	} elseif ($Force -eq $true) {
		$Answer='y'
	}

	if ($Answer -ne 'y') {
		return
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/anynodeConfig/clean -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeConfigState {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/anynodeConfig/status/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Reset-anynodeConfig {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/anynodeConfig/revert -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeDashboard {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('ISO-8601','XZWO')]
		[string] $DateFormat,
		[Parameter()]
		[ValidateSet('sipNodes','requestRouters','sfbUcmaNodes','ldapConnections','certificates','conditions','networkControllers','ldapDirectories','numberValidationRouteSupervision','azureActiveDirectories','directoryRouteSupervisions','directoryRouteSupervision','certificateIssuance','licensesInUse','licensesInstalled')]
		[string] $List,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DateFormat -ne $null -and $DateFormat -ne '') {
		$Parameters.add("dateFormat",$DateFormat)
	}

	if ($List -ne $null -and $List -ne '') {
		$Parameters.add("list",$List)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/dashboard/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeDialStringRewritings {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/dialStringRewriting/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeDialStringRewriting {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[psobject] $Rules,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($Id -ne $null) {
		$Body.add("id",$Id)
	}

	if ($Rules -ne $null) {
		$Body.add("rules",$Rules)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/dialStringRewriting/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeDialStringRewriting {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[psobject] $Rules,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($Id -ne $null) {
		$Body.add("id",$Id)
	}

	if ($Rules -ne $null) {
		$Body.add("rules",$Rules)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/dialStringRewriting/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Remove-anynodeDialStringRewriting {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/dialStringRewriting/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeDialStringRewritingRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/dialStringRewriting/rules/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeDialStringRewritingRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[psobject[]] $Rules,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	$Body=(ConvertTo-Json -InputObject @($Rules))

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/dialStringRewriting/rules/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Import-anynodeDialStringRewritingRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $Json,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filename,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Json -ne $null -and $Json -ne '') {
		$Parameters.add("json",$Json)
	}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/dialStringRewriting/rules/import -Filenames @($Filename) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Export-anynodeDialStringRewritingRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filename,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	Invoke-anynodeDownload -Address $Address -Port $Port -URITail /api/dialStringRewriting/rules/export -Filename $Filename -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeDialStringRewritingRule {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[ValidateSet('prefixAndSuffix','wildcardPattern','cutExtensionNumber','addExt','matchAndModify','matchAndBranch')]
		[string] $RewriteType,
		[Parameter()]
		[ValidateSet('anything','never','list','prefixSuffix','dialStringList','structuralPattern','wildcardPattern','extensionRange')]
		[string] $MatchType,
		[Parameter()]
		[string] $Prefix,
		[Parameter()]
		[string] $Suffix,
		[Parameter()]
		[string] $WildcardPattern,
		[Parameter()]
		[psobject] $MatchDirectory,
		[Parameter()]
		[psobject] $MatchExtensionRange,
		[Parameter()]
		[psobject] $StructuralPattern,
		[Parameter()]
		[int] $DeleteLeadingCharacters,
		[Parameter()]
		[int] $DeleteTrailingCharacters,
		[Parameter()]
		[string] $AddPrefix,
		[Parameter()]
		[string] $AddSuffix,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[psobject] $AddExtTrailingDigits,
		[Parameter()]
		[psobject] $RewriteSegments,
		[Parameter()]
		[psobject] $BranchRewrites,
		[Parameter()]
		[psobject] $BranchElseRewrites,
		[Parameter()]
		[psobject] $MatchTags,
		[Parameter()]
		[psobject] $TagRewrite,
		[Parameter()]
		[ValidateSet('unspecified','sessionSource','sessionAsserted','sessionDestination','sessionConnected','sessionElin','sessionRedirectHistory','sessionTransferrer','reasonRedirection','reasonSelected','mwiSource','mwiDestination')]
		[psobject] $ApplyTo,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ApplyToAllDialStrings,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ApplyToDestinationDialString,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ApplyToSourceDialString,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $SkipFurtherRules,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $SkipRulesBelow,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ClearDisplayName,
		[Parameter()]
		[ValidateSet('lowercase','uppercase','casefold')]
		[string] $Case,
		[Parameter()]
		[string] $Comment,
		[Parameter()]
		[int] $Index,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	$Body=@{}

	if ($RewriteType -ne $null) {
		$Body.add("telRewriteType",$RewriteType)
	}

	if ($MatchType -ne $null) {
		$Body.add("telMatchType",$MatchType)
	}

	if ($Prefix -ne $null) {
		$Body.add("prefix",$Prefix)
	}

	if ($Suffix -ne $null) {
		$Body.add("suffix",$Suffix)
	}

	if ($WildcardPattern -ne $null) {
		$Body.add("wildcardPattern",$WildcardPattern)
	}

	if ($MatchDirectory -ne $null) {
		$Body.add("matchDirectory",$MatchDirectory)
	}

	if ($MatchExtensionRange -ne $null) {
		$Body.add("matchExtensionRange",$MatchExtensionRange)
	}

	if ($StructuralPattern -ne $null) {
		$Body.add("structuralPattern",$StructuralPattern)
	}

	if ($DeleteLeadingCharacters -ne $null) {
		$Body.add("deleteLeadingCharacters",$DeleteLeadingCharacters)
	}

	if ($DeleteTrailingCharacters -ne $null) {
		$Body.add("deleteTrailingCharacters",$DeleteTrailingCharacters)
	}

	if ($AddPrefix -ne $null) {
		$Body.add("addPrefix",$AddPrefix)
	}

	if ($AddSuffix -ne $null) {
		$Body.add("addSuffix",$AddSuffix)
	}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($AddExtTrailingDigits -ne $null) {
		$Body.add("addExtDigits",$AddExtTrailingDigits)
	}

	if ($RewriteSegments -ne $null) {
		$Body.add("rewriteSegments",$RewriteSegments)
	}

	if ($BranchRewrites -ne $null) {
		$Body.add("branchRewrites",$BranchRewrites)
	}

	if ($BranchElseRewrites -ne $null) {
		$Body.add("branchElseRewrites",$BranchElseRewrites)
	}

	if ($MatchTags -ne $null) {
		$Body.add("matchTags",$MatchTags)
	}

	if ($TagRewrite -ne $null) {
		$Body.add("tagRewrite",$TagRewrite)
	}

	if ($ApplyTo -ne $null) {
		$Body.add("applyTo",$ApplyTo)
	}

	if ($ApplyToAllDialStrings -ne $null) {
		$Body.add("applyToAllDialStrings",$ApplyToAllDialStrings)
	}

	if ($ApplyToDestinationDialString -ne $null) {
		$Body.add("applyToDestinationDialString",$ApplyToDestinationDialString)
	}

	if ($ApplyToSourceDialString -ne $null) {
		$Body.add("applyToSourceDialString",$ApplyToSourceDialString)
	}

	if ($SkipFurtherRules -ne $null) {
		$Body.add("skipFurtherRules",$SkipFurtherRules)
	}

	if ($SkipRulesBelow -ne $null) {
		$Body.add("breakValue",$SkipRulesBelow)
	}

	if ($ClearDisplayName -ne $null) {
		$Body.add("clearDisplayName",$ClearDisplayName)
	}

	if ($Case -ne $null) {
		$Body.add("rewriteCase",$Case)
	}

	if ($Comment -ne $null) {
		$Body.add("comment",$Comment)
	}

	if ($Index -ne $null) {
		$Body.add("index",$Index)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/dialStringRewriting/rules/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeDialStringRewritingRule {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Index,
		[Parameter()]
		[string] $Rule,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Index -ne $null -and $Index -ne '') {
		$Parameters.add("index",$Index)
	}

	if ($Rule -ne $null -and $Rule -ne '') {
		$Parameters.add("rule",$Rule)
	}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/dialStringRewriting/rules/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Clear-anynodeDialStringRewritingRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/dialStringRewriting/rules/clear -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeDialStringRewriting {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DialStringRewriting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DialStringRewriting -ne $null -and $DialStringRewriting -ne '') {
		$Parameters.add("dialStringRewriting",$DialStringRewriting)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/dialStringRewriting/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Search-anynodeForwardedSessionRecordingIdentifier {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $SdpMLineAddress,
		[Parameter()]
		[int] $SdpMLinePort,
		[Parameter()]
		[string] $SdpOrigin,
		[Parameter()]
		[string] $SipCallId,
		[Parameter()]
		[string] $SipLocalTag,
		[Parameter()]
		[string] $SipRemoteTag,
		[Parameter()]
		[string] $SipUserAgentDialog,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($SdpMLineAddress -ne $null) {
		$Body.add("sdpMLineAddress",$SdpMLineAddress)
	}

	if ($SdpMLinePort -ne $null) {
		$Body.add("sdpMLinePort",$SdpMLinePort)
	}

	if ($SdpOrigin -ne $null) {
		$Body.add("sdpOrigin",$SdpOrigin)
	}

	if ($SipCallId -ne $null) {
		$Body.add("sipCallId",$SipCallId)
	}

	if ($SipLocalTag -ne $null) {
		$Body.add("sipLocalTag",$SipLocalTag)
	}

	if ($SipRemoteTag -ne $null) {
		$Body.add("sipRemoteTag",$SipRemoteTag)
	}

	if ($SipUserAgentDialog -ne $null) {
		$Body.add("sipUserAgentDialog",$SipUserAgentDialog)
	}

	if (!$ParamPresent) {
		echo 'At least one filter parameter has to be given.'
		return
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/forwardedSessionRecording/lookupByLocalSessionIdentification -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeForwardedSessionRecordingMute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Identifier,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $Mute,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Identifier -ne $null -and $Identifier -ne '') {
		$Parameters.add("identifier",$Identifier)
	}

	if ($Mute -ne $null -and $Mute -ne '') {
		$Parameters.add("mute",$Mute)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/forwardedSessionRecording/setMute -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeLicenses {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/licenses/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Clear-anynodeLicenses {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/licenses/clear -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Import-anynodeLicense {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filepath,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=''
	try {
		$Body=[IO.File]::ReadAllText($Filepath)
	} catch {
		echo 'Could not find file.'
		return
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/licenses/importFile -Filenames @($Filepath) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Import-anynodeLOD {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $CompanyName,
		[Parameter()]
		[string] $EmailAddress,
		[Parameter()]
		[string] $LicensingDevice,
		[Parameter()]
		[string] $LodKey,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($CompanyName -ne $null) {
		$Body.add("companyName",$CompanyName)
	}

	if ($EmailAddress -ne $null) {
		$Body.add("emailAddress",$EmailAddress)
	}

	if ($LicensingDevice -ne $null) {
		$Body.add("licensingDevice",$LicensingDevice)
	}

	if ($LodKey -ne $null) {
		$Body.add("lodKey",$LodKey)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/licenses/importLod -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMaintenanceMode {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/maintenanceMode/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMaintenanceMode {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ModeActive,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($ModeActive -ne $null) {
		$Body.add("modeActive",$ModeActive)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/maintenanceMode/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMediaRecording {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/mediaRecording/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeNetworkControllers {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/networkControllers/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeNetworkInterfaces {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/networkInterfaces/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeNodes {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Enable-anynodeNode {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/enable -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Disable-anynodeNode {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/disable -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeProviderNodeProfiles {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[switch] $CompleteData,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($CompleteData -ne $null -and $CompleteData -ne '') {
		$Parameters.add("completeData",$CompleteData)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/providerProfiles/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeProviderNode {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Profile,
		[Parameter()]
		[psobject] $DeutscheTelekomNumberExtensionRange,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $CreateNewDeutscheTelekomSipTrunk,
		[Parameter()]
		[string] $DeutscheTelekomSipTrunkId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $CreateNewNetworkController,
		[Parameter()]
		[string] $NetworkControllerDisplayName,
		[Parameter()]
		[string] $NetworkControllerInterface,
		[Parameter()]
		[ValidateSet('[Any Address Version]','IN_ADDRESS_VERSION_4','IN_ADDRESS_VERSION_6')]
		[string] $NetworkControllerIpVersion,
		[Parameter()]
		[string] $NetworkControllerIpAddress,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $EnableReverseDnsLookup,
		[Parameter()]
		[string] $NetworkControllerId,
		[Parameter()]
		[int] $UdpTcpPort,
		[Parameter()]
		[int] $TlsPort,
		[Parameter()]
		[psobject] $UdpPortRange,
		[Parameter()]
		[psobject] $TcpPortRange,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ActivateNatTraversal,
		[Parameter()]
		[string] $ExternalHost,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $NatDeviceHasFixedPortMapping,
		[Parameter()]
		[int] $FixedUdpTcpPort,
		[Parameter()]
		[int] $FixedTlsPort,
		[Parameter()]
		[int] $FixedUdpPortRangeStart,
		[Parameter()]
		[int] $FixedTcpPortRangeStart,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $NodeViaSipTrunking,
		[Parameter()]
		[psobject] $RemoteSipDomain,
		[Parameter()]
		[string] $LocalSipDomain,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ActivateClientAuthentication,
		[Parameter()]
		[psobject] $ClientCredentials,
		[Parameter()]
		[string] $RegistrarUri,
		[Parameter()]
		[string] $AddressOfRecord,
		[Parameter()]
		[string] $ProxyUri,
		[Parameter()]
		[string] $PrimaryProxyUri,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $DefineAssertedUri,
		[Parameter()]
		[string] $AssertedUriHost,
		[Parameter()]
		[string] $AssertedUri,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $UseNetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $IncludeRemoteSipDomainInWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $IncludeSipRegistrarInWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $IncludeProxyInWhitelist,
		[Parameter()]
		[psobject] $NetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $AllowOnlyNegotiatedPeersForRtpRtcp,
		[Parameter()]
		[psobject] $IncomingDialStringRewriteRules,
		[Parameter()]
		[psobject] $OutgoingDialStringRewriteRules,
		[Parameter()]
		[ValidateSet('Schweiz','Liechtenstein','sipcall CH','sipcall AT')]
		[string] $Region,
		[Parameter()]
		[psobject] $Telstra1thTrunkGroup,
		[Parameter()]
		[psobject] $Telstra2ndTrunkGroup,
		[Parameter()]
		[psobject] $RoutingDomains,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=2,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Profile -ne $null -and $Profile -ne '') {
		$Parameters.add("profile",$Profile)
	}

	$Body=@{}

	if ($DeutscheTelekomNumberExtensionRange -ne $null) {
		$Body.add("deutscheTelekomNumberExtensionRange",$DeutscheTelekomNumberExtensionRange)
	}

	if ($CreateNewDeutscheTelekomSipTrunk -ne $null) {
		$Body.add("createNewDeutscheTelekomSipTrunk",$CreateNewDeutscheTelekomSipTrunk)
	}

	if ($DeutscheTelekomSipTrunkId -ne $null) {
		$Body.add("deutscheTelekomSipTrunkId",$DeutscheTelekomSipTrunkId)
	}

	if ($CreateNewNetworkController -ne $null) {
		$Body.add("createNewNetworkController",$CreateNewNetworkController)
	}

	if ($NetworkControllerDisplayName -ne $null) {
		$Body.add("networkControllerDisplayName",$NetworkControllerDisplayName)
	}

	if ($NetworkControllerInterface -ne $null) {
		$Body.add("networkControllerInterface",$NetworkControllerInterface)
	}

	if ($NetworkControllerIpVersion -ne $null) {
		$Body.add("networkControllerIpVersion",$NetworkControllerIpVersion)
	}

	if ($NetworkControllerIpAddress -ne $null) {
		$Body.add("networkControllerIpAddress",$NetworkControllerIpAddress)
	}

	if ($EnableReverseDnsLookup -ne $null) {
		$Body.add("enableReverseDnsLookup",$EnableReverseDnsLookup)
	}

	if ($NetworkControllerId -ne $null) {
		$Body.add("networkControllerId",$NetworkControllerId)
	}

	if ($UdpTcpPort -ne $null) {
		$Body.add("udpTcpPort",$UdpTcpPort)
	}

	if ($TlsPort -ne $null) {
		$Body.add("tlsPort",$TlsPort)
	}

	if ($UdpPortRange -ne $null) {
		$Body.add("udpPortRange",$UdpPortRange)
	}

	if ($TcpPortRange -ne $null) {
		$Body.add("tcpPortRange",$TcpPortRange)
	}

	if ($ActivateNatTraversal -ne $null) {
		$Body.add("activateNatTraversal",$ActivateNatTraversal)
	}

	if ($ExternalHost -ne $null) {
		$Body.add("externalHost",$ExternalHost)
	}

	if ($NatDeviceHasFixedPortMapping -ne $null) {
		$Body.add("natDeviceHasFixedPortMapping",$NatDeviceHasFixedPortMapping)
	}

	if ($FixedUdpTcpPort -ne $null) {
		$Body.add("fixedUdpTcpPort",$FixedUdpTcpPort)
	}

	if ($FixedTlsPort -ne $null) {
		$Body.add("fixedTlsPort",$FixedTlsPort)
	}

	if ($FixedUdpPortRangeStart -ne $null) {
		$Body.add("fixedUdpPortRangeStart",$FixedUdpPortRangeStart)
	}

	if ($FixedTcpPortRangeStart -ne $null) {
		$Body.add("fixedTcpPortRangeStart",$FixedTcpPortRangeStart)
	}

	if ($NodeViaSipTrunking -ne $null) {
		$Body.add("nodeViaSipTrunking",$NodeViaSipTrunking)
	}

	if ($RemoteSipDomain -ne $null) {
		$Body.add("remoteSipDomain",$RemoteSipDomain)
	}

	if ($LocalSipDomain -ne $null) {
		$Body.add("localSipDomain",$LocalSipDomain)
	}

	if ($ActivateClientAuthentication -ne $null) {
		$Body.add("activateClientAuthentication",$ActivateClientAuthentication)
	}

	if ($ClientCredentials -ne $null) {
		$Body.add("credentials",$ClientCredentials)
	}

	if ($RegistrarUri -ne $null) {
		$Body.add("registrarUri",$RegistrarUri)
	}

	if ($AddressOfRecord -ne $null) {
		$Body.add("addressOfRecord",$AddressOfRecord)
	}

	if ($ProxyUri -ne $null) {
		$Body.add("proxyUri",$ProxyUri)
	}

	if ($PrimaryProxyUri -ne $null) {
		$Body.add("primaryProxyUri",$PrimaryProxyUri)
	}

	if ($DefineAssertedUri -ne $null) {
		$Body.add("defineAssertedUri",$DefineAssertedUri)
	}

	if ($AssertedUriHost -ne $null) {
		$Body.add("assertedUriHost",$AssertedUriHost)
	}

	if ($AssertedUri -ne $null) {
		$Body.add("assertedUri",$AssertedUri)
	}

	if ($UseNetworkPeerWhitelist -ne $null) {
		$Body.add("useNetworkPeerWhitelist",$UseNetworkPeerWhitelist)
	}

	if ($IncludeRemoteSipDomainInWhitelist -ne $null) {
		$Body.add("includeRemoteSipDomainInWhitelist",$IncludeRemoteSipDomainInWhitelist)
	}

	if ($IncludeSipRegistrarInWhitelist -ne $null) {
		$Body.add("includeSipRegistrarInWhitelist",$IncludeSipRegistrarInWhitelist)
	}

	if ($IncludeProxyInWhitelist -ne $null) {
		$Body.add("includeProxyInWhitelist",$IncludeProxyInWhitelist)
	}

	if ($NetworkPeerWhitelist -ne $null) {
		$Body.add("networkPeerWhitelist",$NetworkPeerWhitelist)
	}

	if ($AllowOnlyNegotiatedPeersForRtpRtcp -ne $null) {
		$Body.add("allowOnlyNegotiatedPeersForRtpRtcp",$AllowOnlyNegotiatedPeersForRtpRtcp)
	}

	if ($IncomingDialStringRewriteRules -ne $null) {
		$Body.add("incomingDialStringRewriteRules",$IncomingDialStringRewriteRules)
	}

	if ($OutgoingDialStringRewriteRules -ne $null) {
		$Body.add("outgoingDialStringRewriteRules",$OutgoingDialStringRewriteRules)
	}

	if ($Region -ne $null) {
		$Body.add("region",$Region)
	}

	if ($Telstra1thTrunkGroup -ne $null) {
		$Body.add("telstra1thTrunkGroup",$Telstra1thTrunkGroup)
	}

	if ($Telstra2ndTrunkGroup -ne $null) {
		$Body.add("telstra2ndTrunkGroup",$Telstra2ndTrunkGroup)
	}

	if ($RoutingDomains -ne $null) {
		$Body.add("routingDomains",$RoutingDomains)
	}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/addProvider -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeSystemNodeProfiles {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[switch] $CompleteData,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($CompleteData -ne $null -and $CompleteData -ne '') {
		$Parameters.add("completeData",$CompleteData)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/systemProfiles/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeSipPhoneRegistrar {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $CreateNewNetworkController,
		[Parameter()]
		[string] $NetworkControllerDisplayName,
		[Parameter()]
		[string] $NetworkControllerInterface,
		[Parameter()]
		[ValidateSet('[Any Address Version]','IN_ADDRESS_VERSION_4','IN_ADDRESS_VERSION_6')]
		[string] $NetworkControllerIpVersion,
		[Parameter()]
		[string] $NetworkControllerIpAddress,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $EnableReverseDnsLookup,
		[Parameter()]
		[string] $NetworkControllerId,
		[Parameter()]
		[int] $UdpTcpPort,
		[Parameter()]
		[int] $TlsPort,
		[Parameter()]
		[psobject] $UdpPortRange,
		[Parameter()]
		[psobject] $TcpPortRange,
		[Parameter()]
		[string] $SipPhoneRegistrarDirectoryId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $UseNetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $IncludeOwnSubnetInWhitelist,
		[Parameter()]
		[psobject] $NetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $AllowOnlyNegotiatedPeersForRtpRtcp,
		[Parameter()]
		[psobject] $IncomingDialStringRewriteRules,
		[Parameter()]
		[psobject] $OutgoingDialStringRewriteRules,
		[Parameter()]
		[psobject] $RoutingDomains,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($CreateNewNetworkController -ne $null) {
		$Body.add("createNewNetworkController",$CreateNewNetworkController)
	}

	if ($NetworkControllerDisplayName -ne $null) {
		$Body.add("networkControllerDisplayName",$NetworkControllerDisplayName)
	}

	if ($NetworkControllerInterface -ne $null) {
		$Body.add("networkControllerInterface",$NetworkControllerInterface)
	}

	if ($NetworkControllerIpVersion -ne $null) {
		$Body.add("networkControllerIpVersion",$NetworkControllerIpVersion)
	}

	if ($NetworkControllerIpAddress -ne $null) {
		$Body.add("networkControllerIpAddress",$NetworkControllerIpAddress)
	}

	if ($EnableReverseDnsLookup -ne $null) {
		$Body.add("enableReverseDnsLookup",$EnableReverseDnsLookup)
	}

	if ($NetworkControllerId -ne $null) {
		$Body.add("networkControllerId",$NetworkControllerId)
	}

	if ($UdpTcpPort -ne $null) {
		$Body.add("udpTcpPort",$UdpTcpPort)
	}

	if ($TlsPort -ne $null) {
		$Body.add("tlsPort",$TlsPort)
	}

	if ($UdpPortRange -ne $null) {
		$Body.add("udpPortRange",$UdpPortRange)
	}

	if ($TcpPortRange -ne $null) {
		$Body.add("tcpPortRange",$TcpPortRange)
	}

	if ($SipPhoneRegistrarDirectoryId -ne $null) {
		$Body.add("sipPhoneRegistrarDirectoryId",$SipPhoneRegistrarDirectoryId)
	}

	if ($UseNetworkPeerWhitelist -ne $null) {
		$Body.add("useNetworkPeerWhitelist",$UseNetworkPeerWhitelist)
	}

	if ($IncludeOwnSubnetInWhitelist -ne $null) {
		$Body.add("includeOwnSubnetInWhitelist",$IncludeOwnSubnetInWhitelist)
	}

	if ($NetworkPeerWhitelist -ne $null) {
		$Body.add("networkPeerWhitelist",$NetworkPeerWhitelist)
	}

	if ($AllowOnlyNegotiatedPeersForRtpRtcp -ne $null) {
		$Body.add("allowOnlyNegotiatedPeersForRtpRtcp",$AllowOnlyNegotiatedPeersForRtpRtcp)
	}

	if ($IncomingDialStringRewriteRules -ne $null) {
		$Body.add("incomingDialStringRewriteRules",$IncomingDialStringRewriteRules)
	}

	if ($OutgoingDialStringRewriteRules -ne $null) {
		$Body.add("outgoingDialStringRewriteRules",$OutgoingDialStringRewriteRules)
	}

	if ($RoutingDomains -ne $null) {
		$Body.add("routingDomains",$RoutingDomains)
	}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/addSipPhoneRegistrar -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Add-anynodeSystemNode {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Profile,
		[Parameter()]
		[psobject] $SipTransport,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $CreateNewNetworkController,
		[Parameter()]
		[string] $NetworkControllerDisplayName,
		[Parameter()]
		[string] $NetworkControllerInterface,
		[Parameter()]
		[ValidateSet('[Any Address Version]','IN_ADDRESS_VERSION_4','IN_ADDRESS_VERSION_6')]
		[string] $NetworkControllerIpVersion,
		[Parameter()]
		[string] $NetworkControllerIpAddress,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $EnableReverseDnsLookup,
		[Parameter()]
		[string] $NetworkControllerId,
		[Parameter()]
		[int] $UdpTcpPort,
		[Parameter()]
		[int] $TlsPort,
		[Parameter()]
		[psobject] $UdpPortRange,
		[Parameter()]
		[psobject] $TcpPortRange,
		[Parameter()]
		[ValidateSet('REGISTRAR','REGISTRATION','TRUNK')]
		[string] $SipInterconnection,
		[Parameter()]
		[psobject] $RemoteSipDomain,
		[Parameter()]
		[string] $LocalSipDomain,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ActivateClientAuthentication,
		[Parameter()]
		[psobject] $ClientCredentials,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ActivateServerAuthentication,
		[Parameter()]
		[psobject] $ServerCredentials,
		[Parameter()]
		[string] $RegistrarUri,
		[Parameter()]
		[string] $AddressOfRecord,
		[Parameter()]
		[psobject] $SipRegistrar,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $UseNetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $IncludeRemoteSipDomainInWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $IncludeSipRegistrarInWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $IncludeOwnSubnetInWhitelist,
		[Parameter()]
		[psobject] $NetworkPeerWhitelist,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $AllowOnlyNegotiatedPeersForRtpRtcp,
		[Parameter()]
		[psobject] $SipNodeNetworkPeerWhitelist,
		[Parameter()]
		[psobject] $IncomingDialStringRewriteRules,
		[Parameter()]
		[psobject] $OutgoingDialStringRewriteRules,
		[Parameter()]
		[psobject] $RoutingDomains,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $CreateNewTeamsCarrierNode,
		[Parameter()]
		[string] $TeamsCarrierNodeId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $CreateNewMsOperatorConnectNode,
		[Parameter()]
		[string] $MsOperatorConnectNodeId,
		[Parameter()]
		[string] $TenantId,
		[Parameter()]
		[string] $SbcFqdn,
		[Parameter()]
		[string] $TenantFqdn,
		[Parameter()]
		[ValidateSet('North America','EMEA','Australia','APAC','LATAM','China','Japan')]
		[string] $Region,
		[Parameter()]
		[string] $XGooglePbxTrunkSecretKey,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=3,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Profile -ne $null -and $Profile -ne '') {
		$Parameters.add("profile",$Profile)
	}

	$Body=@{}

	if ($SipTransport -ne $null) {
		$Body.add("sipTransport",$SipTransport)
	}

	if ($CreateNewNetworkController -ne $null) {
		$Body.add("createNewNetworkController",$CreateNewNetworkController)
	}

	if ($NetworkControllerDisplayName -ne $null) {
		$Body.add("networkControllerDisplayName",$NetworkControllerDisplayName)
	}

	if ($NetworkControllerInterface -ne $null) {
		$Body.add("networkControllerInterface",$NetworkControllerInterface)
	}

	if ($NetworkControllerIpVersion -ne $null) {
		$Body.add("networkControllerIpVersion",$NetworkControllerIpVersion)
	}

	if ($NetworkControllerIpAddress -ne $null) {
		$Body.add("networkControllerIpAddress",$NetworkControllerIpAddress)
	}

	if ($EnableReverseDnsLookup -ne $null) {
		$Body.add("enableReverseDnsLookup",$EnableReverseDnsLookup)
	}

	if ($NetworkControllerId -ne $null) {
		$Body.add("networkControllerId",$NetworkControllerId)
	}

	if ($UdpTcpPort -ne $null) {
		$Body.add("udpTcpPort",$UdpTcpPort)
	}

	if ($TlsPort -ne $null) {
		$Body.add("tlsPort",$TlsPort)
	}

	if ($UdpPortRange -ne $null) {
		$Body.add("udpPortRange",$UdpPortRange)
	}

	if ($TcpPortRange -ne $null) {
		$Body.add("tcpPortRange",$TcpPortRange)
	}

	if ($SipInterconnection -ne $null) {
		$Body.add("sipInterconnection",$SipInterconnection)
	}

	if ($RemoteSipDomain -ne $null) {
		$Body.add("remoteSipDomain",$RemoteSipDomain)
	}

	if ($LocalSipDomain -ne $null) {
		$Body.add("localSipDomain",$LocalSipDomain)
	}

	if ($ActivateClientAuthentication -ne $null) {
		$Body.add("activateClientAuthentication",$ActivateClientAuthentication)
	}

	if ($ClientCredentials -ne $null) {
		$Body.add("clientCredentials",$ClientCredentials)
	}

	if ($ActivateServerAuthentication -ne $null) {
		$Body.add("activateServerAuthentication",$ActivateServerAuthentication)
	}

	if ($ServerCredentials -ne $null) {
		$Body.add("serverCredentials",$ServerCredentials)
	}

	if ($RegistrarUri -ne $null) {
		$Body.add("registrarUri",$RegistrarUri)
	}

	if ($AddressOfRecord -ne $null) {
		$Body.add("addressOfRecord",$AddressOfRecord)
	}

	if ($SipRegistrar -ne $null) {
		$Body.add("sipRegistrar",$SipRegistrar)
	}

	if ($UseNetworkPeerWhitelist -ne $null) {
		$Body.add("useNetworkPeerWhitelist",$UseNetworkPeerWhitelist)
	}

	if ($IncludeRemoteSipDomainInWhitelist -ne $null) {
		$Body.add("includeRemoteSipDomainInWhitelist",$IncludeRemoteSipDomainInWhitelist)
	}

	if ($IncludeSipRegistrarInWhitelist -ne $null) {
		$Body.add("includeSipRegistrarInWhitelist",$IncludeSipRegistrarInWhitelist)
	}

	if ($IncludeOwnSubnetInWhitelist -ne $null) {
		$Body.add("includeOwnSubnetInWhitelist",$IncludeOwnSubnetInWhitelist)
	}

	if ($NetworkPeerWhitelist -ne $null) {
		$Body.add("networkPeerWhitelist",$NetworkPeerWhitelist)
	}

	if ($AllowOnlyNegotiatedPeersForRtpRtcp -ne $null) {
		$Body.add("allowOnlyNegotiatedPeersForRtpRtcp",$AllowOnlyNegotiatedPeersForRtpRtcp)
	}

	if ($SipNodeNetworkPeerWhitelist -ne $null) {
		$Body.add("sipNodeNetworkPeerWhitelist",$SipNodeNetworkPeerWhitelist)
	}

	if ($IncomingDialStringRewriteRules -ne $null) {
		$Body.add("incomingDialStringRewriteRules",$IncomingDialStringRewriteRules)
	}

	if ($OutgoingDialStringRewriteRules -ne $null) {
		$Body.add("outgoingDialStringRewriteRules",$OutgoingDialStringRewriteRules)
	}

	if ($RoutingDomains -ne $null) {
		$Body.add("routingDomains",$RoutingDomains)
	}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($CreateNewTeamsCarrierNode -ne $null) {
		$Body.add("createNewTeamsCarrierNode",$CreateNewTeamsCarrierNode)
	}

	if ($TeamsCarrierNodeId -ne $null) {
		$Body.add("teamsCarrierNodeId",$TeamsCarrierNodeId)
	}

	if ($CreateNewMsOperatorConnectNode -ne $null) {
		$Body.add("createNewMsOperatorConnectNode",$CreateNewMsOperatorConnectNode)
	}

	if ($MsOperatorConnectNodeId -ne $null) {
		$Body.add("msOperatorConnectNodeId",$MsOperatorConnectNodeId)
	}

	if ($TenantId -ne $null) {
		$Body.add("tenantId",$TenantId)
	}

	if ($SbcFqdn -ne $null) {
		$Body.add("sbcFqdn",$SbcFqdn)
	}

	if ($TenantFqdn -ne $null) {
		$Body.add("tenantFqdn",$TenantFqdn)
	}

	if ($Region -ne $null) {
		$Body.add("region",$Region)
	}

	if ($XGooglePbxTrunkSecretKey -ne $null) {
		$Body.add("xGooglePbxTrunkSecretKey",$XGooglePbxTrunkSecretKey)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/addSystem -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeIncomingDialStringRewriteRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $IncludeDefaults,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	if ($IncludeDefaults -ne $null -and $IncludeDefaults -ne '') {
		$Parameters.add("includeDefaults",$IncludeDefaults)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/incomingDialStringRewriteRules/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodeIncomingDialStringRewriteRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[psobject[]] $DialStringRewriteRules,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=(ConvertTo-Json -InputObject @($DialStringRewriteRules))

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/incomingDialStringRewriteRules/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeNodeIncomingDialStringRewriteRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[psobject[]] $DialStringRewriteRules,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=(ConvertTo-Json -InputObject @($DialStringRewriteRules))

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/incomingDialStringRewriteRules/add -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeNodeIncomingDialStringRewriteRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $Index,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	if ($Index -ne $null -and $Index -ne '') {
		$Parameters.add("index",$Index)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/incomingDialStringRewriteRules/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeOutgoingDialStringRewriteRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/outgoingDialStringRewriteRules/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodeOutgoingDialStringRewriteRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[psobject[]] $DialStringRewriteRules,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=(ConvertTo-Json -InputObject @($DialStringRewriteRules))

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/outgoingDialStringRewriteRules/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeNodeOutgoingDialStringRewriteRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[psobject[]] $DialStringRewriteRules,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=(ConvertTo-Json -InputObject @($DialStringRewriteRules))

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/outgoingDialStringRewriteRules/add -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeNodeOutgoingDialStringRewriteRules {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $Index,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	if ($Index -ne $null -and $Index -ne '') {
		$Parameters.add("index",$Index)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/outgoingDialStringRewriteRules/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeNetworkPeerWhitelist {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/networkPeerWhitelist/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodeNetworkPeerWhitelist {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[psobject[]] $NetworkPeerWhitelist,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=(ConvertTo-Json -InputObject @($NetworkPeerWhitelist))

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/networkPeerWhitelist/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeNodeNetworkPeerWhitelistEntry {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[psobject[]] $NetworkPeerWhitelist,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=(ConvertTo-Json -InputObject @($NetworkPeerWhitelist))

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/networkPeerWhitelist/add -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeNodeNetworkPeerWhitelistEntry {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[psobject] $Hostname,
		[Parameter()]
		[string] $Index,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	if ($Hostname -ne $null -and $Hostname -ne '') {
		$Parameters.add("hostname",$Hostname)
	}

	if ($Index -ne $null -and $Index -ne '') {
		$Parameters.add("index",$Index)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/networkPeerWhitelist/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeNode {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[ValidateSet($true,$false)]
		[switch] $Force,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	if ($Force -ne $null -and $Force -ne '') {
		$Parameters.add("force",$Force)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeMsTeamsCarriers {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/msTeamsCarriers/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeNodeCertificate {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/certificates/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodeCertificate {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('CERTIFICATES_AND_PRIVATE_KEY','CERTIFICATES_ONLY','PRIVATE_KEY_ONLY')]
		[string] $Mode,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $CertificatePassword,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $CertificateFilePath,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Mode -ne $null -and $Mode -ne '') {
		$Parameters.add("mode",$Mode)
	}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	if ($CertificatePassword -ne $null -and $CertificatePassword -ne '') {
		$Parameters.add("password",$CertificatePassword)
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/nodes/certificates/set -Filenames @($CertificateFilePath) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeChainCertificates {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/certificateChain/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodeChainCertificates {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $CertificateFile,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/nodes/certificateChain/set -Filenames @($CertificateFile) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeNodeChainCertificate {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $CertificateFilePath,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/nodes/certificateChain/add -Filenames @($CertificateFilePath) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeNodeChainCertificate {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/certificateChain/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Clear-anynodeNodeChainCertificates {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/certificateChain/clear -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Request-anynodeNodeChainCertificates {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/certificateChain/request -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeTrustedCertificates {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/trustedCertificates/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeNodeTrustedCertificate {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $CertificatePassword,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $CertificateFilePath,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	if ($CertificatePassword -ne $null -and $CertificatePassword -ne '') {
		$Parameters.add("password",$CertificatePassword)
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/nodes/trustedCertificates/add -Filenames @($CertificateFilePath) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Clear-anynodeNodeTrustedCertificates {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/trustedCertificates/clear -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeFQDN {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/tenants/fqdn/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodeFQDN {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $NewFQDN,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=$NewFQDN

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/tenants/fqdn/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeOverrideHostname {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/overrideHostname/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodeOverrideHostname {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $NewOverrideHostname,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=$NewOverrideHostname

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/overrideHostname/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodePort {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/port/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodePort {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $NewPort,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=$NewPort

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/port/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeTlsPort {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/tlsPort/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodeTlsPort {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $NewPort,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=$NewPort

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/tlsPort/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeNetworkSecurityProfilename {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/networkSecurityProfileName/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-aynodeNetworkSecurityProfileName {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $NetworkSecurityProfileId,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=$NetworkSecurityProfileId

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/nodes/networkSecurityProfileName/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeNodeTransportConnections {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/nodes/transportConnections/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeNodeTransportConnection {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[psobject] $MatchingCondition,
		[Parameter()]
		[string] $TransportConnectionId,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=@{}

	if ($MatchingCondition -ne $null) {
		$Body.add("matchingCondition",$MatchingCondition)
	}

	if ($TransportConnectionId -ne $null) {
		$Body.add("transportConnectionId",$TransportConnectionId)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/transportConnections/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeNodeTransportConnections {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[psobject[]] $TransportConnections,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	$Body=(ConvertTo-Json -InputObject @($TransportConnections))

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/nodes/transportConnections/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeNodeTransportConnections {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $Index,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	if ($Index -ne $null -and $Index -ne '') {
		$Parameters.add("index",$Index)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/transportConnections/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Clear-anynodeNodeTransportConnections {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Node,
		[Parameter()]
		[string] $RemoveRoutes,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Node -ne $null -and $Node -ne '') {
		$Parameters.add("node",$Node)
	}

	if ($RemoveRoutes -ne $null -and $RemoveRoutes -ne '') {
		$Parameters.add("removeRoutes",$RemoveRoutes)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/nodes/transportConnections/clear -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeTransportConnection {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/transportConnections/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeTransportConnectionAuthentication {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $TransportConnection,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($TransportConnection -ne $null -and $TransportConnection -ne '') {
		$Parameters.add("transportConnection",$TransportConnection)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/transportConnections/authentication/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeTransportConnectionAuthentication {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $TransportConnection,
		[Parameter()]
		[string] $TransportConnectionPassword,
		[Parameter()]
		[string] $TransportConnectionUsername,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($TransportConnection -ne $null -and $TransportConnection -ne '') {
		$Parameters.add("transportConnection",$TransportConnection)
	}

	$Body=@{}

	if ($TransportConnectionPassword -ne $null) {
		$Body.add("password",$TransportConnectionPassword)
	}

	if ($TransportConnectionUsername -ne $null) {
		$Body.add("username",$TransportConnectionUsername)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/transportConnections/authentication/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeTransportConnectionAssertedURI {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $TransportConnection,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($TransportConnection -ne $null -and $TransportConnection -ne '') {
		$Parameters.add("transportConnection",$TransportConnection)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/transportConnections/assertedUri/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeTransportConnectionAssertedURI {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $TransportConnection,
		[Parameter()]
		[string] $AssertedDisplayName,
		[Parameter()]
		[string] $URI,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($TransportConnection -ne $null -and $TransportConnection -ne '') {
		$Parameters.add("transportConnection",$TransportConnection)
	}

	$Body=@{}

	if ($AssertedDisplayName -ne $null) {
		$Body.add("displayName",$AssertedDisplayName)
	}

	if ($URI -ne $null) {
		$Body.add("iri",$URI)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/transportConnections/assertedUri/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeTransportConnectionAddressOfRecord {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $TransportConnection,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($TransportConnection -ne $null -and $TransportConnection -ne '') {
		$Parameters.add("transportConnection",$TransportConnection)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/transportConnections/addressOfRecord/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeTransportConnectionAddressOfRecord {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $TransportConnection,
		[Parameter()]
		[string] $AssertedDisplayName,
		[Parameter()]
		[string] $URI,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($TransportConnection -ne $null -and $TransportConnection -ne '') {
		$Parameters.add("transportConnection",$TransportConnection)
	}

	$Body=@{}

	if ($AssertedDisplayName -ne $null) {
		$Body.add("displayName",$AssertedDisplayName)
	}

	if ($URI -ne $null) {
		$Body.add("iri",$URI)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/transportConnections/assertedUri/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeStandardTransportConnection {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $AssertedDisplayName,
		[Parameter()]
		[string] $AssertedURI,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $AuthenticationEnabled,
		[Parameter()]
		[string] $ConditionId,
		[Parameter()]
		[psobject] $AuthenticationPolicy,
		[Parameter()]
		[psobject] $Proxy,
		[Parameter()]
		[string] $AddressOfRecord,
		[Parameter()]
		[string] $SipDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	$Body=@{}

	if ($AssertedDisplayName -ne $null) {
		$Body.add("displayName",$AssertedDisplayName)
	}

	if ($AssertedURI -ne $null) {
		$Body.add("iri",$AssertedURI)
	}

	if ($AuthenticationEnabled -ne $null) {
		$Body.add("authenticationEnabled",$AuthenticationEnabled)
	}

	if ($ConditionId -ne $null) {
		$Body.add("conditionId",$ConditionId)
	}

	if ($AuthenticationPolicy -ne $null) {
		$Body.add("credentials",$AuthenticationPolicy)
	}

	if ($Proxy -ne $null) {
		$Body.add("proxy",$Proxy)
	}

	if ($AddressOfRecord -ne $null) {
		$Body.add("addressOfRecord",$AddressOfRecord)
	}

	if ($SipDomain -ne $null) {
		$Body.add("sipDomain",$SipDomain)
	}

	$AssertedAddress=@{}

	if ($AssertedDisplayName -ne $null) {
		$AssertedAddress.Add('AssertedDisplayName',$AssertedDisplayName)
	}

	if ($AssertedURI -ne $null) {
		$AssertedAddress.Add('AssertedURI',$AssertedURI)
	}

	if ($AssertedAddress -ne $null -and '' -ne $AssertedAddress) {
		$Body.Add('assertedAddress',$AssertedAddress)
	}

	$Registration=@{}

	if ($AddressOfRecord -ne $null) {
		$Registration.Add('addressOfRecord',$AddressOfRecord)
	}

	if ($SipDomain -ne $null) {
		$Registration.Add('sipDomain',$SipDomain)
	}

	if ($Registration -ne $null -and '' -ne $Registration) {
		$Body.Add('registration',$Registration)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/transportConnections/addStandard -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeLoadBalancingTransportConnection {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[ValidateSet('FAILOVER','SEQUENTIAL_FAILOVER','REDIRECT_INVITE','REDIRECT_NOTIFY','REDIRECT_OPTIONS','REDIRECT_REFER','REDIRECT_REGISTER')]
		[psobject] $Properties,
		[Parameter()]
		[psobject] $Targets,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	$Body=@{}

	if ($Properties -ne $null) {
		$Body.add("properties",$Properties)
	}

	if ($Targets -ne $null) {
		$Body.add("targets",$Targets)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/transportConnections/addSipLoadBalancing -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodePlainTransportConnection {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[psobject] $Uri,
		[Parameter()]
		[psobject] $AssertedUri,
		[Parameter()]
		[ValidateSet('FLOW','FLOW_PRIVATE','OPTIONS','OPTIONS_LOAD','OPTIONS_DOWN_ONLY','OPTIONS_FLOW_ADDRESS','LYNC_MASTER_DOMAIN_NAME','LYNC_INCOMING')]
		[psobject] $Properties,
		[Parameter()]
		[psobject] $InitialRouteSet,
		[Parameter()]
		[string] $SipTransport,
		[Parameter()]
		[ValidateSet('DOWN','TERMINATE_INCOMING','TERMINATE_OUTGOING')]
		[psobject] $OptionsErrorFlags,
		[Parameter()]
		[ValidateSet('DOWN','FAILOVER','TERMINATE_INCOMING','TERMINATE_OUTGOING')]
		[psobject] $TimeoutErrorFlags,
		[Parameter()]
		[ValidateSet('DOWN','FAILOVER','TERMINATE_INCOMING','TERMINATE_OUTGOING')]
		[psobject] $TransportErrorFlags,
		[Parameter()]
		[psobject] $StatusCodesErrorFlags,
		[Parameter()]
		[psobject] $Authentication,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	$Body=@{}

	if ($Uri -ne $null) {
		$Body.add("uri",$Uri)
	}

	if ($AssertedUri -ne $null) {
		$Body.add("assertedUri",$AssertedUri)
	}

	if ($Properties -ne $null) {
		$Body.add("properties",$Properties)
	}

	if ($InitialRouteSet -ne $null) {
		$Body.add("initialRouteSet",$InitialRouteSet)
	}

	if ($SipTransport -ne $null) {
		$Body.add("sipTransport",$SipTransport)
	}

	if ($OptionsErrorFlags -ne $null) {
		$Body.add("optionsErrorFlags",$OptionsErrorFlags)
	}

	if ($TimeoutErrorFlags -ne $null) {
		$Body.add("timeoutErrorFlags",$TimeoutErrorFlags)
	}

	if ($TransportErrorFlags -ne $null) {
		$Body.add("transportErrorFlags",$TransportErrorFlags)
	}

	if ($StatusCodesErrorFlags -ne $null) {
		$Body.add("statusCodesErrorFlags",$StatusCodesErrorFlags)
	}

	if ($Authentication -ne $null) {
		$Body.add("authentication",$Authentication)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/transportConnections/addPlain -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeSipRegistrationAsTransportConnection {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $Registrar,
		[Parameter()]
		[psobject] $Authentication,
		[Parameter()]
		[psobject] $AorUri,
		[Parameter()]
		[psobject] $AssertedUri,
		[Parameter()]
		[string] $SipUserAgentStackId,
		[Parameter()]
		[string] $InnerTransportConnectionId,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	$Body=@{}

	if ($Registrar -ne $null) {
		$Body.add("registrar",$Registrar)
	}

	if ($Authentication -ne $null) {
		$Body.add("authentication",$Authentication)
	}

	if ($AorUri -ne $null) {
		$Body.add("aorUri",$AorUri)
	}

	if ($AssertedUri -ne $null) {
		$Body.add("assertedUri",$AssertedUri)
	}

	if ($SipUserAgentStackId -ne $null) {
		$Body.add("sipUserAgentStackId",$SipUserAgentStackId)
	}

	if ($InnerTransportConnectionId -ne $null) {
		$Body.add("innerTransportConnectionId",$InnerTransportConnectionId)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/transportConnections/addSipRegistrar -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeSrvLoadBalancer {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $SipTransportId,
		[Parameter()]
		[string] $Domain,
		[Parameter()]
		[ValidateSet('UDP','TCP','TLS','SCTP','SCTP_TLS')]
		[string] $TransportProtocol,
		[Parameter()]
		[psobject] $TargetTemplateOptions,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	$Body=@{}

	if ($SipTransportId -ne $null) {
		$Body.add("sipTransportId",$SipTransportId)
	}

	if ($Domain -ne $null) {
		$Body.add("domain",$Domain)
	}

	if ($TransportProtocol -ne $null) {
		$Body.add("transportProtocol",$TransportProtocol)
	}

	if ($TargetTemplateOptions -ne $null) {
		$Body.add("targetTemplateOptions",$TargetTemplateOptions)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/transportConnections/addSrvLoadBalancer -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeTransportConnection {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $TransportConnection,
		[Parameter()]
		[string] $Force,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($TransportConnection -ne $null -and $TransportConnection -ne '') {
		$Parameters.add("transportConnection",$TransportConnection)
	}

	if ($Force -ne $null -and $Force -ne '') {
		$Parameters.add("force",$Force)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/transportConnections/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeObject {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/objects/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeObjects {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[ValidateSet('USRAGG_DIRECTORY (Aggregate Directory)','AN_SIPLB_STACK (Load Balancer)','AN_SIPRT_ROUTE (Standard Transport Connection)','AZURE_ACTIVE_DIRECTORY (Azure Active Directory)','CAPIC_STACK (CAPI Client)','CAPIS_STACK (XCAPI Controller)','CAPIS_DOMAIN (XCAPI Controller Domain)','CS_CONDITION (Condition)','CS_SCHEDULER (Rate Limiter)','CS_STUB (Stub)','USRDB_DIRECTORY (Database Connected Directory)','CRY_X509_STACK (Digital Identity and Trust)','USRRT_ROUTE_SV (Directory Route Supervision)','MISC_TIME_REPORTER (Time Range)','MEDIA_PUMP_DOMAIN (Media Transcoding Options)','HTTP_SERVER (HTTP Server)','HTTP_CLIENT (HTTP Client)','IN_FILTER (Network Peer Whitelist)','IN_MAP_STACK (Network Mapping)','IN_MAP_STATIC_STACK (Network Static Mapping)','IN_STACK (Network Controller)','IPC_CLIENT (IPC Client)','IPC_SERVER (IPC Server)','LDAP_CONNECTION (LDAP Client Connection)','USRLDAP_DIRECTORY (LDAP Directory)','MNS_TEAMS_DOMAIN (Media Negotiation (Microsoft Teams Extensions))','MNS_FORWARDER_DOMAIN (Media Negotiation Forwarder)','MNS_STACK (Media Negotiation)','REC_STACK (Media Recorder)','RECMUX_STACK (Media Recording Multiplexer)','RECFILE_STACK (Media File Recorder)','RECTEL_STACK (Media Telephony Recorder)','MS_AUDIO_PROVIDER (Audio Media Source)','MSFILE_PROVIDER (File Media Source)','MS_GRAPH_CLIENT (Microsoft Graph Client)','TELTEAMS_STACK (Microsoft Teams Connectivity)','TELTEAMS_TENANT (Microsoft Teams Tenant)','INS_STACK (Network Security Profile)','NUMVALRT_ROUTE_SV (Number Validation Route Supervision)','OAUTH_CLIENT (OAuth Client)','PROVISIONING_SERVER (Provisioning Server)','IN_QOS_STACK (QoS Profile)','RESTRT_ROUTE_SV (REST Client Route Supervision)','SIPAUTH_STACK (Authentication Profile)','SIPLB_STACK (Load Balancing Transport Connection)','SIPPX_STACK (Request Router)','RECSIPREC_STACK (SIPREC Media Recorder)','TELSIPREG_REGISTRAR (SIP Registrar)','SIPRT_PLAIN_ROUTE (Plain Transport Connection)','SIPST_STACK (SIP Transport)','SIPUA_REGISTRATION (SIP Registration)','SIPUA_STACK (SIP User Agent)','SRVLB_STACK (SRV Load Balancer)','USRTEL_DIRECTORY (Static Dial string Directory)','USRT_DIRECTORY (Static User Directory)','MSTEL_PROVIDER (Telephony Media Source)','TELELIN_DOMAIN (ELIN Binding Database)','TELFW_STACK (Telephony Forwarding)','TEL_REWRITE_DOMAIN (Dial String Rewriting)','TELRT_FORWARDER_DOMAIN (Routing Forward Profile)','TELRT_STACK (Routing Domain)','TEL_STACK (Node)','TELCAPIC_STACK (CAPI Client Node)','TELCAPIS_STACK (XCAPI Controller Node)','TELSIP_STACK (SIP Node)','TELUCMA_STACK (SfB UCMA Node)','TELWEBRTC_STACK (WebRTC Node)','WEBRTC_STACK (WebRTC Server)','WEBSOCKET_STACK (Websocket)')]
		[string] $Sort,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	if ($Sort -ne $null -and $Sort -ne '') {
		$Parameters.add("sort",$Sort)
	}

	if ($Parameters.Sort -eq $null -AND $Parameters.DisplayName -eq $null) {
        do {
		    $p1=Read-Host 'Choose a filter: sort (s)/display name (d)/both (b)'
        } while ($p1 -eq $null)

        switch ($p1) {
            's'{
                $Parameters.Add('sort',(Read-Host 'Sort'))
            }
            'd' {
                $Parameters.Add('displayName',(Read-Host 'Display Name'))
            }
            'b' {
                $Parameters.Add('sort',(Read-Host 'Sort'))
                $Parameters.Add('displayName',(Read-Host 'Display Name'))
            }
            default {
                echo 'Invalid input. Needs to be "s" or "d" or "b".'
                return
            }
        }
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/objects/sorts -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeUnreferencedObjects {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Sort,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Sort -ne $null -and $Sort -ne '') {
		$Parameters.add("sort",$Sort)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/objects/unreferenced/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeObjectTree {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/objects/objectTree/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeSubTreeObjects {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[ValidateSet('USRAGG_DIRECTORY (Aggregate Directory)','AN_SIPLB_STACK (Load Balancer)','AN_SIPRT_ROUTE (Standard Transport Connection)','AZURE_ACTIVE_DIRECTORY (Azure Active Directory)','CAPIC_STACK (CAPI Client)','CAPIS_STACK (XCAPI Controller)','CAPIS_DOMAIN (XCAPI Controller Domain)','CS_CONDITION (Condition)','CS_SCHEDULER (Rate Limiter)','CS_STUB (Stub)','USRDB_DIRECTORY (Database Connected Directory)','CRY_X509_STACK (Digital Identity and Trust)','USRRT_ROUTE_SV (Directory Route Supervision)','MISC_TIME_REPORTER (Time Range)','MEDIA_PUMP_DOMAIN (Media Transcoding Options)','HTTP_SERVER (HTTP Server)','HTTP_CLIENT (HTTP Client)','IN_FILTER (Network Peer Whitelist)','IN_MAP_STACK (Network Mapping)','IN_MAP_STATIC_STACK (Network Static Mapping)','IN_STACK (Network Controller)','IPC_CLIENT (IPC Client)','IPC_SERVER (IPC Server)','LDAP_CONNECTION (LDAP Client Connection)','USRLDAP_DIRECTORY (LDAP Directory)','MNS_TEAMS_DOMAIN (Media Negotiation (Microsoft Teams Extensions))','MNS_FORWARDER_DOMAIN (Media Negotiation Forwarder)','MNS_STACK (Media Negotiation)','REC_STACK (Media Recorder)','RECMUX_STACK (Media Recording Multiplexer)','RECFILE_STACK (Media File Recorder)','RECTEL_STACK (Media Telephony Recorder)','MS_AUDIO_PROVIDER (Audio Media Source)','MSFILE_PROVIDER (File Media Source)','MS_GRAPH_CLIENT (Microsoft Graph Client)','TELTEAMS_STACK (Microsoft Teams Connectivity)','TELTEAMS_TENANT (Microsoft Teams Tenant)','INS_STACK (Network Security Profile)','NUMVALRT_ROUTE_SV (Number Validation Route Supervision)','OAUTH_CLIENT (OAuth Client)','PROVISIONING_SERVER (Provisioning Server)','IN_QOS_STACK (QoS Profile)','RESTRT_ROUTE_SV (REST Client Route Supervision)','SIPAUTH_STACK (Authentication Profile)','SIPLB_STACK (Load Balancing Transport Connection)','SIPPX_STACK (Request Router)','RECSIPREC_STACK (SIPREC Media Recorder)','TELSIPREG_REGISTRAR (SIP Registrar)','SIPRT_PLAIN_ROUTE (Plain Transport Connection)','SIPST_STACK (SIP Transport)','SIPUA_REGISTRATION (SIP Registration)','SIPUA_STACK (SIP User Agent)','SRVLB_STACK (SRV Load Balancer)','USRTEL_DIRECTORY (Static Dial string Directory)','USRT_DIRECTORY (Static User Directory)','MSTEL_PROVIDER (Telephony Media Source)','TELELIN_DOMAIN (ELIN Binding Database)','TELFW_STACK (Telephony Forwarding)','TEL_REWRITE_DOMAIN (Dial String Rewriting)','TELRT_FORWARDER_DOMAIN (Routing Forward Profile)','TELRT_STACK (Routing Domain)','TEL_STACK (Node)','TELCAPIC_STACK (CAPI Client Node)','TELCAPIS_STACK (XCAPI Controller Node)','TELSIP_STACK (SIP Node)','TELUCMA_STACK (SfB UCMA Node)','TELWEBRTC_STACK (WebRTC Node)','WEBRTC_STACK (WebRTC Server)','WEBSOCKET_STACK (Websocket)')]
		[string] $Sort,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	if ($Sort -ne $null -and $Sort -ne '') {
		$Parameters.add("sort",$Sort)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/objects/subTreeObjects/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeObject {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[psobject] $Object,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	$Body=(ConvertTo-Json $Object)

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/objects/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeObject {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/objects/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeObjects {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/objects/bulkRemove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Import-anynodeObject {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $CloneComment,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $ReassignPortOnCollision,
		[Parameter()]
		[ValidateSet('SKIP','OVERRIDE','CLONE','CANCEL')]
		[string] $DuplicateOption,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filepath,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($CloneComment -ne $null -and $CloneComment -ne '') {
		$Parameters.add("cloneComment",$CloneComment)
	}

	if ($ReassignPortOnCollision -ne $null -and $ReassignPortOnCollision -ne '') {
		$Parameters.add("reassignPortOnCollision",$ReassignPortOnCollision)
	}

	if ($DuplicateOption -ne $null -and $DuplicateOption -ne '') {
		$Parameters.add("action",$DuplicateOption)
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/objects/import -Filenames @($Filepath) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Export-anynodeObject {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $IncludePrivateData,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filename,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($IncludePrivateData -ne $null -and $IncludePrivateData -ne '') {
		$Parameters.add("includePrivateData",$IncludePrivateData)
	}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeDownload -Address $Address -Port $Port -URITail /api/objects/export -Filename $Filename -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeResources {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('disk','cpu','ram')]
		[string] $Value,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Value -ne $null -and $Value -ne '') {
		$Parameters.add("value",$Value)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/resources/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeRoutingDomains {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/routingDomains/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeRoutingDomainProceedingSignal {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/routingDomains/proceedingSignal/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Enable-anynodeRoutingDomainProceedingSignal {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/proceedingSignal/enable -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Disable-anynodeRoutingDomainProceedingSignal {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/proceedingSignal/disable -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeRoutingDomainRouteInMaintenance {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/routingDomains/routeInMaintenance/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Enable-anynodeRoutingDomainRouteInMaintenance {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routeInMaintenance/enable -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Disable-anynodeRoutingDomainRouteInMaintenance {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routeInMaintenance/disable -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeRoutingDomain {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[psobject] $SourceNodes,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=1,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($SourceNodes -ne $null) {
		$Body.add("sourceNodes",$SourceNodes)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeRoutingDomain {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[psobject] $SourceNodes,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=1,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($SourceNodes -ne $null) {
		$Body.add("sourceNodes",$SourceNodes)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeRoutingDomain {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/routingDomains/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeRoutingDomainSourceNodes {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/routingDomains/sourceNodes/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeRoutingDomainSourceNodes {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[psobject[]] $SourceNodeIds,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=1,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	$Body=(ConvertTo-Json -InputObject @($SourceNodeIds))

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/sourceNodes/add -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeRoutingDomainSourceNodes {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[psobject[]] $SourceNodeIds,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	$Body=(ConvertTo-Json -InputObject @($SourceNodeIds))

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/sourceNodes/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeRoutingDomainSourceNodes {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[psobject[]] $sourceNodeIds,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=2,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	$Body=(ConvertTo-Json -InputObject @($sourceNodeIds))

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/routingDomains/sourceNodes/remove -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeRoutes {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=1,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/routingDomains/routes/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Copy-anynodeRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('establishment','pathReplacement')]
		[string] $Mode,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $Index,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Mode -ne $null -and $Mode -ne '') {
		$Parameters.add("mode",$Mode)
	}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	if ($Index -ne $null -and $Index -ne '') {
		$Parameters.add("index",$Index)
	}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routes/clone -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('establishment','pathReplacement')]
		[string] $Mode,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[psobject] $Establishment,
		[Parameter()]
		[psobject] $Filters,
		[Parameter()]
		[psobject] $PathReplacement,
		[Parameter()]
		[string] $SupervisionId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $UseRouteSupervision,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $UseUnconditionalRouting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Mode -ne $null -and $Mode -ne '') {
		$Parameters.add("mode",$Mode)
	}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($Establishment -ne $null) {
		$Body.add("establishment",$Establishment)
	}

	if ($Filters -ne $null) {
		$Body.add("filters",$Filters)
	}

	if ($PathReplacement -ne $null) {
		$Body.add("pathReplacement",$PathReplacement)
	}

	if ($SupervisionId -ne $null) {
		$Body.add("supervisionId",$SupervisionId)
	}

	if ($UseRouteSupervision -ne $null) {
		$Body.add("useRouteSupervision",$UseRouteSupervision)
	}

	if ($UseUnconditionalRouting -ne $null) {
		$Body.add("useUnconditionalRouting",$UseUnconditionalRouting)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routes/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeRouteV1 {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('establishment','pathReplacement')]
		[string] $Mode,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[psobject] $Establishment,
		[Parameter()]
		[psobject] $Filters,
		[Parameter()]
		[psobject] $PathReplacement,
		[Parameter()]
		[string] $SupervisionId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $UseRouteSupervision,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $UseUnconditionalRouting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=1,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Mode -ne $null -and $Mode -ne '') {
		$Parameters.add("mode",$Mode)
	}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($Establishment -ne $null) {
		$Body.add("establishment",$Establishment)
	}

	if ($Filters -ne $null) {
		$Body.add("filters",$Filters)
	}

	if ($PathReplacement -ne $null) {
		$Body.add("pathReplacement",$PathReplacement)
	}

	if ($SupervisionId -ne $null) {
		$Body.add("supervisionId",$SupervisionId)
	}

	if ($UseRouteSupervision -ne $null) {
		$Body.add("useRouteSupervision",$UseRouteSupervision)
	}

	if ($UseUnconditionalRouting -ne $null) {
		$Body.add("useUnconditionalRouting",$UseUnconditionalRouting)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routes/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('establishment','pathReplacement')]
		[string] $Mode,
		[Parameter()]
		[string] $RoutingDomain,
		[Parameter()]
		[string] $Index,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[psobject] $Establishment,
		[Parameter()]
		[psobject] $Filters,
		[Parameter()]
		[psobject] $PathReplacement,
		[Parameter()]
		[string] $SupervisionId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $UseRouteSupervision,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $UseUnconditionalRouting,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=1,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Mode -ne $null -and $Mode -ne '') {
		$Parameters.add("mode",$Mode)
	}

	if ($RoutingDomain -ne $null -and $RoutingDomain -ne '') {
		$Parameters.add("routingDomain",$RoutingDomain)
	}

	if ($Index -ne $null -and $Index -ne '') {
		$Parameters.add("index",$Index)
	}

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($Establishment -ne $null) {
		$Body.add("establishment",$Establishment)
	}

	if ($Filters -ne $null) {
		$Body.add("filters",$Filters)
	}

	if ($PathReplacement -ne $null) {
		$Body.add("pathReplacement",$PathReplacement)
	}

	if ($SupervisionId -ne $null) {
		$Body.add("supervisionId",$SupervisionId)
	}

	if ($UseRouteSupervision -ne $null) {
		$Body.add("useRouteSupervision",$UseRouteSupervision)
	}

	if ($UseUnconditionalRouting -ne $null) {
		$Body.add("useUnconditionalRouting",$UseUnconditionalRouting)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingDomains/routes/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeRoute {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/routingDomains/routes/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeRoutingForwardProfiles {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[switch] $CompleteData,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($CompleteData -ne $null -and $CompleteData -ne '') {
		$Parameters.add("completeData",$CompleteData)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/routingForwardProfiles/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeRoutingForwardProfile {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $AnnouncementIncoming,
		[Parameter()]
		[string] $AnnouncementMediaSourceId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $AnnouncementOutgoing,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $EarlyMediaPassThrough,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[ValidateSet('TRANSCODING','BYPASS','PASSTHROUGH','PASSTHROUGH_WITH_FALLBACK_TO_BYPASS')]
		[string] $MediaMode,
		[Parameter()]
		[string] $MediaNegotiationForwarderId,
		[Parameter()]
		[string] $MediaRecorderId,
		[Parameter()]
		[string] $MediaTranscodingOptionsId,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $RemoveAnynodeFromSignalingPath,
		[Parameter()]
		[psobject] $SignalingForwardingFromCalledToCalling,
		[Parameter()]
		[psobject] $SignalingForwardingFromCallingToCalled,
		[Parameter()]
		[string] $TelephonyForwardingStackId,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($AnnouncementIncoming -ne $null) {
		$Body.add("announcementIncoming",$AnnouncementIncoming)
	}

	if ($AnnouncementMediaSourceId -ne $null) {
		$Body.add("announcementMediaSourceId",$AnnouncementMediaSourceId)
	}

	if ($AnnouncementOutgoing -ne $null) {
		$Body.add("announcementOutgoing",$AnnouncementOutgoing)
	}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($EarlyMediaPassThrough -ne $null) {
		$Body.add("earlyMediaPassThrough",$EarlyMediaPassThrough)
	}

	if ($Id -ne $null) {
		$Body.add("id",$Id)
	}

	if ($MediaMode -ne $null) {
		$Body.add("mediaMode",$MediaMode)
	}

	if ($MediaNegotiationForwarderId -ne $null) {
		$Body.add("mediaNegotiationForwarderId",$MediaNegotiationForwarderId)
	}

	if ($MediaRecorderId -ne $null) {
		$Body.add("mediaRecorderId",$MediaRecorderId)
	}

	if ($MediaTranscodingOptionsId -ne $null) {
		$Body.add("mediaTranscodingOptionsId",$MediaTranscodingOptionsId)
	}

	if ($RemoveAnynodeFromSignalingPath -ne $null) {
		$Body.add("removeAnynodeFromSignalingPath",$RemoveAnynodeFromSignalingPath)
	}

	if ($SignalingForwardingFromCalledToCalling -ne $null) {
		$Body.add("signalingForwardingFromCalledToCalling",$SignalingForwardingFromCalledToCalling)
	}

	if ($SignalingForwardingFromCallingToCalled -ne $null) {
		$Body.add("signalingForwardingFromCallingToCalled",$SignalingForwardingFromCallingToCalled)
	}

	if ($TelephonyForwardingStackId -ne $null) {
		$Body.add("telephonyForwardingStackId",$TelephonyForwardingStackId)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/routingForwardProfiles/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeRoles {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/roles/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Add-anynodeRole {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[ValidateSet('NO_ACCESS','ACCESS')]
		[string] $Administration,
		[Parameter()]
		[ValidateSet('NO_ACCESS','ACCESS')]
		[string] $Api,
		[Parameter()]
		[ValidateSet('NO_ACCESS','READ_ACCESS','READ_WRITE_ACCESS')]
		[string] $Backends,
		[Parameter()]
		[ValidateSet('NO_ACCESS','READ_ACCESS','READ_WRITE_ACCESS')]
		[string] $ConfigurationMode,
		[Parameter()]
		[ValidateSet('NO_ACCESS','ACCESS')]
		[string] $MediaRecording,
		[Parameter()]
		[ValidateSet('NO_ACCESS','READ_ACCESS','READ_WRITE_ACCESS')]
		[string] $MonitoringMode,
		[Parameter()]
		[ValidateSet('NO_ACCESS','ACCESS')]
		[string] $Tracing,
		[Parameter()]
		[ValidateSet('NO_ACCESS','ACCESS')]
		[string] $UserInterface,
		[Parameter()]
		[ValidateSet('NO_ACCESS','READ_ACCESS','READ_WRITE_ACCESS')]
		[string] $UserManagement,
		[Parameter()]
		[ValidateSet('NO_ACCESS','READ_ACCESS','READ_WRITE_ACCESS')]
		[string] $WebServer,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($Id -ne $null) {
		$Body.add("id",$Id)
	}

	if ($Administration -ne $null) {
		$Body.add("administration",$Administration)
	}

	if ($Api -ne $null) {
		$Body.add("api",$Api)
	}

	if ($Backends -ne $null) {
		$Body.add("backends",$Backends)
	}

	if ($ConfigurationMode -ne $null) {
		$Body.add("configurationMode",$ConfigurationMode)
	}

	if ($MediaRecording -ne $null) {
		$Body.add("mediaRecording",$MediaRecording)
	}

	if ($MonitoringMode -ne $null) {
		$Body.add("monitoringMode",$MonitoringMode)
	}

	if ($Tracing -ne $null) {
		$Body.add("tracing",$Tracing)
	}

	if ($UserInterface -ne $null) {
		$Body.add("userInterface",$UserInterface)
	}

	if ($UserManagement -ne $null) {
		$Body.add("userManagement",$UserManagement)
	}

	if ($WebServer -ne $null) {
		$Body.add("webServer",$WebServer)
	}

	$Permissions=@{}

	if ($Administration -ne $null) {
		$Permissions.Add('administration',$Administration)
	}

	if ($Api -ne $null) {
		$Permissions.Add('api',$Api)
	}

	if ($Backends -ne $null) {
		$Permissions.Add('backends',$Backends)
	}

	if ($ConfigurationMode -ne $null) {
		$Permissions.Add('configurationMode',$ConfigurationMode)
	}

	if ($MediaRecording -ne $null) {
		$Permissions.Add('mediaRecording',$MediaRecording)
	}

	if ($MonitoringMode -ne $null) {
		$Permissions.Add('monitoringMode',$MonitoringMode)
	}

	if ($Tracing -ne $null) {
		$Permissions.Add('tracing',$Tracing)
	}

	if ($UserInterface -ne $null) {
		$Permissions.Add('userInterface',$UserInterface)
	}

	if ($UserManagement -ne $null) {
		$Permissions.Add('userManagement',$UserManagement)
	}

	if ($WebServer -ne $null) {
		$Permissions.Add('webServer',$WebServer)
	}

	if ($Permissions -ne $null -and '' -ne $Permissions) {
		$Body.Add('permissions',$Permissions)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/roles/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Remove-anynodeRole {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Role,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Role -ne $null -and $Role -ne '') {
		$Parameters.add("role",$Role)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/roles/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSba {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSba {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[psobject] $Fingerprint,
		[Parameter()]
		[string] $Identity,
		[Parameter()]
		[string] $TenantId,
		[Parameter()]
		[string] $Application,
		[Parameter()]
		[string] $Secret,
		[Parameter()]
		[psobject] $Log,
		[Parameter()]
		[psobject] $Certificate,
		[Parameter()]
		[string] $IpAddress,
		[Parameter()]
		[psobject] $PrivateKey,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($Fingerprint -ne $null) {
		$Body.add("fingerprint",$Fingerprint)
	}

	if ($Identity -ne $null) {
		$Body.add("identity",$Identity)
	}

	if ($TenantId -ne $null) {
		$Body.add("tenantId",$TenantId)
	}

	if ($Application -ne $null) {
		$Body.add("application",$Application)
	}

	if ($Secret -ne $null) {
		$Body.add("secret",$Secret)
	}

	if ($Log -ne $null) {
		$Body.add("log",$Log)
	}

	if ($Certificate -ne $null) {
		$Body.add("certificate",$Certificate)
	}

	if ($IpAddress -ne $null) {
		$Body.add("ipAddress",$IpAddress)
	}

	if ($PrivateKey -ne $null) {
		$Body.add("privateKey",$PrivateKey)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/sba/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Enable-anynodeMsTeamsSba {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/sba/enable -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Disable-anynodeMsTeamsSba {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/sba/disable -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaUsers {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/users/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Update-anynodeMsTeamsSbaUsers {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/sba/users/refresh -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaStatus {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=1,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/status/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaPrivateKey {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filename,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeDownload -Address $Address -Port $Port -URITail /api/sba/privateKey/get -Filename $Filename -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSbaPrivateKey {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $PrivateKeyPassword,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filepath,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($PrivateKeyPassword -ne $null -and $PrivateKeyPassword -ne '') {
		$Parameters.add("password",$PrivateKeyPassword)
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/sba/privateKey/set -Filenames @($Filepath) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaCertificate {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/certificate/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSbaCertificate {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $CertificatePassword,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filepath,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($CertificatePassword -ne $null -and $CertificatePassword -ne '') {
		$Parameters.add("password",$CertificatePassword)
	}

	Invoke-anynodeUpload -Address $Address -Port $Port -URITail /api/sba/certificate/set -Filenames @($Filepath) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaFingerprint {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/fingerprint/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSbaFingerprint {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('NONE','CUSTOM','MS_TEAMS_NODE')]
		[string] $Source,
		[Parameter()]
		[string] $Value,
		[Parameter()]
		[string] $MsTeamsNodeId,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($Source -ne $null) {
		$Body.add("source",$Source)
	}

	if ($Value -ne $null) {
		$Body.add("value",$Value)
	}

	if ($MsTeamsNodeId -ne $null) {
		$Body.add("msTeamsNodeId",$MsTeamsNodeId)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/sba/fingerprint/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaIpAddress {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/ipAddress/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSbaIpAddress {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $IpAddress,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=$IpAddress

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/sba/ipAddress/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaIdentity {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/identity/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSbaIdentity {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Identity,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=$Identity

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/sba/identity/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaTenantId {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/tenantId/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSbaTenantId {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $TenantId,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=$TenantId

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/sba/tenantId/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaApplication {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/application/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSbaApplication {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Application,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=$Application

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/sba/application/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSbaSecret {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Secret,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=$Secret

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/sba/secret/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeMsTeamsSbaLog {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/sba/log/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeMsTeamsSbaLog {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Directory,
		[Parameter()]
		[ValidateSet('TRACE','DEBUG','INFO','WARN','ERROR','FATAL')]
		[string] $Level,
		[Parameter()]
		[int] $MaxArchiveFiles,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($Directory -ne $null) {
		$Body.add("directory",$Directory)
	}

	if ($Level -ne $null) {
		$Body.add("level",$Level)
	}

	if ($MaxArchiveFiles -ne $null) {
		$Body.add("maxArchiveFiles",$MaxArchiveFiles)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/sba/log/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeSorts {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[ValidateSet('USRAGG_DIRECTORY (Aggregate Directory)','AN_SIPLB_STACK (Load Balancer)','AN_SIPRT_ROUTE (Standard Transport Connection)','AZURE_ACTIVE_DIRECTORY (Azure Active Directory)','CAPIC_STACK (CAPI Client)','CAPIS_STACK (XCAPI Controller)','CAPIS_DOMAIN (XCAPI Controller Domain)','CS_CONDITION (Condition)','CS_SCHEDULER (Rate Limiter)','CS_STUB (Stub)','USRDB_DIRECTORY (Database Connected Directory)','CRY_X509_STACK (Digital Identity and Trust)','USRRT_ROUTE_SV (Directory Route Supervision)','MISC_TIME_REPORTER (Time Range)','MEDIA_PUMP_DOMAIN (Media Transcoding Options)','HTTP_SERVER (HTTP Server)','HTTP_CLIENT (HTTP Client)','IN_FILTER (Network Peer Whitelist)','IN_MAP_STACK (Network Mapping)','IN_MAP_STATIC_STACK (Network Static Mapping)','IN_STACK (Network Controller)','IPC_CLIENT (IPC Client)','IPC_SERVER (IPC Server)','LDAP_CONNECTION (LDAP Client Connection)','USRLDAP_DIRECTORY (LDAP Directory)','MNS_TEAMS_DOMAIN (Media Negotiation (Microsoft Teams Extensions))','MNS_FORWARDER_DOMAIN (Media Negotiation Forwarder)','MNS_STACK (Media Negotiation)','REC_STACK (Media Recorder)','RECMUX_STACK (Media Recording Multiplexer)','RECFILE_STACK (Media File Recorder)','RECTEL_STACK (Media Telephony Recorder)','MS_AUDIO_PROVIDER (Audio Media Source)','MSFILE_PROVIDER (File Media Source)','MS_GRAPH_CLIENT (Microsoft Graph Client)','TELTEAMS_STACK (Microsoft Teams Connectivity)','TELTEAMS_TENANT (Microsoft Teams Tenant)','INS_STACK (Network Security Profile)','NUMVALRT_ROUTE_SV (Number Validation Route Supervision)','OAUTH_CLIENT (OAuth Client)','PROVISIONING_SERVER (Provisioning Server)','IN_QOS_STACK (QoS Profile)','RESTRT_ROUTE_SV (REST Client Route Supervision)','SIPAUTH_STACK (Authentication Profile)','SIPLB_STACK (Load Balancing Transport Connection)','SIPPX_STACK (Request Router)','RECSIPREC_STACK (SIPREC Media Recorder)','TELSIPREG_REGISTRAR (SIP Registrar)','SIPRT_PLAIN_ROUTE (Plain Transport Connection)','SIPST_STACK (SIP Transport)','SIPUA_REGISTRATION (SIP Registration)','SIPUA_STACK (SIP User Agent)','SRVLB_STACK (SRV Load Balancer)','USRTEL_DIRECTORY (Static Dial string Directory)','USRT_DIRECTORY (Static User Directory)','MSTEL_PROVIDER (Telephony Media Source)','TELELIN_DOMAIN (ELIN Binding Database)','TELFW_STACK (Telephony Forwarding)','TEL_REWRITE_DOMAIN (Dial String Rewriting)','TELRT_FORWARDER_DOMAIN (Routing Forward Profile)','TELRT_STACK (Routing Domain)','TEL_STACK (Node)','TELCAPIC_STACK (CAPI Client Node)','TELCAPIS_STACK (XCAPI Controller Node)','TELSIP_STACK (SIP Node)','TELUCMA_STACK (SfB UCMA Node)','TELWEBRTC_STACK (WebRTC Node)','WEBRTC_STACK (WebRTC Server)','WEBSOCKET_STACK (Websocket)')]
		[string] $Sort,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($DisplayName -ne $null -and $DisplayName -ne '') {
		$Parameters.add("displayName",$DisplayName)
	}

	if ($Sort -ne $null -and $Sort -ne '') {
		$Parameters.add("sort",$Sort)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/objects/sorts -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeSystemIds {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/licenses/systemIds/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeTrace {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('anynode','anynodeFrontend','anynodeMonitor','anynodeAdministration')]
		[string] $Product,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter(mandatory=$true)]
		[string] $Filename,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Product -ne $null -and $Product -ne '') {
		$Parameters.add("product",$Product)
	}

	Invoke-anynodeDownload -Address $Address -Port $Port -URITail /api/trace/get -Filename $Filename -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeTracing {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('anynode','anynodeFrontend','anynodeMonitor','anynodeAdministration')]
		[string] $Product,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Product -ne $null -and $Product -ne '') {
		$Parameters.add("product",$Product)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/tracing/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeTracing {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('anynode','anynodeFrontend','anynodeMonitor','anynodeAdministration')]
		[string] $Product,
		[Parameter()]
		[ValidateSet('FILE','SPLIT')]
		[string] $Selection,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $TraceActive,
		[Parameter()]
		[psobject] $Options,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Product -ne $null -and $Product -ne '') {
		$Parameters.add("product",$Product)
	}

	$Body=@{}

	if ($Selection -ne $null) {
		$Body.add("selection",$Selection)
	}

	if ($TraceActive -ne $null) {
		$Body.add("traceActive",$TraceActive)
	}

	if ($Options -ne $null) {
		$Body.add("options",$Options)
	}

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/tracing/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeUsers {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/users/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Add-anynodeUser {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[string] $EmailAddress,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $Enabled,
		[Parameter()]
		[psobject] $LdapAuthentications,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $LdapLogin,
		[Parameter()]
		[string] $LocalPassword,
		[Parameter()]
		[string] $LocalUsername,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $LoginLocally,
		[Parameter()]
		[ValidateSet('NOT_NECESSARY','REQUIERED','RECOMMENDED')]
		[string] $PasswordChange,
		[Parameter()]
		[psobject] $RoleIds,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($EmailAddress -ne $null) {
		$Body.add("emailAddress",$EmailAddress)
	}

	if ($Enabled -ne $null) {
		$Body.add("enabled",$Enabled)
	}

	if ($LdapAuthentications -ne $null) {
		$Body.add("ldapAuthentications",$LdapAuthentications)
	}

	if ($LdapLogin -ne $null) {
		$Body.add("ldapLogin",$LdapLogin)
	}

	if ($LocalPassword -ne $null) {
		$Body.add("localPassword",$LocalPassword)
	}

	if ($LocalUsername -ne $null) {
		$Body.add("localUsername",$LocalUsername)
	}

	if ($LoginLocally -ne $null) {
		$Body.add("loginLocally",$LoginLocally)
	}

	if ($PasswordChange -ne $null) {
		$Body.add("passwordChange",$PasswordChange)
	}

	if ($RoleIds -ne $null) {
		$Body.add("roleIds",$RoleIds)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/users/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Remove-anynodeUser {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $User,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($User -ne $null -and $User -ne '') {
		$Parameters.add("user",$User)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/users/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Submit-anynodePasswordChange {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('NOT_NECESSARY','REQUIERED','RECOMMENDED')]
		[string] $PasswordChange,
		[Parameter()]
		[string] $OldPassword,
		[Parameter()]
		[string] $NewPassword,
		[Parameter()]
		[string] $User,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($PasswordChange -ne $null -and $PasswordChange -ne '') {
		$Parameters.add("passwordChange",$PasswordChange)
	}

	if ($OldPassword -ne $null -and $OldPassword -ne '') {
		$Parameters.add("oldPassword",$OldPassword)
	}

	if ($NewPassword -ne $null -and $NewPassword -ne '') {
		$Parameters.add("newPassword",$NewPassword)
	}

	if ($User -ne $null -and $User -ne '') {
		$Parameters.add("user",$User)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/users/changePassword -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeLdapAuthorities {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/users/ldapAuthorities/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeVersion {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('anynode','ucma','administration','monitor','frontend','java')]
		[string] $Product,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Product -ne $null -and $Product -ne '') {
		$Parameters.add("product",$Product)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/version/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeWebserverConnector {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $DisplayName,
		[Parameter()]
		[ValidateSet('HTTP','HTTPS')]
		[string] $HttpMode,
		[Parameter()]
		[int] $HttpRedirectorPort,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[psobject] $NetworkOptions,
		[Parameter()]
		[ValidateSet($true,$false)]
		[bool] $OpenPortOnDemand,
		[Parameter()]
		[int] $ConnectorPort,
		[Parameter()]
		[psobject] $RequestDomains,
		[Parameter()]
		[psobject] $TlsOptions,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($DisplayName -ne $null) {
		$Body.add("displayName",$DisplayName)
	}

	if ($HttpMode -ne $null) {
		$Body.add("httpMode",$HttpMode)
	}

	if ($HttpRedirectorPort -ne $null) {
		$Body.add("httpRedirectorPort",$HttpRedirectorPort)
	}

	if ($Id -ne $null) {
		$Body.add("id",$Id)
	}

	if ($NetworkOptions -ne $null) {
		$Body.add("networkOptions",$NetworkOptions)
	}

	if ($OpenPortOnDemand -ne $null) {
		$Body.add("openPortOnDemand",$OpenPortOnDemand)
	}

	if ($ConnectorPort -ne $null) {
		$Body.add("port",$ConnectorPort)
	}

	if ($RequestDomains -ne $null) {
		$Body.add("requestDomains",$RequestDomains)
	}

	if ($TlsOptions -ne $null) {
		$Body.add("tlsOptions",$TlsOptions)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/webserverConnectors/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeWebserverConnector {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/webserverConnectors/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Remove-anynodeWebserverConnector {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Identifier,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Identifier -ne $null -and $Identifier -ne '') {
		$Parameters.add("identifier",$Identifier)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/webserverConnectors/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeWebserverConnectors {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[psobject[]] $WebserverConnectorTable,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=(ConvertTo-Json -InputObject @($WebserverConnectorTable))

	Invoke-anynodeRequest -Method Put -Address $Address -Port $Port -URITail /api/webserverConnectors/set -Body $Body -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Get-anynodeRequestRouter {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RequestRouter,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RequestRouter -ne $null -and $RequestRouter -ne '') {
		$Parameters.add("requestRouter",$RequestRouter)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/requestRouters/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeRequestRouter {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[ValidateSet('BI_DIRECTIONAL','PRIMARY_TO_SECONDARY_EDGE','SECONDARY_TO_PRIMARY_EDGE','DISABLED')]
		[string] $ForwardingMode,
		[Parameter()]
		[string] $OperationalStateConditionId,
		[Parameter()]
		[psobject] $PrimarySideOptions,
		[Parameter()]
		[psobject] $SecondarySideOptions,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Body=@{}

	if ($Id -ne $null) {
		$Body.add("id",$Id)
	}

	if ($ForwardingMode -ne $null) {
		$Body.add("forwardingMode",$ForwardingMode)
	}

	if ($OperationalStateConditionId -ne $null) {
		$Body.add("operationalStateConditionId",$OperationalStateConditionId)
	}

	if ($PrimarySideOptions -ne $null) {
		$Body.add("primarySideOptions",$PrimarySideOptions)
	}

	if ($SecondarySideOptions -ne $null) {
		$Body.add("secondarySideOptions",$SecondarySideOptions)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/requestRouters/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Username $Username -Password $Password
}



function Set-anynodeRequestRouter {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RequestRouter,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[ValidateSet('BI_DIRECTIONAL','PRIMARY_TO_SECONDARY_EDGE','SECONDARY_TO_PRIMARY_EDGE','DISABLED')]
		[string] $ForwardingMode,
		[Parameter()]
		[string] $OperationalStateConditionId,
		[Parameter()]
		[psobject] $PrimarySideOptions,
		[Parameter()]
		[psobject] $SecondarySideOptions,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RequestRouter -ne $null -and $RequestRouter -ne '') {
		$Parameters.add("requestRouter",$RequestRouter)
	}

	$Body=@{}

	if ($Id -ne $null) {
		$Body.add("id",$Id)
	}

	if ($ForwardingMode -ne $null) {
		$Body.add("forwardingMode",$ForwardingMode)
	}

	if ($OperationalStateConditionId -ne $null) {
		$Body.add("operationalStateConditionId",$OperationalStateConditionId)
	}

	if ($PrimarySideOptions -ne $null) {
		$Body.add("primarySideOptions",$PrimarySideOptions)
	}

	if ($SecondarySideOptions -ne $null) {
		$Body.add("secondarySideOptions",$SecondarySideOptions)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/requestRouters/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Remove-anynodeRequestRouter {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $RequestRouter,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($RequestRouter -ne $null -and $RequestRouter -ne '') {
		$Parameters.add("requestRouter",$RequestRouter)
	}

	Invoke-anynodeRequest -Method Delete -Address $Address -Port $Port -URITail /api/requestRouters/remove -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeHotStandby {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/hotStandby/get -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeHotStandby {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[ValidateSet('addMainSystemHotStandbyObjects','addStandbySystemHotStandbyObjects','addMainAndStandbySystemHotStandbyObjects')]
		[string] $Mode,
		[Parameter()]
		[psobject] $MainSystem,
		[Parameter()]
		[psobject] $StandbySystem,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Mode -ne $null -and $Mode -ne '') {
		$Parameters.add("mode",$Mode)
	}

	$Body=@{}

	if ($MainSystem -ne $null) {
		$Body.add("mainSystem",$MainSystem)
	}

	if ($StandbySystem -ne $null) {
		$Body.add("standbySystem",$StandbySystem)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/hotStandby/add -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeHotStandby {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[psobject] $MainSystem,
		[Parameter()]
		[psobject] $StandbySystem,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	$Body=@{}

	if ($MainSystem -ne $null) {
		$Body.add("mainSystem",$MainSystem)
	}

	if ($StandbySystem -ne $null) {
		$Body.add("standbySystem",$StandbySystem)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/hotStandby/set -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeHotStandbyStatus {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/hotStandby/status -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Add-anynodeHotStandbyCondiitons {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[psobject] $CreateIsMainSystemCondition,
		[Parameter()]
		[psobject] $CreateIsNotSystemIntentToBecomeActiveCondition,
		[Parameter()]
		[psobject] $CreateIsStandbySystemCondition,
		[Parameter()]
		[psobject] $CreateIsSystemActiveCondition,
		[Parameter()]
		[psobject] $CreateIsSystemIntentToBecomeActiveCondition,
		[Parameter()]
		[psobject] $CreateIsSystemNotActiveCondition,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	$Body=@{}

	if ($CreateIsMainSystemCondition -ne $null) {
		$Body.add("createIsMainSystemCondition",$CreateIsMainSystemCondition)
	}

	if ($CreateIsNotSystemIntentToBecomeActiveCondition -ne $null) {
		$Body.add("createIsNotSystemIntentToBecomeActiveCondition",$CreateIsNotSystemIntentToBecomeActiveCondition)
	}

	if ($CreateIsStandbySystemCondition -ne $null) {
		$Body.add("createIsStandbySystemCondition",$CreateIsStandbySystemCondition)
	}

	if ($CreateIsSystemActiveCondition -ne $null) {
		$Body.add("createIsSystemActiveCondition",$CreateIsSystemActiveCondition)
	}

	if ($CreateIsSystemIntentToBecomeActiveCondition -ne $null) {
		$Body.add("createIsSystemIntentToBecomeActiveCondition",$CreateIsSystemIntentToBecomeActiveCondition)
	}

	if ($CreateIsSystemNotActiveCondition -ne $null) {
		$Body.add("createIsSystemNotActiveCondition",$CreateIsSystemNotActiveCondition)
	}

	Invoke-anynodeRequest -Method Post -Address $Address -Port $Port -URITail /api/hotStandby/createConditions -Body ($Body | ConvertTo-Json -Depth 99) -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeHotStandbyHandover {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/hotStandby/handover -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Set-anynodeHotStandbyRecover {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/hotStandby/recover -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Enable-anynodeHotStandbyMainSystem {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/hotStandby/enableMainSystem -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Enable-anynodeHotStandbyStandbySystem {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $Id,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($Id -ne $null -and $Id -ne '') {
		$Parameters.add("id",$Id)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/hotStandby/enableStandbySystem -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}



function Get-anynodeMessageHistory {
	[CmdletBinding()]
	Param(
		[Parameter()]
		[string] $Address,
		[Parameter()]
		[int] $Port,
		[Parameter()]
		[string] $FilterSystemIdentifier,
		[Parameter()]
		[ValidateSet('pcap','textFlow','textMessages','HTML','details')]
		[string] $ResultFormat,
		[Parameter()]
		[string] $FilterSessionIdentifier,
		[Parameter()]
		[PSCredential] $Credential,
		[Parameter()]
		[int] $APIVersion=0,
		[Parameter()]
		[string] $Username,
		[Parameter()]
		[string] $Password
	)

	$Parameters=@{}

	if ($FilterSystemIdentifier -ne $null -and $FilterSystemIdentifier -ne '') {
		$Parameters.add("filterSystemIdentifier",$FilterSystemIdentifier)
	}

	if ($ResultFormat -ne $null -and $ResultFormat -ne '') {
		$Parameters.add("resultFormat",$ResultFormat)
	}

	if ($FilterSessionIdentifier -ne $null -and $FilterSessionIdentifier -ne '') {
		$Parameters.add("filterSessionIdentifier",$FilterSessionIdentifier)
	}

	Invoke-anynodeRequest -Method Get -Address $Address -Port $Port -URITail /api/messageHistory -Credential $Credential -BackendName $BackendName -APIVersion $APIVersion -Parameters $Parameters -Username $Username -Password $Password
}
# SIG # Begin signature block
# MIIt5QYJKoZIhvcNAQcCoIIt1jCCLdICAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUUWH5texN9ogyDBACEMlouIQw
# 9oSggiceMIIFcjCCA1qgAwIBAgIQdlP+rHVGSJP15ddKSDpO+DANBgkqhkiG9w0B
# AQwFADBTMQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFsU2lnbiBudi1zYTEp
# MCcGA1UEAxMgR2xvYmFsU2lnbiBDb2RlIFNpZ25pbmcgUm9vdCBSNDUwHhcNMjAw
# MzE4MDAwMDAwWhcNNDUwMzE4MDAwMDAwWjBTMQswCQYDVQQGEwJCRTEZMBcGA1UE
# ChMQR2xvYmFsU2lnbiBudi1zYTEpMCcGA1UEAxMgR2xvYmFsU2lnbiBDb2RlIFNp
# Z25pbmcgUm9vdCBSNDUwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQC2
# LcUw3Xroq5A9A3KwOkuZFmGy5f+lZx03HOV+7JODqoT1o0ObmEWKuGNXXZsAiAQl
# 6fhokkuC2EvJSgPzqH9qj4phJ72hRND99T8iwqNPkY2zBbIogpFd+1mIBQuXBsKY
# +CynMyTuUDpBzPCgsHsdTdKoWDiW6d/5G5G7ixAs0sdDHaIJdKGAr3vmMwoMWWuO
# vPSrWpd7f65V+4TwgP6ETNfiur3EdaFvvWEQdESymAfidKv/aNxsJj7pH+XgBIet
# MNMMjQN8VbgWcFwkeCAl62dniKu6TjSYa3AR3jjK1L6hwJzh3x4CAdg74WdDhLbP
# /HS3L4Sjv7oJNz1nbLFFXBlhq0GD9awd63cNRkdzzr+9lZXtnSuIEP76WOinV+Gz
# z6ha6QclmxLEnoByPZPcjJTfO0TmJoD80sMD8IwM0kXWLuePmJ7mBO5Cbmd+QhZx
# YucE+WDGZKG2nIEhTivGbWiUhsaZdHNnMXqR8tSMeW58prt+Rm9NxYUSK8+aIkQI
# qIU3zgdhVwYXEiTAxDFzoZg1V0d+EDpF2S2kUZCYqaAHN8RlGqocaxZ396eX7D8Z
# MJlvMfvqQLLn0sT6ydDwUHZ0WfqNbRcyvvjpfgP054d1mtRKkSyFAxMCK0KA8olq
# Ns/ITKDOnvjLja0Wp9Pe1ZsYp8aSOvGCY/EuDiRk3wIDAQABo0IwQDAOBgNVHQ8B
# Af8EBAMCAYYwDwYDVR0TAQH/BAUwAwEB/zAdBgNVHQ4EFgQUHwC/RoAK/Hg5t6W0
# Q9lWULvOljswDQYJKoZIhvcNAQEMBQADggIBAF4runSXNERfdkgoQIST7gFu6aGz
# 1oAl5nvkvAmRPQ/8dq3X1DAgu49g0JHWHPKc73gaK5QyAsEkllJSAtDz0fzymzlu
# meEfjkNBfZoeW8ldmoT8JuaH83RyJq2kG9k9O2pSoDwJHi8ee7MztEXH96yxr5Ng
# rXauuLIVeOuDauv/20arJOXuAvqQH1nAL13Wt12kXBC3clP4QU7M+ngaJUrK/oVi
# Q2HDtDeqgdL01joPvY1ZfjBH3itr5yFQM1/UZ5vUuGefPCeZA/+FQ45zEsogzehh
# 1bFm3BfWOW0P288jN6GCiU4caz/WoM2qB50+Qiaq1wzu+ke/GlJ+0XWB08mKYhdt
# T4igIaAmPq9t2WIwH+mYKK5ujdWOTHJmk4CNKuNVx2BnkEJWXCJRD7PcTjnuTd3Z
# HXgQVDtu0JdvA7UesiNzxhKymmTQ/JWFJKj/36Gw3JFArt8JM6u53ZK38cyRdDtp
# 62eXG5C/58egb3G7V7+3j1rtekBqFs2AhC0v4QLUJJRDsxX8DCsb/XFv/Mu8dRc6
# XoPSybMvG9WcjX9U/n5+5Fajh6ed4VlSlEGPbVu+hpWa/xp23UDSUUpwtB8zYyN3
# P+wnHlnkCIftNIJKDz/+oB3B9WdzRYZ49Kop6SeHxhnbxhMUwzlJh02gl+BlE/Wd
# d1bp2rNYxzrywM2CMIIFjTCCBHWgAwIBAgIQDpsYjvnQLefv21DiCEAYWjANBgkq
# hkiG9w0BAQwFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5j
# MRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2VydCBB
# c3N1cmVkIElEIFJvb3QgQ0EwHhcNMjIwODAxMDAwMDAwWhcNMzExMTA5MjM1OTU5
# WjBiMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQL
# ExB3d3cuZGlnaWNlcnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBUcnVzdGVkIFJv
# b3QgRzQwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQC/5pBzaN675F1K
# PDAiMGkz7MKnJS7JIT3yithZwuEppz1Yq3aaza57G4QNxDAf8xukOBbrVsaXbR2r
# snnyyhHS5F/WBTxSD1Ifxp4VpX6+n6lXFllVcq9ok3DCsrp1mWpzMpTREEQQLt+C
# 8weE5nQ7bXHiLQwb7iDVySAdYyktzuxeTsiT+CFhmzTrBcZe7FsavOvJz82sNEBf
# sXpm7nfISKhmV1efVFiODCu3T6cw2Vbuyntd463JT17lNecxy9qTXtyOj4DatpGY
# QJB5w3jHtrHEtWoYOAMQjdjUN6QuBX2I9YI+EJFwq1WCQTLX2wRzKm6RAXwhTNS8
# rhsDdV14Ztk6MUSaM0C/CNdaSaTC5qmgZ92kJ7yhTzm1EVgX9yRcRo9k98FpiHaY
# dj1ZXUJ2h4mXaXpI8OCiEhtmmnTK3kse5w5jrubU75KSOp493ADkRSWJtppEGSt+
# wJS00mFt6zPZxd9LBADMfRyVw4/3IbKyEbe7f/LVjHAsQWCqsWMYRJUadmJ+9oCw
# ++hkpjPRiQfhvbfmQ6QYuKZ3AeEPlAwhHbJUKSWJbOUOUlFHdL4mrLZBdd56rF+N
# P8m800ERElvlEFDrMcXKchYiCd98THU/Y+whX8QgUWtvsauGi0/C1kVfnSD8oR7F
# wI+isX4KJpn15GkvmB0t9dmpsh3lGwIDAQABo4IBOjCCATYwDwYDVR0TAQH/BAUw
# AwEB/zAdBgNVHQ4EFgQU7NfjgtJxXWRM3y5nP+e6mK4cD08wHwYDVR0jBBgwFoAU
# Reuir/SSy4IxLVGLp6chnfNtyA8wDgYDVR0PAQH/BAQDAgGGMHkGCCsGAQUFBwEB
# BG0wazAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEMGCCsG
# AQUFBzAChjdodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRBc3N1
# cmVkSURSb290Q0EuY3J0MEUGA1UdHwQ+MDwwOqA4oDaGNGh0dHA6Ly9jcmwzLmRp
# Z2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcmwwEQYDVR0gBAow
# CDAGBgRVHSAAMA0GCSqGSIb3DQEBDAUAA4IBAQBwoL9DXFXnOF+go3QbPbYW1/e/
# Vwe9mqyhhyzshV6pGrsi+IcaaVQi7aSId229GhT0E0p6Ly23OO/0/4C5+KH38nLe
# JLxSA8hO0Cre+i1Wz/n096wwepqLsl7Uz9FDRJtDIeuWcqFItJnLnU+nBgMTdydE
# 1Od/6Fmo8L8vC6bp8jQ87PcDx4eo0kxAGTVGamlUsLihVo7spNU96LHc/RzY9Hda
# XFSMb++hUD38dglohJ9vytsgjTVgHAIDyyCwrFigDkBjxZgiwbJZ9VVrzyerbHbO
# byMt9H5xaiNrIv8SuFQtJ37YOtnwtoeW/VvRXKwYw02fc7cBqZ9Xql4o4rmUMIIG
# rjCCBJagAwIBAgIQBzY3tyRUfNhHrP0oZipeWzANBgkqhkiG9w0BAQsFADBiMQsw
# CQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cu
# ZGlnaWNlcnQuY29tMSEwHwYDVQQDExhEaWdpQ2VydCBUcnVzdGVkIFJvb3QgRzQw
# HhcNMjIwMzIzMDAwMDAwWhcNMzcwMzIyMjM1OTU5WjBjMQswCQYDVQQGEwJVUzEX
# MBUGA1UEChMORGlnaUNlcnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lDZXJ0IFRydXN0
# ZWQgRzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0YW1waW5nIENBMIICIjANBgkqhkiG
# 9w0BAQEFAAOCAg8AMIICCgKCAgEAxoY1BkmzwT1ySVFVxyUDxPKRN6mXUaHW0oPR
# nkyibaCwzIP5WvYRoUQVQl+kiPNo+n3znIkLf50fng8zH1ATCyZzlm34V6gCff1D
# tITaEfFzsbPuK4CEiiIY3+vaPcQXf6sZKz5C3GeO6lE98NZW1OcoLevTsbV15x8G
# ZY2UKdPZ7Gnf2ZCHRgB720RBidx8ald68Dd5n12sy+iEZLRS8nZH92GDGd1ftFQL
# IWhuNyG7QKxfst5Kfc71ORJn7w6lY2zkpsUdzTYNXNXmG6jBZHRAp8ByxbpOH7G1
# WE15/tePc5OsLDnipUjW8LAxE6lXKZYnLvWHpo9OdhVVJnCYJn+gGkcgQ+NDY4B7
# dW4nJZCYOjgRs/b2nuY7W+yB3iIU2YIqx5K/oN7jPqJz+ucfWmyU8lKVEStYdEAo
# q3NDzt9KoRxrOMUp88qqlnNCaJ+2RrOdOqPVA+C/8KI8ykLcGEh/FDTP0kyr75s9
# /g64ZCr6dSgkQe1CvwWcZklSUPRR8zZJTYsg0ixXNXkrqPNFYLwjjVj33GHek/45
# wPmyMKVM1+mYSlg+0wOI/rOP015LdhJRk8mMDDtbiiKowSYI+RQQEgN9XyO7ZONj
# 4KbhPvbCdLI/Hgl27KtdRnXiYKNYCQEoAA6EVO7O6V3IXjASvUaetdN2udIOa5kM
# 0jO0zbECAwEAAaOCAV0wggFZMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYE
# FLoW2W1NhS9zKXaaL3WMaiCPnshvMB8GA1UdIwQYMBaAFOzX44LScV1kTN8uZz/n
# upiuHA9PMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDCDB3Bggr
# BgEFBQcBAQRrMGkwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNv
# bTBBBggrBgEFBQcwAoY1aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lD
# ZXJ0VHJ1c3RlZFJvb3RHNC5jcnQwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2Ny
# bDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZFJvb3RHNC5jcmwwIAYDVR0g
# BBkwFzAIBgZngQwBBAIwCwYJYIZIAYb9bAcBMA0GCSqGSIb3DQEBCwUAA4ICAQB9
# WY7Ak7ZvmKlEIgF+ZtbYIULhsBguEE0TzzBTzr8Y+8dQXeJLKftwig2qKWn8acHP
# HQfpPmDI2AvlXFvXbYf6hCAlNDFnzbYSlm/EUExiHQwIgqgWvalWzxVzjQEiJc6V
# aT9Hd/tydBTX/6tPiix6q4XNQ1/tYLaqT5Fmniye4Iqs5f2MvGQmh2ySvZ180HAK
# fO+ovHVPulr3qRCyXen/KFSJ8NWKcXZl2szwcqMj+sAngkSumScbqyQeJsG33irr
# 9p6xeZmBo1aGqwpFyd/EjaDnmPv7pp1yr8THwcFqcdnGE4AJxLafzYeHJLtPo0m5
# d2aR8XKc6UsCUqc3fpNTrDsdCEkPlM05et3/JWOZJyw9P2un8WbDQc1PtkCbISFA
# 0LcTJM3cHXg65J6t5TRxktcma+Q4c6umAU+9Pzt4rUyt+8SVe+0KXzM5h0F4ejjp
# nOHdI/0dKNPH+ejxmF/7K9h+8kaddSweJywm228Vex4Ziza4k9Tm8heZWcpw8De/
# mADfIBZPJ/tgZxahZrrdVcA6KYawmKAr7ZVBtzrVFZgxtGIJDwq9gdkT/r+k0fNX
# 2bwE+oLeMt8EifAAzV3C+dAjfwAL5HYCJtnwZXZCpimHCUcr5n8apIUP/JiW9lVU
# Kx+A+sDyDivl1vupL0QVSucTDh3bNzgaoSv27dZ8/DCCBsIwggSqoAMCAQICEAVE
# r/OUnQg5pr/bP1/lYRYwDQYJKoZIhvcNAQELBQAwYzELMAkGA1UEBhMCVVMxFzAV
# BgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMTswOQYDVQQDEzJEaWdpQ2VydCBUcnVzdGVk
# IEc0IFJTQTQwOTYgU0hBMjU2IFRpbWVTdGFtcGluZyBDQTAeFw0yMzA3MTQwMDAw
# MDBaFw0zNDEwMTMyMzU5NTlaMEgxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdp
# Q2VydCwgSW5jLjEgMB4GA1UEAxMXRGlnaUNlcnQgVGltZXN0YW1wIDIwMjMwggIi
# MA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQCjU0WHHYOOW6w+VLMj4M+f1+XS
# 512hDgncL0ijl3o7Kpxn3GIVWMGpkxGnzaqyat0QKYoeYmNp01icNXG/OpfrlFCP
# HCDqx5o7L5Zm42nnaf5bw9YrIBzBl5S0pVCB8s/LB6YwaMqDQtr8fwkklKSCGtpq
# utg7yl3eGRiF+0XqDWFsnf5xXsQGmjzwxS55DxtmUuPI1j5f2kPThPXQx/ZILV5F
# dZZ1/t0QoRuDwbjmUpW1R9d4KTlr4HhZl+NEK0rVlc7vCBfqgmRN/yPjyobutKQh
# ZHDr1eWg2mOzLukF7qr2JPUdvJscsrdf3/Dudn0xmWVHVZ1KJC+sK5e+n+T9e3M+
# Mu5SNPvUu+vUoCw0m+PebmQZBzcBkQ8ctVHNqkxmg4hoYru8QRt4GW3k2Q/gWEH7
# 2LEs4VGvtK0VBhTqYggT02kefGRNnQ/fztFejKqrUBXJs8q818Q7aESjpTtC/XN9
# 7t0K/3k0EH6mXApYTAA+hWl1x4Nk1nXNjxJ2VqUk+tfEayG66B80mC866msBsPf7
# Kobse1I4qZgJoXGybHGvPrhvltXhEBP+YUcKjP7wtsfVx95sJPC/QoLKoHE9nJKT
# BLRpcCcNT7e1NtHJXwikcKPsCvERLmTgyyIryvEoEyFJUX4GZtM7vvrrkTjYUQfK
# lLfiUKHzOtOKg8tAewIDAQABo4IBizCCAYcwDgYDVR0PAQH/BAQDAgeAMAwGA1Ud
# EwEB/wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwIAYDVR0gBBkwFzAIBgZn
# gQwBBAIwCwYJYIZIAYb9bAcBMB8GA1UdIwQYMBaAFLoW2W1NhS9zKXaaL3WMaiCP
# nshvMB0GA1UdDgQWBBSltu8T5+/N0GSh1VapZTGj3tXjSTBaBgNVHR8EUzBRME+g
# TaBLhklodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkRzRS
# U0E0MDk2U0hBMjU2VGltZVN0YW1waW5nQ0EuY3JsMIGQBggrBgEFBQcBAQSBgzCB
# gDAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMFgGCCsGAQUF
# BzAChkxodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVk
# RzRSU0E0MDk2U0hBMjU2VGltZVN0YW1waW5nQ0EuY3J0MA0GCSqGSIb3DQEBCwUA
# A4ICAQCBGtbeoKm1mBe8cI1PijxonNgl/8ss5M3qXSKS7IwiAqm4z4Co2efjxe0m
# gopxLxjdTrbebNfhYJwr7e09SI64a7p8Xb3CYTdoSXej65CqEtcnhfOOHpLawkA4
# n13IoC4leCWdKgV6hCmYtld5j9smViuw86e9NwzYmHZPVrlSwradOKmB521BXIxp
# 0bkrxMZ7z5z6eOKTGnaiaXXTUOREEr4gDZ6pRND45Ul3CFohxbTPmJUaVLq5vMFp
# GbrPFvKDNzRusEEm3d5al08zjdSNd311RaGlWCZqA0Xe2VC1UIyvVr1MxeFGxSjT
# redDAHDezJieGYkD6tSRN+9NUvPJYCHEVkft2hFLjDLDiOZY4rbbPvlfsELWj+MX
# kdGqwFXjhr+sJyxB0JozSqg21Llyln6XeThIX8rC3D0y33XWNmdaifj2p8flTzU8
# AL2+nCpseQHc2kTmOt44OwdeOVj0fHMxVaCAEcsUDH6uvP6k63llqmjWIso765qC
# NVcoFstp8jKastLYOrixRoZruhf9xHdsFWyuq69zOuhJRrfVf8y2OMDY7Bz1tqG4
# QyzfTkx9HmhwwHcK1ALgXGC7KP845VJa1qwXIiNO9OzTF/tQa/8Hdx9xl0RBybhG
# 02wyfFgvZ0dl5Rtztpn5aywGRu9BHvDwX+Db2a2QgESvgBBBijCCBugwggTQoAMC
# AQICEHe9DgW3WQu2HUdhUx4/de0wDQYJKoZIhvcNAQELBQAwUzELMAkGA1UEBhMC
# QkUxGTAXBgNVBAoTEEdsb2JhbFNpZ24gbnYtc2ExKTAnBgNVBAMTIEdsb2JhbFNp
# Z24gQ29kZSBTaWduaW5nIFJvb3QgUjQ1MB4XDTIwMDcyODAwMDAwMFoXDTMwMDcy
# ODAwMDAwMFowXDELMAkGA1UEBhMCQkUxGTAXBgNVBAoTEEdsb2JhbFNpZ24gbnYt
# c2ExMjAwBgNVBAMTKUdsb2JhbFNpZ24gR0NDIFI0NSBFViBDb2RlU2lnbmluZyBD
# QSAyMDIwMIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAyyDvlx65ATJD
# oFupiiP9IF6uOBKLyizU/0HYGlXUGVO3/aMX53o5XMD3zhGj+aXtAfq1upPvr5Pc
# +OKzGUyDsEpEUAR4hBBqpNaWkI6B+HyrL7WjVzPSWHuUDm0PpZEmKrODT3Kxintk
# ktDwtFVflgsR5Zq1LLIRzyUbfVErmB9Jo1/4E541uAMC2qQTL4VK78QvcA7B1Mwz
# Euy9QJXTEcrmzbMFnMhT61LXeExRAZKC3hPzB450uoSAn9KkFQ7or+v3ifbfcfDR
# vqeyQTMgdcyx1e0dBxnE6yZ38qttF5NJqbfmw5CcxrjszMl7ml7FxSSTY29+EIth
# z5hVoySiiDby+Z++ky6yBp8mwAwBVhLhsoqfDh7cmIsuz9riiTSmHyagqK54beyh
# iBU8wurut9itYaWvcDaieY7cDXPA8eQsq5TsWAY5NkjWO1roIs50Dq8s8RXa0bSV
# 6KzVSW3lr92ba2MgXY5+O7JD2GI6lOXNtJizNxkkEnJzqwSwCdyF5tQiBO9AKh0u
# bcdp0263AWwN4JenFuYmi4j3A0SGX2JnTLWnN6hV3AM2jG7PbTYm8Q6PsD1xwOEy
# p4LktjICMjB8tZPIIf08iOZpY/judcmLwqvvujr96V6/thHxvvA9yjI+bn3eD36b
# lcQSh+cauE7uLMHfoWXoJIPJKsL9uVMCAwEAAaOCAa0wggGpMA4GA1UdDwEB/wQE
# AwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzASBgNVHRMBAf8ECDAGAQH/AgEAMB0G
# A1UdDgQWBBQlndD8WQmGY8Xs87ETO1ccA5I2ETAfBgNVHSMEGDAWgBQfAL9GgAr8
# eDm3pbRD2VZQu86WOzCBkwYIKwYBBQUHAQEEgYYwgYMwOQYIKwYBBQUHMAGGLWh0
# dHA6Ly9vY3NwLmdsb2JhbHNpZ24uY29tL2NvZGVzaWduaW5ncm9vdHI0NTBGBggr
# BgEFBQcwAoY6aHR0cDovL3NlY3VyZS5nbG9iYWxzaWduLmNvbS9jYWNlcnQvY29k
# ZXNpZ25pbmdyb290cjQ1LmNydDBBBgNVHR8EOjA4MDagNKAyhjBodHRwOi8vY3Js
# Lmdsb2JhbHNpZ24uY29tL2NvZGVzaWduaW5ncm9vdHI0NS5jcmwwVQYDVR0gBE4w
# TDBBBgkrBgEEAaAyAQIwNDAyBggrBgEFBQcCARYmaHR0cHM6Ly93d3cuZ2xvYmFs
# c2lnbi5jb20vcmVwb3NpdG9yeS8wBwYFZ4EMAQMwDQYJKoZIhvcNAQELBQADggIB
# ACV1oAnJObq3oTmJLxifq9brHUvolHwNB2ibHJ3vcbYXamsCT7M/hkWHzGWbTONY
# BgIiZtVhAsVjj9Si8bZeJQt3lunNcUAziCns7vOibbxNtT4GS8lzM8oIFC09TOiw
# unWmdC2kWDpsE0n4pRUKFJaFsWpoNCVCr5ZW9BD6JH3xK3LBFuFr6+apmMc+WvTQ
# GJ39dJeGd0YqPSN9KHOKru8rG5q/bFOnFJ48h3HAXo7I+9MqkjPqV01eB17KwRis
# gS0aIfpuz5dhe99xejrKY/fVMEQ3Mv67Q4XcuvymyjMZK3dt28sF8H5fdS6itr81
# qjZjyc5k2b38vCzzSVYAyBIrxie7N69X78TPHinE9OItziphz1ft9QpA4vUY1h7p
# kC/K04dfk4pIGhEd5TeFny5mYppegU6VrFVXQ9xTiyV+PGEPigu69T+m1473BFZe
# Ibuf12pxgL+W3nID2NgiK/MnFk846FFADK6S7749ffeAxkw2V4SVp4QVSDAOUicI
# jY6ivSLHGcmmyg6oejbbarphXxEklaTijmjuGalJmV7QtDS91vlAxxCXMVI5NSkR
# hyTTxPupY8t3SNX6Yvwk4AR6TtDkbt7OnjhQJvQhcWXXCSXUyQcAerjH83foxdTi
# VdDTHvZ/UuJJjbkRcgyIRCYzZgFE3+QzDiHeYolIB9r1MIIHrzCCBZegAwIBAgIM
# H7b9bdAxkf8/itS3MA0GCSqGSIb3DQEBCwUAMFwxCzAJBgNVBAYTAkJFMRkwFwYD
# VQQKExBHbG9iYWxTaWduIG52LXNhMTIwMAYDVQQDEylHbG9iYWxTaWduIEdDQyBS
# NDUgRVYgQ29kZVNpZ25pbmcgQ0EgMjAyMDAeFw0yMjAzMTQxNjAyNTZaFw0yNTA0
# MjQwOTA3MjBaMIIBEzEdMBsGA1UEDwwUUHJpdmF0ZSBPcmdhbml6YXRpb24xEzAR
# BgNVBAUTCkhSQiAxMDA1MjExEzARBgsrBgEEAYI3PAIBAxMCREUxHjAcBgsrBgEE
# AYI3PAIBAhMNTmllZGVyc2FjaHNlbjEdMBsGCysGAQQBgjc8AgEBEwxCcmF1bnNj
# aHdlaWcxCzAJBgNVBAYTAkRFMRYwFAYDVQQIEw1OaWVkZXJzYWNoc2VuMRIwEAYD
# VQQHEwlXb2xmc2J1cmcxHDAaBgNVBAkTE01heC12b24tTGF1ZS1XZWcgMTkxGDAW
# BgNVBAoTD1RFLVNZU1RFTVMgR21iSDEYMBYGA1UEAxMPVEUtU1lTVEVNUyBHbWJI
# MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAsNpWT+JEuh43teGS+r3W
# 5zFgXA4a42GqnmEOIkUp+UPqC+6k9++zXUFIZjlCVluKJ3Xv4NDyrDu26i6/P96J
# iLzBJOX9uZItduUBL7o3lANnjom7FEVpmJJGf+mbxvB04tdSvhf9IZsxpPZEaOth
# ZAQaWDTsg0LzZQeHAxaOueoVOG9CHQRsJAwFX3HhaVHSd8atmi7kDXQAkpwDn6+2
# W1OT/IQKUAEr2vxysctY5Bx4PCX/ZxRk1J/D5WyapcHOZyMMCtMXUTFse4nvLxMS
# j6ha94PCdCZC4bOijYguV3JAYuCsFH99s36cLUpLxX7pnH0I6KIEttXIyER+WAbI
# oNr2UDgd6RZJ+XsU4WE7w5YguOiOwiBHn3/guBorvFOFaVPHVu8N9ZqjfInvvAdS
# fqT5rBNryGWvUrB2SPd0y21DxC4DKph0IcwVPCJiP/5dtAS3aXZTePco8+W/PNNo
# 0uu4wwLfGgZwJSQFwTtA7vaKmhGPgT8mBfhtB2UjMG+8XT7gr3ov/+5hg+Ru/u2J
# FI7N1O/jr+1gs6JlK4YF8ETFxXC+nETwykd49OugjVyRQEhQjnHM48gvvFq/uL2K
# IVbf6gojJHW93KYiGPMQ2gcXcfnNcFu2Ro69ve58OQxNxYSKIi/sXCRKoNC65HUs
# 7i+riSNPNVBIzaqPgHU6iXECAwEAAaOCAbYwggGyMA4GA1UdDwEB/wQEAwIHgDCB
# nwYIKwYBBQUHAQEEgZIwgY8wTAYIKwYBBQUHMAKGQGh0dHA6Ly9zZWN1cmUuZ2xv
# YmFsc2lnbi5jb20vY2FjZXJ0L2dzZ2NjcjQ1ZXZjb2Rlc2lnbmNhMjAyMC5jcnQw
# PwYIKwYBBQUHMAGGM2h0dHA6Ly9vY3NwLmdsb2JhbHNpZ24uY29tL2dzZ2NjcjQ1
# ZXZjb2Rlc2lnbmNhMjAyMDBVBgNVHSAETjBMMEEGCSsGAQQBoDIBAjA0MDIGCCsG
# AQUFBwIBFiZodHRwczovL3d3dy5nbG9iYWxzaWduLmNvbS9yZXBvc2l0b3J5LzAH
# BgVngQwBAzAJBgNVHRMEAjAAMEcGA1UdHwRAMD4wPKA6oDiGNmh0dHA6Ly9jcmwu
# Z2xvYmFsc2lnbi5jb20vZ3NnY2NyNDVldmNvZGVzaWduY2EyMDIwLmNybDATBgNV
# HSUEDDAKBggrBgEFBQcDAzAfBgNVHSMEGDAWgBQlndD8WQmGY8Xs87ETO1ccA5I2
# ETAdBgNVHQ4EFgQUtGt3xHpCCOM3zHPw8Rlox5B+tvQwDQYJKoZIhvcNAQELBQAD
# ggIBAHwLtY4rR+w31n+vChCCzr6LIw/iefwzwLQ33L0UOm9alIEnxcISt4Fna0WJ
# Nd3p8f8AUQ9rv0qbO8QNyS9kPd34rN/K8SqCj3gtEyBBTaGWTz7SAnynn3Wid1bw
# FMzlkebJ7m9ug1eOhnKJudeiadKLetz32Miaagr4pEM0r7zeKanW4voDLAHib/fA
# NEEBUq2j9f3WPnw2shLhG4PRVIqpnBtqf/nR3hhEldibZx4xuI+OEgjnC49Rz4cy
# ZuZLw1SO6R4vwx9cHtnZC5kZPE+awrj72ic0lQxbvK9HJTJBUOJuYRjVA86uxjPG
# wvaR8XdeX0Xw1ytdJd4X7iCvj2cqO92iV/yq+Y1mLapJi5POezf6LdDcLUKF4vpJ
# w0bFGz9A5ZwnXhNQ2HTFOIg7hjoq+dY82dfnh+fHVfFNU2OZZ6CxEXad+Gaw4grA
# LnLHbitzbue0FXPW3yoWWVsjLhHWZFhfTOpXgziZB0slTI1kXydXfhfcFoFladlc
# BA3dQnfieH+fuVFc2cYSfS1UJu5juDjB3UjRZYTAe+cKq/2IGlK/eiyGjxxKkPCc
# hrlp70FyDlSgAyl+NUYNCAGAgHZlS3x7usATC8rlfKffuXPHzCepG0/Lpvj0b6PM
# p5XuRnRpvlk0j5CbwZVguLIsvFVHzd9z/gUsvhqHSvcP08+iMYIGMTCCBi0CAQEw
# bDBcMQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFsU2lnbiBudi1zYTEyMDAG
# A1UEAxMpR2xvYmFsU2lnbiBHQ0MgUjQ1IEVWIENvZGVTaWduaW5nIENBIDIwMjAC
# DB+2/W3QMZH/P4rUtzAJBgUrDgMCGgUAoHgwGAYKKwYBBAGCNwIBDDEKMAigAoAA
# oQKAADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4w
# DAYKKwYBBAGCNwIBFTAjBgkqhkiG9w0BCQQxFgQUSG4mnhPutngSP9D23fnXSO9T
# FSgwDQYJKoZIhvcNAQEBBQAEggIAP4CeH6N4mKyAkVg6fJznqh3gydZTgAPkwrZ9
# rk7Rm+i+SziNHwAIp3nJwJqaNaBp2YWIRu4lapzZ8IPv85KzEEO2UUpkCmCS9yKG
# n0K74Z4nZcCthYCHDBF7tyMR19KBieiI2BlTOutjuqTZIApBXCVmH2hjC9+kTEEJ
# 5F5K0ZCFhxDy5Uf9uULSTCBAIwwHvjXlkOGNl0yP3SCI0dNdTRMP6oeEeBHZR/2w
# 6gyVwyqzpbgaDAg4dDUgs1LxhedemzFo91FmU/8VFv9wb48XIa5yH4eYk5FYNxH9
# a4WP7bDd4wcABqBsJmyCiyAiCrtesylPcFjrGQwbQhB8+b6WGahdjlV7u/p8CSJ/
# PG144q1rhX6VL3OLtnMCfU+MeBGA83JylI+guwRNLfz70si2YdjSzCp4IHcwLHPT
# E68iG6l9m67qDlG16J7wkAHBEMyZcrtCAUEL8yqNKbFhwnxiGHTNWRjX0omvvNHy
# 81/3GrdaaQB9hpsw8zXk0bYCCtfDaXVR1dZ8ruCmUJzHFKQzSCRTxLykqeSnQFJB
# Lt63iqdeLPibS/hNnWCPhqYkdC/hLFMA0XRwvf4ieFWGwhe8eRGDGGUM9v3/0N0p
# 48d2wSjVZYSn+/ywJQxth2MlrotR9XVDrvwKI9K+r5dOi2ryKiVxgzplNx+ZtJxJ
# cq2nAXmhggMgMIIDHAYJKoZIhvcNAQkGMYIDDTCCAwkCAQEwdzBjMQswCQYDVQQG
# EwJVUzEXMBUGA1UEChMORGlnaUNlcnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lDZXJ0
# IFRydXN0ZWQgRzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0YW1waW5nIENBAhAFRK/z
# lJ0IOaa/2z9f5WEWMA0GCWCGSAFlAwQCAQUAoGkwGAYJKoZIhvcNAQkDMQsGCSqG
# SIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMjQwOTEzMTAwMTA5WjAvBgkqhkiG9w0B
# CQQxIgQgCn7FQfIHslc5YJwoeRMTeMv/9tEHDhnhCI18pG16jQEwDQYJKoZIhvcN
# AQEBBQAEggIABkQNbDCNUc/TK7JKaGh6ZN0WLtnv2cuOQJoDkQnlJA3fBbHAptMf
# 3m2Im4gU6YmC4FC8t1pNrugWn+erQLJWW4BZvxNjgjeJSHUyB6dKb8ZBMtWilVgo
# hLH1jK1LH6s0J/rsVdv5Etmlr1tD1f3Puf/Doqr7Hps0LdPo8N7mT/1NnCVNsVfJ
# 6ZEFzpsr5VDbximsCH85hCpqlGUzjhlCObufokHphvZGPc44SQpxqOlZPMireS2M
# 0dPWPv4CLvmWdO3m9TxAUyQU8Qzc8fAMNCeyvCg26WPBOy5TR6Lk8UbuZ9nVbiNI
# QA7MaVmEY2IOWJDkCjC+H95KMm+pGvbfQfvw6Wn6TSpgMrlbU3H0R80tftZZkv6D
# DAW+ON5fbjAN9nTdh2gTiUGvOyypSYTTZBKGXxIrldojwfeCe+EViGGhF4k4YJxM
# k+6h5ksiPsMS2lTBLPVUA6o95x5XyC+8at5jCk9diWkRmUFMWoyYPTD7ePkGZ8CK
# zeQk/3iUXef0jKggQbvUzr678iJ0eBUpC7j19Y0HhSIsshVxAv+wfHq2lo4yXuWR
# OCpFh7hC/DbJTavOpkUV0fpuPSF51k0cYsXW4xIhq0DIq6j2whPuhHyexXgvynoT
# U1s7C4Pp3bAp/NVEcRHJ10ANO7mKSkNvIcRvfPepJIhB6vEesvIOBA4=
# SIG # End signature block
